<?php
use Symfony\Component\Config\Loader\LoaderInterface;
use Symfony\Component\HttpKernel\Bundle\BundleInterface;

// Silencing errors required to get no error if no timezone is defined in php.ini
// @codingStandardsIgnoreStart
date_default_timezone_set(@date_default_timezone_get());
// @codingStandardsIgnoreEnd

require_once __DIR__ . "/../version.inc.php";

/**
 * Kernel Class
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class AppKernel extends \Symfony\Component\HttpKernel\Kernel
{

    /**
     * Returns an array of bundles to registers.
     * @return BundleInterface[] An array of bundle instances.
     *
     * @api
     */
    public function registerBundles()
    {
        $bundles = array(
            new \Topalis\Configuration\TopalisConfigurationBundle(),
            new \Topalis\ActiveSyncFilter\TopalisAsfBundle(),

            new Symfony\Bundle\FrameworkBundle\FrameworkBundle(),
            new Symfony\Bundle\SecurityBundle\SecurityBundle(),
            new Symfony\Bundle\TwigBundle\TwigBundle(),
            new Doctrine\Bundle\DoctrineBundle\DoctrineBundle(),
            new Doctrine\Bundle\MigrationsBundle\DoctrineMigrationsBundle(),

            new Sensio\Bundle\FrameworkExtraBundle\SensioFrameworkExtraBundle(),
            new Symfony\Bundle\AsseticBundle\AsseticBundle(),
            new Symfony\Bundle\MonologBundle\MonologBundle(),
            new FOS\JsRoutingBundle\FOSJsRoutingBundle(),
            new Maxmind\Bundle\GeoipBundle\MaxmindGeoipBundle(),
            new \Genemu\Bundle\FormBundle\GenemuFormBundle(),
            new Oneup\UploaderBundle\OneupUploaderBundle(),

            new \Topalis\System\TopalisSystemBundle(),
            new \Topalis\Kernel\TopalisKernelBundle(),
            new \Topalis\Form\TopalisFormBundle(),
            new \Topalis\Mail\TopalisMailBundle(),
            new \Topalis\Apache\TopalisApacheBundle(),
            new \Topalis\License\TopalisLicenseBundle(),
            new \Topalis\Update\TopalisUpdateBundle(),

            new \Topalis\Admin\TopalisAdminBundle(),
            new \Topalis\Dashboard\TopalisDashboardBundle(),
            new \Topalis\Device\TopalisDeviceBundle(),
            new \Topalis\Scep\TopalisScepBundle(),
            new \Topalis\Gcm\TopalisGcmBundle(),
            new \Topalis\MDM\TopalisMdmBundle(),
            new \Topalis\User\TopalisUserBundle(),
            new \Topalis\Profile\TopalisProfileBundle(),
            new \Topalis\Certificate\TopalisCertificateBundle(),
            new \Topalis\ReverseProxy\TopalisReverseProxyBundle(),
            new \Topalis\Wifi\TopalisWifiBundle(),
            new \Topalis\Maintenance\TopalisMaintenanceBundle(),
            new \Topalis\Log\TopalisLogBundle(),
            new \Topalis\ConfigGroup\TopalisConfigGroupBundle(),

            new \Topalis\Rollout\TopalisRolloutBundle(),
            new \Topalis\WinPhone\TopalisWinPhoneBundle(),

            new \Topalis\Aggregate\TopalisAggregateBundle(),
            new \Topalis\SystemMonitor\TopalisSystemMonitorBundle(),
            new \Topalis\Apns\TopalisApnsBundle(),
            new \Topalis\App\TopalisAppBundle(),
            new \Topalis\Util\TopalisUtilBundle(),
            new \Topalis\WebClip\TopalisWebClipBundle(),
        );

        if (in_array($this->getEnvironment(), array('dev', 'test'))) {
            $bundles[] = new Symfony\Bundle\WebProfilerBundle\WebProfilerBundle();
            $bundles[] = new Sensio\Bundle\DistributionBundle\SensioDistributionBundle();
            $bundles[] = new RaulFraile\Bundle\LadybugBundle\RaulFraileLadybugBundle();
            $bundles[] = new Topalis\Generator\TopalisGeneratorBundle();
        }

        return $bundles;
    }

    /**
     * Loads the container configuration
     *
     * @param LoaderInterface $loader A LoaderInterface instance
     *
     * @api
     */
    public function registerContainerConfiguration(LoaderInterface $loader)
    {
        $loader->load(__DIR__."/config/config_".$this->getEnvironment().".yml");
    }

    protected function getContainerBaseClass()
    {
        if ("test" == $this->environment) {
            return "\PSS\SymfonyMockerContainer\DependencyInjection\MockerContainer";
        }

        return parent::getContainerBaseClass();
    }
}
