<?php

namespace Application\Migrations;

use Doctrine\DBAL\Migrations\AbstractMigration;
use Doctrine\DBAL\Schema\Schema;
use Topalis\Database\Migrations\AbstractAuralisMigration;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
class Version020305 extends AbstractAuralisMigration
{
    /**
     * @param Schema $schema
     */
    public function up(Schema $schema)
    {
        $this->abortIf($this->connection->getDatabasePlatform()->getName() != 'mysql', 'Migration can only be executed safely on \'mysql\'.');


        // we have to check if an index exists before because it's name depends
        // on the auralis version that was installed initially.
        if ($this->existsIndex('admins', 'idx_admins_login')) {
            $this->addSql('DROP INDEX idx_admins_login ON admins');
            $this->addSql('CREATE UNIQUE INDEX UNIQ_A2E0150FAA08CB10 ON admins (login)');
        }

        if ($this->existsIndex('clientcerts', 'fk_clientcerts_device_id')) {
            $this->addSql('ALTER TABLE clientcerts DROP FOREIGN KEY fk_clientcerts_device_id');
            $this->addSql('DROP INDEX fk_clientcerts_device_id ON clientcerts');
            $this->addSql('ALTER TABLE clientcerts ADD CONSTRAINT fk_clientcerts_device_id FOREIGN KEY (device_id) REFERENCES devices (id)');
        }

        if ($this->existsIndex('clientcerts', 'idx_clientcerts_challenge')) {
            $this->addSql('DROP INDEX idx_clientcerts_challenge ON clientcerts');
            $this->addSql('CREATE INDEX IDX_4DDB6E8294A4C7D4 ON clientcerts (device_id)');
        }

        if (!$this->existsIndex('clientcerts', 'UNIQ_4DDB6E82D7098951')) {
            $this->addSql('CREATE UNIQUE INDEX UNIQ_4DDB6E82D7098951 ON clientcerts (challenge)');
        }

        if ($this->existsIndex('accesslog', 'cert_id')) {
            $this->addSql('ALTER TABLE accesslog DROP FOREIGN KEY fk_accesslog_cert_id');
            $this->addSql('DROP INDEX cert_id ON accesslog');
            $this->addSql('CREATE INDEX IDX_A694CA3A81448FA9 ON accesslog (cert_id)');
            $this->addSql('ALTER TABLE accesslog ADD CONSTRAINT fk_accesslog_cert_id FOREIGN KEY (cert_id) REFERENCES clientcerts (id)');
        }

        if ($this->existsIndex('accesslog', 'fk_accesslog_device_id')) {
            $this->addSql('ALTER TABLE accesslog DROP FOREIGN KEY fk_accesslog_device_id');
            $this->addSql('DROP INDEX fk_accesslog_device_id ON accesslog');
            $this->addSql('CREATE INDEX IDX_A694CA3A94A4C7D4 ON accesslog (device_id)');
            $this->addSql('ALTER TABLE accesslog ADD CONSTRAINT fk_accesslog_device_id FOREIGN KEY (device_id) REFERENCES devices (id)');
        }

        if ($this->existsIndex('mailaccounts', 'fk_mailacounts_cert_id')) {
            $this->addSql('ALTER TABLE mailaccounts DROP FOREIGN KEY fk_mailacounts_cert_id');
            $this->addSql('DROP INDEX fk_mailacounts_cert_id ON mailaccounts');
            $this->addSql('CREATE INDEX IDX_643DFEBE81448FA9 ON mailaccounts (cert_id)');
            $this->addSql('ALTER TABLE mailaccounts ADD CONSTRAINT fk_mailacounts_cert_id FOREIGN KEY (cert_id) REFERENCES clientcerts (id)');
        }

        if ($this->existsIndex('access_cert_stat', 'fk_1aa4e41994a4c7d4')) {
            $this->addSql('ALTER TABLE access_cert_stat DROP FOREIGN KEY FK_1AA4E41994A4C7D4');
            $this->addSql('DROP INDEX fk_1aa4e41994a4c7d4 ON access_cert_stat');
            $this->addSql('CREATE INDEX IDX_1AA4E41994A4C7D4 ON access_cert_stat (device_id)');
            $this->addSql('ALTER TABLE access_cert_stat ADD CONSTRAINT FK_1AA4E41994A4C7D4 FOREIGN KEY (device_id) REFERENCES devices (id)');
        }

        if ($this->existsIndex('devices', 'fk_11074e9a81448fa9')) {
            $this->addSql('ALTER TABLE devices DROP FOREIGN KEY FK_11074E9A81448FA9');
            $this->addSql('DROP INDEX fk_11074e9a81448fa9 ON devices');
            $this->addSql('CREATE INDEX IDX_11074E9A81448FA9 ON devices (cert_id)');
            $this->addSql('ALTER TABLE devices ADD CONSTRAINT FK_11074E9A81448FA9 FOREIGN KEY (cert_id) REFERENCES clientcerts (id)');
        }

        if ($this->existsIndex('devices', 'fk_11074e9a15c91997')) {
            $this->addSql('ALTER TABLE devices DROP FOREIGN KEY FK_11074E9A15C91997');
            $this->addSql('DROP INDEX fk_11074e9a15c91997 ON devices');
            $this->addSql('CREATE INDEX IDX_11074E9A15C91997 ON devices (mdm_cert_id)');
            $this->addSql('ALTER TABLE devices ADD CONSTRAINT FK_11074E9A15C91997 FOREIGN KEY (mdm_cert_id) REFERENCES clientcerts (id)');
        }

        if ($this->existsIndex('mdmqueue', 'fk_mdmqueue_device')) {
            $this->addSql('ALTER TABLE mdmqueue DROP FOREIGN KEY fk_mdmqueue_device');
            $this->addSql('DROP INDEX fk_mdmqueue_device ON mdmqueue');
            $this->addSql('CREATE INDEX IDX_5C80960F94A4C7D4 ON mdmqueue (device_id)');
            $this->addSql('ALTER TABLE mdmqueue ADD CONSTRAINT fk_mdmqueue_device FOREIGN KEY (device_id) REFERENCES devices (id) ON DELETE CASCADE');
        }

        if ($this->existsIndex('mdmcommands', 'fk_mdmcommands_admin')) {
            $this->addSql('ALTER TABLE mdmcommands DROP FOREIGN KEY fk_mdmcommands_admin');
            $this->addSql('DROP INDEX fk_mdmcommands_admin ON mdmcommands');
            $this->addSql('CREATE INDEX IDX_7AFB646F642B8210 ON mdmcommands (admin_id)');
            $this->addSql('ALTER TABLE mdmcommands ADD CONSTRAINT fk_mdmcommands_admin FOREIGN KEY (admin_id) REFERENCES admins (id) ON DELETE SET NULL');
        }

        if ($this->existsIndex('mdmcommands', 'fk_mdmcommands_device')) {
            $this->addSql('ALTER TABLE mdmcommands DROP FOREIGN KEY fk_mdmcommands_device');
            $this->addSql('DROP INDEX fk_mdmcommands_device ON mdmcommands');
            $this->addSql('CREATE INDEX IDX_7AFB646F94A4C7D4 ON mdmcommands (device_id)');
            $this->addSql('ALTER TABLE mdmcommands ADD CONSTRAINT fk_mdmcommands_device FOREIGN KEY (device_id) REFERENCES devices (id) ON DELETE CASCADE');
        }

        if ($this->existsIndex('reverseproxy_urls', 'idx_reverse_proxy_url_external_path_hash')) {
            $this->addSql('DROP INDEX idx_reverse_proxy_url_external_path_hash ON reverseproxy_urls');
            $this->addSql('CREATE UNIQUE INDEX UNIQ_DDB61E0D988F44FB ON reverseproxy_urls (external_path_hash)');
        }

        if ($this->existsIndex('configgroups_reverseproxyurls', 'idx_75ffcb31d460fb91')) {
            $this->addSql('ALTER TABLE configgroups_reverseproxyurls DROP FOREIGN KEY FK_40A18A41D460FB91');
            $this->addSql('DROP INDEX idx_75ffcb31d460fb91 ON configgroups_reverseproxyurls');
            $this->addSql('CREATE INDEX IDX_40A18A41D460FB91 ON configgroups_reverseproxyurls (reverseproxyurl_id)');
            $this->addSql('ALTER TABLE configgroups_reverseproxyurls ADD CONSTRAINT FK_40A18A41D460FB91 FOREIGN KEY (reverseproxyurl_id) REFERENCES reverseproxy_urls (id)');
        }

        if ($this->existsIndex('configgroups_reverseproxyurls', 'idx_75ffcb31ff00faca')) {
            $this->addSql('ALTER TABLE configgroups_reverseproxyurls DROP FOREIGN KEY FK_40A18A41FF00FACA');
            $this->addSql('DROP INDEX idx_75ffcb31ff00faca ON configgroups_reverseproxyurls');
            $this->addSql('CREATE INDEX IDX_40A18A41FF00FACA ON configgroups_reverseproxyurls (configgroup_id)');
            $this->addSql('ALTER TABLE configgroups_reverseproxyurls ADD CONSTRAINT FK_40A18A41FF00FACA FOREIGN KEY (configgroup_id) REFERENCES configgroups (id)');
        }

        if ($this->existsIndex('users', 'idx_users_login')) {
            $this->addSql('DROP INDEX idx_users_login ON users');
            $this->addSql('CREATE UNIQUE INDEX UNIQ_1483A5E9AA08CB10 ON users (login)');
        }
    }

    /**
     * @param Schema $schema
     */
    public function down(Schema $schema)
    {
        // this down() migration is auto-generated, please modify it to your needs
        $this->abortIf($this->connection->getDatabasePlatform()->getName() != 'mysql', 'Migration can only be executed safely on \'mysql\'.');

        $this->addSql('ALTER TABLE access_cert_stat DROP FOREIGN KEY FK_1AA4E41994A4C7D4');
        $this->addSql('DROP INDEX idx_1aa4e41994a4c7d4 ON access_cert_stat');
        $this->addSql('CREATE INDEX FK_1AA4E41994A4C7D4 ON access_cert_stat (device_id)');
        $this->addSql('ALTER TABLE access_cert_stat ADD CONSTRAINT FK_1AA4E41994A4C7D4 FOREIGN KEY (device_id) REFERENCES devices (id)');
        $this->addSql('ALTER TABLE accesslog DROP FOREIGN KEY FK_A694CA3A81448FA9');
        $this->addSql('ALTER TABLE accesslog DROP FOREIGN KEY FK_A694CA3A94A4C7D4');
        $this->addSql('DROP INDEX idx_a694ca3a81448fa9 ON accesslog');
        $this->addSql('CREATE INDEX cert_id ON accesslog (cert_id)');
        $this->addSql('DROP INDEX idx_a694ca3a94a4c7d4 ON accesslog');
        $this->addSql('CREATE INDEX fk_accesslog_device_id ON accesslog (device_id)');
        $this->addSql('ALTER TABLE accesslog ADD CONSTRAINT FK_A694CA3A81448FA9 FOREIGN KEY (cert_id) REFERENCES clientcerts (id)');
        $this->addSql('ALTER TABLE accesslog ADD CONSTRAINT FK_A694CA3A94A4C7D4 FOREIGN KEY (device_id) REFERENCES devices (id)');
        $this->addSql('DROP INDEX uniq_a2e0150faa08cb10 ON admins');
        $this->addSql('CREATE UNIQUE INDEX idx_admins_login ON admins (login)');
        $this->addSql('ALTER TABLE clientcerts DROP FOREIGN KEY FK_4DDB6E8294A4C7D4');
        $this->addSql('DROP INDEX uniq_4ddb6e82d7098951 ON clientcerts');
        $this->addSql('CREATE UNIQUE INDEX idx_clientcerts_challenge ON clientcerts (challenge)');
        $this->addSql('DROP INDEX idx_4ddb6e8294a4c7d4 ON clientcerts');
        $this->addSql('CREATE INDEX fk_clientcerts_device_id ON clientcerts (device_id)');
        $this->addSql('ALTER TABLE clientcerts ADD CONSTRAINT FK_4DDB6E8294A4C7D4 FOREIGN KEY (device_id) REFERENCES devices (id)');
        $this->addSql('ALTER TABLE configgroups_reverseproxyurls DROP FOREIGN KEY FK_40A18A41D460FB91');
        $this->addSql('ALTER TABLE configgroups_reverseproxyurls DROP FOREIGN KEY FK_40A18A41FF00FACA');
        $this->addSql('DROP INDEX idx_40a18a41ff00faca ON configgroups_reverseproxyurls');
        $this->addSql('CREATE INDEX IDX_75FFCB31FF00FACA ON configgroups_reverseproxyurls (configgroup_id)');
        $this->addSql('DROP INDEX idx_40a18a41d460fb91 ON configgroups_reverseproxyurls');
        $this->addSql('CREATE INDEX IDX_75FFCB31D460FB91 ON configgroups_reverseproxyurls (reverseproxyurl_id)');
        $this->addSql('ALTER TABLE configgroups_reverseproxyurls ADD CONSTRAINT FK_40A18A41D460FB91 FOREIGN KEY (reverseproxyurl_id) REFERENCES reverseproxy_urls (id)');
        $this->addSql('ALTER TABLE configgroups_reverseproxyurls ADD CONSTRAINT FK_40A18A41FF00FACA FOREIGN KEY (configgroup_id) REFERENCES configgroups (id)');
        $this->addSql('ALTER TABLE devices DROP FOREIGN KEY FK_11074E9A81448FA9');
        $this->addSql('ALTER TABLE devices DROP FOREIGN KEY FK_11074E9A15C91997');
        $this->addSql('DROP INDEX idx_11074e9a81448fa9 ON devices');
        $this->addSql('CREATE INDEX FK_11074E9A81448FA9 ON devices (cert_id)');
        $this->addSql('DROP INDEX idx_11074e9a15c91997 ON devices');
        $this->addSql('CREATE INDEX FK_11074E9A15C91997 ON devices (mdm_cert_id)');
        $this->addSql('ALTER TABLE devices ADD CONSTRAINT FK_11074E9A81448FA9 FOREIGN KEY (cert_id) REFERENCES clientcerts (id)');
        $this->addSql('ALTER TABLE devices ADD CONSTRAINT FK_11074E9A15C91997 FOREIGN KEY (mdm_cert_id) REFERENCES clientcerts (id)');
        $this->addSql('ALTER TABLE mailaccounts DROP FOREIGN KEY FK_643DFEBE81448FA9');
        $this->addSql('DROP INDEX idx_643dfebe81448fa9 ON mailaccounts');
        $this->addSql('CREATE INDEX fk_mailacounts_cert_id ON mailaccounts (cert_id)');
        $this->addSql('ALTER TABLE mailaccounts ADD CONSTRAINT FK_643DFEBE81448FA9 FOREIGN KEY (cert_id) REFERENCES clientcerts (id)');
        $this->addSql('ALTER TABLE mdmcommands DROP FOREIGN KEY FK_7AFB646F642B8210');
        $this->addSql('ALTER TABLE mdmcommands DROP FOREIGN KEY FK_7AFB646F94A4C7D4');
        $this->addSql('DROP INDEX idx_7afb646f642b8210 ON mdmcommands');
        $this->addSql('CREATE INDEX fk_mdmcommands_admin ON mdmcommands (admin_id)');
        $this->addSql('DROP INDEX idx_7afb646f94a4c7d4 ON mdmcommands');
        $this->addSql('CREATE INDEX fk_mdmcommands_device ON mdmcommands (device_id)');
        $this->addSql('ALTER TABLE mdmcommands ADD CONSTRAINT FK_7AFB646F642B8210 FOREIGN KEY (admin_id) REFERENCES admins (id) ON DELETE SET NULL');
        $this->addSql('ALTER TABLE mdmcommands ADD CONSTRAINT FK_7AFB646F94A4C7D4 FOREIGN KEY (device_id) REFERENCES devices (id) ON DELETE CASCADE');
        $this->addSql('ALTER TABLE mdmqueue DROP FOREIGN KEY FK_5C80960F94A4C7D4');
        $this->addSql('DROP INDEX idx_5c80960f94a4c7d4 ON mdmqueue');
        $this->addSql('CREATE INDEX fk_mdmqueue_device ON mdmqueue (device_id)');
        $this->addSql('ALTER TABLE mdmqueue ADD CONSTRAINT FK_5C80960F94A4C7D4 FOREIGN KEY (device_id) REFERENCES devices (id) ON DELETE CASCADE');
        $this->addSql('DROP INDEX uniq_ddb61e0d988f44fb ON reverseproxy_urls');
        $this->addSql('CREATE UNIQUE INDEX idx_reverse_proxy_url_external_path_hash ON reverseproxy_urls (external_path_hash)');
        $this->addSql('DROP INDEX uniq_1483a5e9aa08cb10 ON users');
        $this->addSql('CREATE UNIQUE INDEX idx_users_login ON users (login)');
    }
}
