<?php

namespace Topalis\ActiveSyncFilter\Entity;

use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Table(name="asf_stat_mimetype")
 * @ORM\Entity
 */
class AsfStatMimeType
{
    /**
     * @var \DateTime
     *
     * @ORM\Column(name="`date`", type="string")
     * @ORM\Id
     */
    protected $date;

    /**
     * @var string
     *
     * @ORM\Column(name="mediatype", type="string", length=255)
     * @ORM\Id
     */
    protected $mediaType;

    /**
     * @var string
     *
     * @ORM\Column(name="subtype", type="string", length=255)
     * @ORM\Id
     */
    protected $subType;

    /**
     * @var integer
     *
     * @ORM\Column(name="total", type="integer", options={"unsigned"=true})
     */
    protected $total;

    /**
     * @var integer
     *
     * @ORM\Column(name="changed", type="integer", options={"unsigned"=true})
     */
    protected $changed;

    /**
     * @var integer
     *
     * @ORM\Column(name="unchanged", type="integer", options={"unsigned"=true})
     */
    protected $unchanged;

    /**
     * @var integer
     *
     * @ORM\Column(name="removed", type="integer", options={"unsigned"=true})
     */
    protected $removed;

    /**
     * @var integer
     *
     * @ORM\Column(name="failed", type="integer", options={"unsigned"=true})
     */
    protected $failed;

    /**
     * @var integer
     *
     * @ORM\Column(name="insize", type="bigint", options={"unsigned"=true})
     */
    protected $inSize;

    /**
     * @var integer
     *
     * @ORM\Column(name="outsize", type="bigint", options={"unsigned"=true})
     */
    protected $outSize;

    /**
     * @var integer
     *
     * @ORM\Column(name="diff", type="bigint")
     */
    protected $diff;

    /**
     * @var integer
     *
     * @ORM\Column(name="lastchange", type="bigint")
     */
    protected $lastChange;

    /**
     * Set mediaType
     *
     * @param string $mediaType
     * @return AsfStatMimeType
     */
    public function setMediaType($mediaType)
    {
        $this->mediaType = $mediaType;

        return $this;
    }

    /**
     * Get mediaType
     *
     * @return string
     */
    public function getMediaType()
    {
        return $this->mediaType;
    }

    /**
     * Set subType
     *
     * @param string $subType
     * @return AsfStatMimeType
     */
    public function setSubType($subType)
    {
        $this->subType = $subType;

        return $this;
    }

    /**
     * Get subType
     *
     * @return string
     */
    public function getSubType()
    {
        return $this->subType;
    }

    /**
     * @return string <media type>/<sub type>
     */
    public function getMimeType()
    {
        return $this->mediaType . "/" . $this->subType;
    }

    /**
     * Set total
     *
     * @param integer $total
     * @return AsfStatMimeType
     */
    public function setTotal($total)
    {
        $this->total = $total;

        return $this;
    }

    /**
     * Get total
     *
     * @return integer
     */
    public function getTotal()
    {
        return $this->total;
    }

    /**
     * Set changed
     *
     * @param integer $changed
     * @return AsfStatMimeType
     */
    public function setChanged($changed)
    {
        $this->changed = $changed;

        return $this;
    }

    /**
     * Get changed
     *
     * @return integer
     */
    public function getChanged()
    {
        return $this->changed;
    }

    /**
     * Set unchanged
     *
     * @param integer $unchanged
     * @return AsfStatMimeType
     */
    public function setUnchanged($unchanged)
    {
        $this->unchanged = $unchanged;

        return $this;
    }

    /**
     * Get unchanged
     *
     * @return integer
     */
    public function getUnchanged()
    {
        return $this->unchanged;
    }

    /**
     * Set removed
     *
     * @param integer $removed
     * @return AsfStatMimeType
     */
    public function setRemoved($removed)
    {
        $this->removed = $removed;

        return $this;
    }

    /**
     * Get removed
     *
     * @return integer
     */
    public function getRemoved()
    {
        return $this->removed;
    }

    /**
     * Set failed
     *
     * @param integer $failed
     * @return AsfStatMimeType
     */
    public function setFailed($failed)
    {
        $this->failed = $failed;

        return $this;
    }

    /**
     * Get failed
     *
     * @return integer
     */
    public function getFailed()
    {
        return $this->failed;
    }

    /**
     * Set lastChange
     *
     * @param integer $lastChange
     * @return AsfStatMimeType
     */
    public function setLastChange($lastChange)
    {
        $this->lastChange = $lastChange;

        return $this;
    }

    /**
     * Get lastChange
     *
     * @return integer
     */
    public function getLastChange()
    {
        return $this->lastChange;
    }

    /**
     * Set inSize
     *
     * @param integer $inSize
     * @return AsfStatMimeType
     */
    public function setInSize($inSize)
    {
        $this->inSize = $inSize;

        return $this;
    }

    /**
     * Get inSize
     *
     * @return integer
     */
    public function getInSize()
    {
        return $this->inSize;
    }

    /**
     * Set outSize
     *
     * @param integer $outSize
     * @return AsfStatMimeType
     */
    public function setOutSize($outSize)
    {
        $this->outSize = $outSize;

        return $this;
    }

    /**
     * Get outSize
     *
     * @return integer
     */
    public function getOutSize()
    {
        return $this->outSize;
    }

    /**
     * Set diff
     *
     * @param integer $diff
     * @return AsfStatMimeType
     */
    public function setDiff($diff)
    {
        $this->diff = $diff;

        return $this;
    }

    /**
     * Get diff
     *
     * @return integer
     */
    public function getDiff()
    {
        return $this->diff;
    }

    /**
     * Set date
     *
     * @param \DateTime $date
     * @return AsfStatMimeType
     */
    public function setDate(\DateTime $date)
    {
        $this->date = $date;

        return $this;
    }

    /**
     * Get date
     *
     * @return \DateTime
     */
    public function getDate()
    {
        return $this->date;
    }
}
