<?php
namespace Topalis\ActiveSyncFilter\Form\Extension\EventListener;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\Form\Exception\UnexpectedTypeException;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\Form\FormFactoryInterface;
use Symfony\Component\Validator\Constraints as Assert;
use Topalis\ActiveSyncFilter\Config\Form\FormConfigEntry;

/**
 * Add Config Entries as needed
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class ConfigEntryListener implements EventSubscriberInterface
{
    protected $formFactory;

    public function __construct(FormFactoryInterface $formFactory)
    {
        $this->formFactory = $formFactory;
    }

    public static function getSubscribedEvents()
    {
        return array(
            FormEvents::PRE_SET_DATA => "setData"
        );
    }

    public function setData(FormEvent $event)
    {
        $form = $event->getForm();
        $data = $event->getData();

        if (!$data instanceof FormConfigEntry) {
            throw new UnexpectedTypeException($data, "Topalis\ActiveSyncFilter\Config\Form\FormConfigEntry");
        }

        $entry = $data->getEntry();

        $options = array(
            "property_path" => "value",
            "label" => $entry->getName(),
            "help" => $entry->getDesc(),
            "helpPlacement" => "left",
            "helpHtml" => true,
            "attr" => array("class" => "span6"),
            "required" => false,
            "auto_initialize" => false
        );

        $constraints = array();
        if ($entry->isInt()) {
            if (null !== $entry->getMax() || null !== $entry->getMin()) {
                $constraints[] = new Assert\Range(array(
                    "min" => $entry->getMin(),
                    "max" => $entry->getMax()
                ));
            }
        }

        if ($entry->isString()) {
            if (null !== $entry->getMaxLength() || null !== $entry->getMinLength()) {
                $constraints[] = new Assert\Length(array(
                    "min" => $entry->getMinLength(),
                    "max" => $entry->getMaxLength()
                ));
            }
        }
        $options['constraints'] = $constraints;

        if ($entry->isBool()) {
            $form->add($this->formFactory->createNamed("value", "checkbox", null, $options));
        } else if ($entry->isInt()) {
            $form->add($this->formFactory->createNamed("value", "number", null, $options));
        } else if ($entry->isSelect()) {
            $form->add($this->formFactory->createNamed("value", "choice", null,
                array_replace($options,
                    array("choices" => array_flip($entry->getOptions())),
                    array("empty_value" => false)
                )
            ));
        } else {
            $form->add($this->formFactory->createNamed("value", "text", null, $options));
        }

    }
}
