<?php
namespace Topalis\ActiveSyncFilter\Plugin;

use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\Util\StringUtil;
use Topalis\Configuration\DatabaseConfiguration;
use Topalis\ActiveSyncFilter\FilterServer;

/**
 * This class allows to access the plugins configuration. It defines if a plugin
 * is enabled or disabled.
 *
 * @author Dominic Tubach
 */
class PluginsConfig extends DatabaseConfiguration
{
    /**
     * Contructor.
     *
     * @param FilterServer $filterServer The filter server to access the
     *   plugins configuration for.
     * @param ConfigGroup $config The configuration to access.
     */
    public function __construct(FilterServer $filterServer, ConfigGroup $config)
    {
        $db = $filterServer->getDatabase();
        $table = $filterServer->getServerConfig()->getJdbcTablePrefix() . "conf_plugins";
        parent::__construct($db, $table, "key", "enabled");
        $this->addRequiredValue("configid", $config->getId());
    }

    /**
     *
     * @param string $key The key of the plugin.
     * @return boolean true if the plugin with the specified file name is
     *   enabled.
     */
    public function isEnabled($key)
    {
        $enabled = $this->getProperty($key, "true");
        return StringUtil::strtobool($enabled);
    }

    /**
     * @param string $key The key of the plugin to disable.
     * @return boolean true if something has been changed, false if the plugin
     *   already was disabled.
     */
    public function disable($key)
    {
        return $this->setProperty($key, "false");
    }

    /**
     * @param string $key The key of the plugin to enable.
     * @return boolean true if something has been changed, false if the plugin
     *   already was enabled.
     */
    public function enable($key)
    {
        return $this->setProperty($key, "true");
    }

    /**
     * Enables or disables the plugin with the specified key.
     *
     * @param string $key The key of the plugin .
     * @param string $enabled true, to enable the plugin, false to disable the
     *   plugin.
     * @return boolean true if the plugin status has been changed, false if the
     *   plugin already had the specified enabled state.
     */
    public function setEnabled($key, $enabled)
    {
        if ($enabled) {
            return $this->enable($key);
        }

        return $this->disable($key);
    }
}
