<?php
namespace Topalis\Admin\Form;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Validator\Constraints as Assert;
use Topalis\Admin\Entity\Admin;

/**
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class AdminForm extends AbstractType
{
    /**
     * @var boolean
     */
    private $new;

    public function __construct($new = false)
    {
        $this->new = $new;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $options; // unused
        $builder
            ->add("login", "text", array(
                "label" => "admin.users.login",
                "constraints" => array(
                    new Assert\NotBlank(),
                    new Assert\NotEqualTo(array("value" => "auradmin")),
                )
            ))
            ->add("realname", "text", array(
                "label" => "admin.users.real"
            ))
            ->add("level", "choice", array(
                "label" => "admin.admins.level",
                "choices" => array(
                    Admin::LEVEL_ADMIN => "admin.admins.levels.admin",
                    Admin::LEVEL_SUPPORTER => "admin.admins.levels.supporter",
                    Admin::LEVEL_MONITOR => "admin.admins.levels.monitor",
                ),
                "empty_data" => 0
            ))
            ->add("plainPassword", "repeated", array(
                "type" => "password",
                "invalid_message" => "password.doNotMatch",
                "first_options" => array("label" => "Password", "had_value" => !$this->new),
                "second_options" => array("label" => "repeatPassword", "had_value" => !$this->new),
            ))
        ;
    }

    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver
            ->setDefaults(array(
                "data_class" => "Topalis\Admin\Entity\Admin"
            ))
        ;
    }

    /**
     * Returns the name of this type.
     * @return string The name of this type
     */
    public function getName()
    {
        return "admin";
    }
}
