<?php
namespace Topalis\Aggregate\Command;

use Symfony\Component\Console\Command\Command;

/**
 * This class aggregates device access log to daily access statistics.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class AggregateAccessStatDailyCommand extends Command
{
    /**
     * @var \Topalis\Database\MySqlDb
     */
    private $db;

    /**
     * @return \Topalis\Database\MySqlDb
     */
    public function getDB()
    {
        return $this->db;
    }

    /**
     * @param \Topalis\Database\MySqlDb $db
     */
    public function setDB(\Topalis\Database\MySqlDb $db)
    {
        $this->db = $db;
    }

    protected function configure()
    {
        $this
        ->setName("aggregate:access_stat_daily")
        ->setDescription("Aggregates device access log to daily access statistics.")
        ->setHelp("Aggregates device access log to daily access statistics.");
    }

    protected function execute(\Symfony\Component\Console\Input\InputInterface $input,
        \Symfony\Component\Console\Output\OutputInterface $output)
    {
        $input; // unused

        try {
            $this->updateAccessStatDaily();
        } catch (\Exception $e) {
            if ($output instanceof \Symfony\Component\Console\Output\ConsoleOutput) {
                $output = $output->getErrorOutput();
            }
            $output->writeln("Error updating table access_stat_daily:");
            $output->writeln((string) $e);
            return -1;
        }
    }

    /**
     * Updates contents in table access_stat_daily.
     */
    protected function updateAccessStatDaily()
    {
        $db = $this->getDB();
        try {
            $period1Day = new \DateInterval("P1D");

            $db->beginTransaction();
            $lastDate = $db->fetchValue("access_stat_daily", "MAX(`date`)");
            if ($lastDate !== null) {
                $date = new \DateTime($lastDate);
                $date->add($period1Day);
            } else {
                $lastDate = $db->fetchValue("accesslog", "DATE(MIN(tstamp))");
                if ($lastDate === null) {
                    // no traffic, yet, so nothing to do
                    return;
                }
                $date = new \DateTime($lastDate);
            }
            $starttime = $date->format("U");

            $sql = "INSERT INTO access_stat_daily (`date`, traffic, access_count, distinct_ips, distinct_devices)
                SELECT DATE(FROM_UNIXTIME(:starttime)), IFNULL(SUM(traffic), 0), IFNULL(COUNT(id), 0),
                IFNULL(COUNT(DISTINCT ip), 0), IFNULL(COUNT(DISTINCT device_id), 0) FROM accesslog
                WHERE tstamp >= FROM_UNIXTIME(:starttime) AND tstamp < FROM_UNIXTIME(:endtime)";

            $curtime = strtotime(date("Y-m-d"));
            while ($starttime < $curtime) {
                $date->add($period1Day);
                $endtime = $date->format("U");
                $db->exec($sql, array("starttime" => $starttime, "endtime" => $endtime));
                $starttime = $endtime;
            }
            $db->commitTransaction();
        } catch (Exception $e) {
            $db->rollbackTransaction();
            throw $e;
        }
    }

}
