<?php
namespace Topalis\Aggregate\Command;

use Symfony\Component\Console\Command\Command;

/**
 * This class aggregates device access log grouped by certificates
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class AggregateCertStatCommand extends Command
{
    /**
     * @var \Topalis\Database\MySqlDb
     */
    private $db;

    /**
     * @return \Topalis\Database\MySqlDb
     */
    public function getDB()
    {
        return $this->db;
    }

    /**
     * @param \Topalis\Database\MySqlDb $db
     */
    public function setDB(\Topalis\Database\MySqlDb $db)
    {
        $this->db = $db;
    }

    protected function configure()
    {
        $this
        ->setName("aggregate:cert_stat")
        ->setDescription("Aggregates device access log grouped by certificates.")
        ->setHelp("Aggregates device access log grouped by certificates.");
    }

    protected function execute(\Symfony\Component\Console\Input\InputInterface $input,
        \Symfony\Component\Console\Output\OutputInterface $output)
    {
        $input; // unused

        try {
            $this->updateAccessCertStat();
        } catch (\Exception $e) {
            if ($output instanceof \Symfony\Component\Console\Output\ConsoleOutput) {
                $output = $output->getErrorOutput();
            }
            $output->writeln("Error updating table access_cert_stat:");
            $output->writeln((string) $e);
            return -1;
        }
    }

    /**
     * Updates contents in table access_cert_stat.
     */
    protected function updateAccessCertStat()
    {
        $db = $this->getDB();
        try {
            $db->beginTransaction();
            // update existing rows
            $db->exec(
                "UPDATE access_cert_stat stat
                 JOIN (SELECT MAX(id) as maxid,
                              MAX(tstamp) as maxtstamp,
                              cert_id
                       FROM accesslog GROUP BY cert_id) a
                 ON a.cert_id = stat.cert_id AND maxtstamp > stat.last_tstamp
                 SET
                     stat.traffic = stat.traffic + (
                         SELECT SUM(traffic) FROM accesslog
                         WHERE cert_id = stat.cert_id AND tstamp > stat.last_tstamp GROUP BY cert_id),
                     stat.last_tstamp = a.maxtstamp,
                     stat.last_ip = (SELECT ip FROM accesslog WHERE id = a.maxid);"
            );

            // insert rows for certificates that didn't have an access before
            $db->exec(
                "INSERT INTO access_cert_stat (device_id, cert_id, last_ip, last_tstamp, traffic)
                 SELECT a.device_id,
                        a.cert_id,
                        (SELECT ip FROM accesslog WHERE id = max(a.id)),
                        MAX(a.tstamp),
                        SUM(a.traffic)
                 FROM accesslog a
                 WHERE (SELECT 1 FROM access_cert_stat WHERE cert_id = a.cert_id) IS NULL
                 GROUP BY a.cert_id;"
            );

            $db->commitTransaction();
        } catch (Exception $e) {
            $db->rollbackTransaction();
            throw $e;
        }
    }

}
