<?php
namespace Topalis\Apache\Command;

use Topalis\Apache\Apache;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * This command refreshes the apache config using the values set in the system
 * configuration. Returns 64 if the config is unchanged.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class ApacheRefreshConfigCommand extends ContainerAwareCommand
{
    const EX_UNCHANGED = 64;

    /**
     * @return \Topalis\Apache\Apache
     */
    private function getApache()
    {
        return $this->getContainer()->get("apache");
    }

    /**
     * @return \Topalis\Configuration\AuralisSysConfig
     */
    private function getSysConfig()
    {
        return $this->getContainer()->get("config.auralis.system");
    }

    protected function configure()
    {
        $this
        ->setName("apache:config:refresh")
        ->setDescription("Refresh Apache HTTPD config with values in the auralis configuration file.")
        ->setHelp("Refresh Apache HTTPD config with values in the auralis configuration file.");
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $input; // unused

        $container = $this->getContainer();
        $apache = $this->getApache();
        $sysConfig = $this->getSysConfig();
        $filterServcerConfig = $this->getContainer()->get("asf.server_config");

        $action = Apache::ACTION_RESTART;
        $changed = false;
        $output->writeln("Set WebRoot");
        $changed = $apache->setDefine("auralis_webroot", $sysConfig->getWebRoot(), $action) ?: $changed;
        $output->writeln("Set database configuration");
        $changed = $apache->setDefine("auralis_dbhost", $container->getParameter("database_host"), $action) ?: $changed;
        $changed = $apache->setDefine("auralis_dbport", $container->getParameter("database_port"), $action) ?: $changed;
        $changed = $apache->setDefine("auralis_dbname", $container->getParameter("database_name"), $action) ?: $changed;
        $changed = $apache->setDefine("auralis_dbuser", $container->getParameter("database_user"), $action) ?: $changed;
        $changed = $apache->setDefine("auralis_dbpass", $container->getParameter("database_password"), $action)
            ?: $changed;
        $output->writeln("Set ActiveSync Filter port");
        $changed = $apache->setDefine("activesyncfilter_port", $filterServcerConfig->getPort(), $action) ?: $changed;

        if ($changed) {
            $output->writeln("Configuration changed. Restart Apache HTTPD.");
            return 0;
        } else {
            $output->writeln("Configuration unchanged.");
            return self::EX_UNCHANGED;
        }
    }
}
