<?php
namespace Topalis\App\Controller;

use CFPropertyList\CFPropertyList;
use CFPropertyList\CFString;
use CFPropertyList\CFTypeDetector;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Topalis\App\Entity\IosApp;
use Topalis\Kernel\Exception\Http\NotFoundException;
use Topalis\Util\StringUtil;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class IosManifestController extends Controller
{
    /**
     * @Route("/app/manifest/ios/{app}/manifest.plist", name="app_ios_manifest")
     * @param Request $request
     * @param IosApp $app
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function manifestAction(Request $request, IosApp $app)
    {
        $request; // unused

        if (IosApp::TYPE_UPLOADED !== $app->getType()) {
            throw new NotFoundException();
        }
        $ipa = realpath($app->getIpaPath());
        $uploadDir = realpath($this->container->getParameter("kernel.root_dir") . "/../web/uploads");
        if (!StringUtil::startsWith($ipa, $uploadDir)) {
            throw new AccessDeniedHttpException();
        }
        $relativePath = str_replace($uploadDir, "", $ipa);
        $port = $this->getConfig()->getSecondaryPort();
        $url = "https://" . $this->getConfig()->getHostname() . ":" . $port . "/uploads" . $relativePath;

        $plist = new CFPropertyList();
        $manifest = array(
            "items" => array(
                array(
                    "assets"   => array(
                        array(
                            "kind" => "software-package",
                            "url"  => $url,
                        )
                    ),
                    "metadata" => array(
                        "bundle-identifier" => $app->getIdentifier(),
                        "bundle-version"    => new CFString($app->getVersion()),
                        "kind"              => "software",
                        "title"             => $app->getName()
                    )
                )
            )
        );
        $typeDetector = new CFTypeDetector();
        $plist->add($typeDetector->toCFType($manifest));

        $response = new Response($plist->toXML(true));
        $response->headers->set("Content-Type", "application/x-plist");

        return $response;
    }

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    private function getConfig()
    {
        return $this->get("config.auralis");
    }
}
