<?php
namespace Topalis\App\Controller\Section;

use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Topalis\App\Entity\App;
use Topalis\App\Entity\IosApp;
use Topalis\App\Entity\IosAppState;
use Topalis\ConfigGroup\Entity\ConfigGroup;

/**
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @Route("/admin/app/ios")
 */
class IosAppSectionController extends GenericAppSectionController
{
    /**
     * @Route("/managed", name="admin_app_ios_managed")
     * @Template("TopalisAppBundle:Section/GenericAppSection:managed.html.twig")
     *
     * @param ConfigGroup $group
     * @param boolean $embed
     * @return array
     */
    public function managedAction(ConfigGroup $group = null, $embed = false)
    {
        return $this->doManagedAction($group, $embed);
    }

    /**
     * @Route("/inventory", name="admin_app_ios_inventory")
     * @Template("TopalisAppBundle:Section/GenericAppSection:inventory.html.twig")
     *
     * @return array
     */
    public function inventoryAction()
    {
        return $this->doInventoryAction();
    }

    /**
     * @Route("/{storeId}/add", name="admin_app_ios_add")
     * @param int $storeId
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function addAction($storeId)
    {
        try {
            $app = $this->getAppStore()->lookupByITunesStoreId($storeId);
        } catch (\Exception $e) {
            $this->get("session")->getFlashBag()->add("error",
                $this->getTranslator()->trans("admin.app.error.appStoreConnection",
                    array("%errmsg%" => $e->getMessage())
                ));
        }
        if (null === $app) {
            return $this->createNotFoundException();
        }
        $app->setManaged(true);

        return $this->forward("TopalisAppBundle:Section/IosAppSection:edit", array("app" => $app));
    }

    /**
     * @Route("/{app}/edit", name="admin_app_ios_edit")
     * @Template("TopalisAppBundle:Section/GenericAppSection:edit.html.twig")
     *
     * @param Request $request
     * @param IosApp $app
     * @return array
     */
    public function editAction(Request $request, IosApp $app)
    {
        return $this->doEditAction($request, $app);
    }

    /**
     * @Route("/upload", name="admin_app_ios_upload")
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function uploadAction()
    {
        $app = new IosApp();
        $app
            ->setManaged(true)
            ->setType(IosApp::TYPE_UPLOADED)
            ->setForceInstall(true)
        ;

        return $this->forward("TopalisAppBundle:Section/IosAppSection:edit",
            array("app" => $app, "standalone" => true));
    }

    /**
     * @Route("/{app}/delete", name="admin_app_ios_delete")
     * @param IosApp $app
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function deleteAction(IosApp $app)
    {
        if (IosApp::TYPE_UPLOADED === $app->getType()) {
            $this->getAppEntityManager()->delete($app);
        } else {
            $app->setManaged(false);
            $app->setConfigGroups(array());
            $this->getAppEntityManager()->save($app);
        }

        return $this->redirect($this->generateUrl("admin_app_ios_managed"));
    }

    /**
     * @Route("/search", name="admin_app_ios_search")
     * @Template("TopalisAppBundle:Section/GenericAppSection:search.html.twig")
     *
     * @param Request $request
     * @return array
     */
    public function searchAction(Request $request)
    {
        return $this->doSearchAction($request);
    }

    /**
     * @Route("/state/{state}/retryinstall", name="admin_app_ios_state_retry_install")
     * @Template("TopalisAppBundle:Section/GenericAppSection:retryInstall.html.twig")
     *
     * @param IosAppState $state
     * @return array
     */
    public function retryInstallAction(IosAppState $state)
    {
        return $this->doRetryInstallAction($state);
    }

    /**
     * @Route("/state/{state}/remove", name="admin_app_ios_state_remove")
     * @Template("TopalisAppBundle:Section/GenericAppSection:remove.html.twig")
     *
     * @param AndroidAppState $state
     * @return array
     */
    public function removeAction(IosAppState $state)
    {
        return $this->doRemoveAction($state);
    }

    /**
     * @Route("/{app}/enable/{group}", name="admin_app_ios_enable")
     *
     * @param ConfigGroup $group
     * @param IosApp $app
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function enableAction(ConfigGroup $group, IosApp $app)
    {
        return $this->doEnableAction($group, $app);
    }

    /**
     * @Route("/{app}/disable/{group}", name="admin_app_ios_disable")
     *
     * @param ConfigGroup $group
     * @param IosApp $app
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function disableAction(ConfigGroup $group, IosApp $app)
    {
        return $this->doDisableAction($group, $app);
    }

    protected function getAppEntityManager()
    {
        return $this->get("app.entity.manager.ios_app");
    }

    protected function getAppStateEntityManager()
    {
        return $this->get("app.entity.manager.ios_app_state");
    }

    /**
     * @return \Topalis\App\Ios\IosAppStore
     */
    protected function getAppStore()
    {
        return $this->get("app.ios.store");
    }

    protected function getEditFormHandler()
    {
        return $this->get("app.form.handler.edit.ios");
    }

    protected function getOsName()
    {
        return "ios";
    }
}
