<?php
namespace Topalis\App\Entity;

use Topalis\Database\EntityManager;
use Topalis\Device\Entity\Device;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class AppStateEntityManager extends EntityManager
{
    /**
     * Returns all app states for the specified device that are not assigned to
     * the group the device belongs to.
     *
     * @param Device $device
     * @return AppState[]
     */
    public function findUnassignedForDevice(Device $device)
    {
        $qb = $this->createQueryBuilder("s")
            ->select("s")
            ->join("s.app", "a")
            ->leftJoin("a.configGroups", "g")
            ->where("s.device = :device")
            ->setParameter("device", $device);
        $states = $qb->getQuery()->getResult();
        foreach ($states as $key => $state) {
            if ($state->getApp()->getConfigGroups()->contains($device->getConfigGroup())) {
                unset($states[$key]);
            }
        }

        return $states;
    }

    /**
     * Find states for an app which are managed but not assigend to a device in the current config groups.
     * @param App $app
     *
     * @return AppState[]
     */
    public function findUnusedManaged(App $app)
    {
        return $this->createQueryBuilder("s")
            ->select("s")
            ->join("s.device", "d")
            ->where("s.app = :app AND s.managed = 1 AND d.configGroup NOT IN (:groups)")
            ->setParameter("app", $app)
            ->setParameter("groups", $app->getConfigGroups()->toArray())
            ->getQuery()
            ->getResult()
        ;
    }
}
