<?php
namespace Topalis\App\Entity;

use Doctrine\ORM\Mapping as ORM;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Doctrine\Common\Collections\ArrayCollection;
use Topalis\WinPhone\MS\MSAppPolicy;

/**
 * This entity represents an app policy for Windows Phones.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 *
 * @ORM\Entity
 * @UniqueEntity(fields={"name"})
 */
class WinPhoneAppPolicy
{
    const TYPE_BLACKLIST = 0;
    const TYPE_WHITELIST = 1;

    /**
     * @var integer
     * @ORM\Column(type="integer", options={"unsigned"=true})
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    protected $id;

    /**
     * @var string
     * @ORM\Column(type="string", unique=true)
     */
    protected $name;

    /**
     * @var integer
     * @ORM\Column(type="smallint")
     */
    protected $type;

    /**
     * @var string
     * @ORM\Column(type="text", length=16777215)
     */
    protected $policy;

    /**
     * @var \Doctrine\Common\Collections\ArrayCollection
     * @ORM\OneToMany(targetEntity="Topalis\ConfigGroup\Entity\ConfigGroup",
     *     mappedBy="winPhoneAppPolicy", cascade={"persist"})
     */
    protected $configGroups;

    public function __construct()
    {
        $this->configGroups = new ArrayCollection();
    }

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set name
     *
     * @param string $name
     * @return WinPhoneAppPolicy
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set type
     *
     * @param integer $type
     * @return WinPhoneAppPolicy
     */
    public function setType($type)
    {
        $this->type = $type;

        return $this;
    }

    /**
     * Get type
     *
     * @return integer
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @return string
     */
    public function getPolicy()
    {
        return $this->policy;
    }

    /**
     * @param string|MSAppPolicy $policy
     * @return WinPhoneAppPolicy self
     */
    public function setPolicy($policy)
    {
        if ($policy instanceof MSAppPolicy) {
            $this->policy = $policy->toXml();
            $this->type = $policy->isBlacklist() ? self::TYPE_BLACKLIST : self::TYPE_WHITELIST;
        } else {
            $this->policy = $policy;
        }

        return $this;
    }

    /**
     * Add configGroups
     *
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup
     * @return WifiConfig
     */
    public function addConfigGroup(\Topalis\ConfigGroup\Entity\ConfigGroup $configGroup)
    {
        if (!$this->configGroups->contains($configGroup)) {
            $this->configGroups[] = $configGroup;
            $configGroup->setWinPhoneAppPolicy($this);
        }

        return $this;
    }

    /**
     * Remove configGroup
     *
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup
     */
    public function removeConfigGroup(\Topalis\ConfigGroup\Entity\ConfigGroup $configGroup)
    {
        if ($this->configGroups->removeElement($configGroup)) {
            $configGroup->setWinPhoneAppPolicy(null);
        }
    }

    /**
     * Get configGroups
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getConfigGroups()
    {
        return $this->configGroups;
    }

    /**
     * @see \Serializable::serialize()
     */
    public function serialize()
    {
        return serialize(array(
            $this->id,
        ));
    }

    /**
     * @param string $serialized The value returned by serialize().
     * @see \Serializable::unserialize()
     */
    public function unserialize($serialized)
    {
        list ($this->id) = unserialize($serialized);
    }
}
