<?php
namespace Topalis\App\EventListener\MDM;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Topalis\MDM\Event\MdmCommandResendEvent;
use Topalis\App\Entity\AppStateEntityManager;
use Topalis\MDM\Event\MdmCommandEvents;
use Topalis\Device\Entity\Device;

/**
 * AppMdmCommandSubscriber
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
abstract class AppMdmCommandResendSubscriber implements EventSubscriberInterface
{
    /**
     * @var \Topalis\App\Entity\AppStateEntityManager
     */
    protected $appStateEntityManager;

    public function __construct(AppStateEntityManager $appStateEntityManager)
    {
        $this->appStateEntityManager = $appStateEntityManager;
    }

    public static function getSubscribedEvents()
    {
        return array(
            MdmCommandEvents::RESEND => "handleResend",
        );
    }

    /**
     * Update command references in app states to new command.
     *
     * @param MdmCommandResendEvent $event
     */
    public function handleResend(MdmCommandResendEvent $event)
    {
        foreach ($this->getCommandFields() as $field) {
            $this->appStateEntityManager->createQueryBuilder("s")
                ->update()
                ->set("s." . $field, $event->getNewCommand()->getId())
                ->where("s." . $field . " = :oldCommand")
                ->setParameter("oldCommand", $event->getOldCommand())
                ->getQuery()->execute();
        }
    }

    /**
     *
     * @return string[] Fields in the state entity that reference an MDM
     *     command.
     */
    protected function getCommandFields()
    {
        return array("installCommand", "removeCommand");
    }

    /**
     * @param Device $device
     *
     * @return boolean true if the device is supported, false otherwise.
     */
    abstract protected function supports(Device $device);
}
