<?php
namespace Topalis\App\EventListener\MDM;

use Topalis\App\Entity\AppEntityManager;
use Topalis\App\Entity\AppStateEntityManager;
use Topalis\App\Entity\IosAppState;
use Topalis\Configuration\AuralisConfig;
use Topalis\Device\Entity\Device;
use Topalis\Device\Type\IosDeviceType;
use Topalis\MDM\MdmAdmin;
use Topalis\MDM\MdmEvent;
use Topalis\App\Entity\AppState;
use Psr\Log\LoggerInterface;

/**
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class IosAppMdmListener extends AppMdmListener
{

    /**
     * @var AuralisConfig
     */
    private $config;

    public function __construct(
        MdmAdmin $mdmAdmin,
        AppEntityManager $appManager,
        AppStateEntityManager $appStateEntityManager,
        AuralisConfig $config,
        LoggerInterface $logger)
    {
        parent::__construct($mdmAdmin, $appManager, $appStateEntityManager, $logger);

        $this->config = $config;
    }

    public function supports(Device $device)
    {
        return $device->getType() instanceof IosDeviceType;
    }

    public function handleManagedApplicationList(MdmEvent $event)
    {
        parent::handleManagedApplicationList($event);

        // get states to retry installation
        /** @var IosAppState[] $states */
        $states = $this->appStateEntityManager->createQueryBuilder("s")
            ->select("s, a")
            ->join("s.app", "a")
            ->where("a.forceInstall = 1 AND (s.rejected = :userRejected
                OR (s.state = :removed AND s.managed = 1))
                AND s.installAttempts < :installAttempts
                AND s.state != :installed AND s.state != :removalRequested")
            ->setParameter("userRejected", AppState::REJECTED_USER)
            ->setParameter("removed", AppState::REMOVED)
            ->setParameter("installAttempts", $this->config->getAppInstallAttempts())
            ->setParameter("installed", AppState::INSTALLED)
            ->setParameter("removalRequested", AppState::REMOVAL_REQUESTED)
            ->getQuery()
            ->getResult()
        ;

        foreach ($states as $state) {
            if (null !== $state->getLastInstallAttempt()
                && $state->getLastInstallAttempt()->getTimestamp() + 86400 >= time()) {
                continue;
            }
            $this->mdmAdmin->installApp($event->getDevice(), $state->getApp(), true);
        }
    }
}
