<?php
namespace Topalis\App\EventListener\ORM;

use Doctrine\ORM\Event\LifecycleEventArgs;
use Topalis\App\Entity\AndroidApp;
use Topalis\Util\FileSystemUtil;
use Doctrine\ORM\Event\PostFlushEventArgs;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class AndroidUploadEventListener
{
    private $inPostFlush = false;
    private $preRemovedAppIds = array();
    private $postRemovedApps = array();

    private $webDir;

    private $uploadDir = "/uploads/android_app_icons/";

    public function __construct($webDir)
    {
        $this->webDir = $webDir;
    }

    public function postUpdate(LifecycleEventArgs $eventArgs)
    {
        $app = $eventArgs->getEntity();
        if (!$app instanceof AndroidApp) {
            return;
        }

        $uow = $eventArgs->getEntityManager()->getUnitOfWork();
        $changeSet = $uow->getEntityChangeSet($app);

        if (isset($changeSet['apkPath'])) {
            if ($changeSet['apkPath'][0]) {
                // remove previous APK
                FileSystemUtil::tryUnlinkFile($changeSet['apkPath'][0]);
            }
            if (!$changeSet['apkPath'][1]) {
                $this->removeAppDir($app->getId());
            }
        }
    }

    public function preRemove(LifecycleEventArgs $eventArgs)
    {
        $app = $eventArgs->getEntity();
        if (!$app instanceof AndroidApp) {
            return;
        }

        // ids are not available in postRemove, so we use both events to remove
        // the app dir in postFlush if necessary
        $this->preRemovedAppIds[$app->getIdentifier()] = $app->getId();
    }

    public function postRemove(LifecycleEventArgs $eventArgs)
    {
        $app = $eventArgs->getEntity();
        if (!$app instanceof AndroidApp) {
            return;
        }

        // ids are not available in postRemove, so we use both events to remove
        // the app dir in postFlush if necessary
        $this->postRemovedApps[$app->getIdentifier()] = $app;
    }

    public function postFlush(PostFlushEventArgs $eventArgs)
    {
        $eventArgs; // unused

        if ($this->inPostFlush) {
            return;
        }

        $this->inPostFlush = true;

        foreach ($this->preRemovedAppIds as $identifier => $id) {
            if (isset($this->postRemovedApps[$identifier])) {
                $app = $this->postRemovedApps[$identifier];
                if ($app->getApkPath()) {
                    FileSystemUtil::tryUnlinkFile($app->getApkPath());
                }

                $this->removeAppDir($id);
                unset($this->postRemovedApps[$identifier]);
            }
            unset($this->preRemovedAppIds[$identifier]);
        }

        $this->inPostFlush = false;
    }

    private function removeAppDir($id)
    {
        if ($id <= 0) {
            throw new \InvalidArgumentException("id must be a non negative integer");
        }
        if (!is_int($id)) {
            $intId = (int) $id;
            if ((string) $intId != $id) {
                throw new \InvalidArgumentException("id must be a non negative integer");
            }
        }

        $appDir = $this->webDir . $this->uploadDir . "/" . $id;
        if (is_dir($appDir)) {
            FileSystemUtil::rmdirRecursive($appDir);
        }
    }
}
