<?php
namespace Topalis\App\EventListener\ORM;

use Doctrine\ORM\Event\LifecycleEventArgs;
use Symfony\Component\DependencyInjection\ContainerAware;
use Topalis\App\Entity\WinPhoneAppPolicy;

/**
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class WinPhoneAppPolicyListener extends ContainerAware
{
    /**
     * @return \Topalis\WinPhone\OMA\SyncML\SyncMLMdmFacade
     */
    private function getSyncMLMdmFacade()
    {
        return $this->container->get("winphone.oma.syncml.mdmfacade");
    }

    public function postUpdate(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if (!$entity instanceof WinPhoneAppPolicy) {
            return;
        }

        $uow = $eventArgs->getEntityManager()->getUnitOfWork();
        $changeSet = $uow->getEntityChangeSet($entity);
        $configGroups = $entity->getConfigGroups();
        $this->getSyncMLMdmFacade()->updateAppPolicy($entity, $configGroups->getDeleteDiff(),
            $configGroups->getInsertDiff(), $changeSet);
    }

    public function preRemove(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if (!$entity instanceof WinPhoneAppPolicy) {
            return;
        }

        $em = $eventArgs->getEntityManager();
        foreach ($entity->getConfigGroups() as $group) {
            $group->setWinPhoneAppPolicy(null);
            $em->persist($group);
        }
        $em->flush();
    }
}
