<?php
namespace Topalis\App\Form\Handler;

use Symfony\Component\Form\FormInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Form\FormError;
use Topalis\App\Entity\WinPhoneAppPolicy;

/**
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class WinPhoneAppPolicyFormHandler
{
    /**
     * @var \Topalis\App\Entity\WinPhoneAppPolicy
     */
    private $appPolicy;

    /**
     * @var \Symfony\Component\HttpFoundation\Request
     */
    private $request;

    public function __construct(Request $request, WinPhoneAppPolicy $appPolicy)
    {
        $this->request = $request;
        $this->appPolicy = $appPolicy;
    }

    public function process(FormInterface $form)
    {
        $form->setData($this->appPolicy);

        $form->handleRequest($this->request);
        if (!$form->isSubmitted() || !$form->isValid()) {
            return false;
        }

        $policyForm = $form->get("policy");
        $type = $policyForm->get("type")->getData();
        if ($type == WinPhoneAppPolicy::TYPE_BLACKLIST) {
            if ($policyForm->getData()->isDeniedAll()) {
                // the choosen type is blacklist, but no app/publisher is defined
                $policyForm->get("type")->addError(new FormError("app.winphone.policy.blacklistEmpty"));
            }
        }

        return $form->isValid();
    }
}
