<?php
namespace Topalis\App\Form\Type\WinPhone;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Validator\Constraints as Assert;
use Topalis\App\Form\Type\WinPhone\WinPhoneAppPolicyType;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Topalis\App\WinPhone\WinPhoneStore;

/**
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class WinPhoneAppPolicyForm extends AbstractType
{
    /**
     * @var \Topalis\App\WinPhone\WinPhoneStore
     */
    private $store;

    public function __construct(WinPhoneStore $store)
    {
        $this->store = $store;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $options; // unused

        $builder
            ->add("name", "text", array(
                "label"       => "Name",
                "max_length"  => 255,
                "constraints" => array(
                    new Assert\NotBlank(),
                    new Assert\Length(array("max" => 255)),
                )
            ))
            ->add("policy", new WinPhoneAppPolicyType($this->store))
            ->add("configGroups", "configgroup_select", array(
                "label" => "admin.app.winphone.policy.groups",
                "descriptionBlock" => true,
                "description" => "admin.app.winphone.policy.groupsDescription",
                "required" => false,
            ))
            ;
    }

    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(array(
            "data_class" => '\Topalis\App\Entity\WinPhoneAppPolicy'
        ));
    }

    /**
     * Returns the name of this type.
     * @return string The name of this type
     */
    public function getName()
    {
        return "winPhoneAppPolicy";
    }
}
