<?php
namespace Topalis\App\Form\Type\WinPhone;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Topalis\Form\Validator\Constraint\UniqueInCollection;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Form\FormView;
use Symfony\Component\Form\FormInterface;
use Topalis\App\WinPhone\WinPhoneStore;

/**
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class WinPhoneAppProductIdType extends AbstractType
{
    const PATTERN = "/^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$/";

    /**
     * @var \Topalis\App\WinPhone\WinPhoneStore
     */
    private $store;

    public function __construct(WinPhoneStore $store)
    {
        $this->store = $store;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $options; // unused
        $builder
            ->add("productId", "text", array(
                "label" => "admin.app.winphone.policy.guid",
                "descriptionBlock" => true,
                "constraints" => array(
                    new Assert\NotBlank(),
                    new Assert\Regex(
                        array("message" => "app.winphone.guidInvalid", "pattern" => self::PATTERN)),
                    new UniqueInCollection(),
                )
            ))
            ;
    }

    public function finishView(FormView $view, FormInterface $form, array $options)
    {
        $options; // unused

        $productId = $form->get("productId")->getData();
        $productIdView = $view['productId'];

        if ($productIdView->vars['errors']) {
            return;
        }

        if (!$productId) {
            // ensure we have a help block
            $productIdView->vars['description'] = " ";
            return;
        }

        $descriptionParams = array("%id%" => $productId);
        try {
            $app = $this->store->lookup($productId);
            if ($app) {
                $description = "admin.app.winphone.store.link";
                $descriptionParams['%text%'] = $app->getName() . " (" . $app->getVendor() . ")";
            } else {
                $productIdView->vars['status'] = "warning";
                $description = "admin.app.winphone.store.error.notFound";
            }
        } catch (\Exception $e) {
            $productIdView->vars['status'] = "warning";
            $description = "admin.app.error.appStoreConnection";
            $descriptionParams['%errmsg%'] = $e->getMessage();
        }

        $productIdView->vars['description'] = $description;
        $productIdView->vars['descriptionParams'] = $descriptionParams;
    }

    public function getName()
    {
        return "appProductId";
    }
}
