<?php
namespace Topalis\App\Mdm;

use Topalis\Admin\Entity\Admin;
use Topalis\App\Entity\App;
use Topalis\App\Entity\AppState;
use Topalis\Device\Entity\Device;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
interface AppMdmAdminInterface
{
    /**
     * @param Device $device
     * @param App $app
     *
     * @return boolean|NULL true if the specified device supports the specified
     *     app, false if it's not supported, or null if it's unknown.
     */
    public function deviceSupportsApp(Device $device, App $app);

    /**
     * @param Device $device
     * @param array $states States defined in AppState the returned AppStates
     *     must match. If not specified all AppStates are returned.
     *
     * @return array Array of AppState objects for the specified device.
     */
    public function getAppStates(Device $device, array $states = null);

    /**
     * Installs the specified app on the specified device.
     *
     * @param Device $device The device to install the app on.
     * @param App    $app    The app to install.
     * @param bool   $force  If true a install request will be sent even though
     *                       a previous one has been rejected. If false an install
     *                       request will only be sent if there was no previous
     *                       request rejected by the user or the device.
     * @param bool   $removeBeforeInstall If true a remove command will be send
     *                                    before the install command.
     */
    public function installApp(Device $device, App $app, $force = false, $removeBeforeInstall = false);

    /**
     * Installs the specified iOS app on devices of the config group set in $app.
     *
     * @param App  $app   The app to install.
     * @param bool $force If true a install request will be sent even though
     *                    a previous one has been rejected. If false an install
     *                    request will only be sent if there was no previous
     *                    request rejected by the user or the device.
     */
    public function installApps(App $app, $force = false);

    /**
     * Install all applications for a device
     *
     * @param Device $device The device to install the apps on.
     */
    public function installAppsForDevice(Device $device);

    /**
     * Removes the app from the device specified in the $state
     *
     * @param AppState $state
     *
     * @return boolean true if a remove is requested, false otherwise.
     */
    public function removeApp(AppState $state);

    /**
     * Removes the specified app from all devices on which the app has been
     * installed through MDM.
     *
     * @param App $app the app to remove
     */
    public function removeApps(App $app);

    /**
     * Removes the specified app from devices that are not part of a group the
     * app is assigned to.
     *
     * @param App $app the app to remove.
     */
    public function removeUnassignedApps(App $app);

    /**
     * Removes all managed apps from the specified device that are not assigned
     * to it (anymore). It also cancels outstanding install commands.
     *
     * @param Device $device The device to remove unassigned apps.
     */
    public function removeUnassignedAppsFromDevice(Device $device);

    /**
     * @param Device $device
     *
     * @return boolean
     */
    public function supportsDevice(Device $device);

    /**
     * @param App $app
     *
     * @return boolean
     */
    public function supportsApp(App $app);

    /**
     * Sets the admin issuing the commands
     *
     * @param Admin $admin
     */
    public function setAdmin(Admin $admin);
}
