<?php
namespace Topalis\App\Mdm;

use Topalis\App\Entity\App;
use Topalis\Device\Entity\Device;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class AppMdmAdminRegistry
{
    /**
     * @var array
     */
    private $admins;

    /**
     * @param AppMdmAdminInterface[] $admins
     */
    public function __construct(array $admins)
    {
        $this->admins = $admins;
    }

    /**
     * @param Device $device
     *
     * @return AppMdmAdminInterface
     */
    public function getForDevice(Device $device)
    {
        foreach ($this->admins as $admin) {
            if ($admin->supportsDevice($device)) {
                return $admin;
            }
        }

        throw new \InvalidArgumentException(sprintf("no app mdm admin found for device %s", $device->getId()));
    }

    /**
     * @param App $app
     *
     * @return AppMdmAdminInterface
     */
    public function getForApp(App $app)
    {
        foreach ($this->admins as $admin) {
            if ($admin->supportsApp($app)) {
                return $admin;
            }
        }

        throw new \InvalidArgumentException(
            sprintf("no app mdm admin found for app %s (%s)", $app->getId(), get_class($app)));
    }
}
