<?php
namespace Topalis\App\Util;

use Symfony\Component\DomCrawler\Crawler;

/**
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class CrawlerUtil
{
    /**
     * @param Crawler $crawler The crawler to use.
     * @param string $filter The filter to use.
     * @param string $type text, html, or attr.
     * @param string $attribute The name of the attribute, if $type is attr.
     * @return array Values in nodes that matched the filter.
     */
    public static function getAllFromCrawler(Crawler $crawler, $filter, $type = "text", $attribute = null)
    {
        $filteredCrawler = $crawler->filter($filter);

        switch ($type) {
            case "text":
                $rv = $filteredCrawler->each(function (Crawler $node) {
                    return $node->text();
                });
                return $rv;
            case "html":
                $rv = $filteredCrawler->each(function (Crawler $node) {
                    return $node->html();
                });
                return $rv;
            case "attr":
                $rv = $filteredCrawler->each(function (Crawler $node) use ($attribute) {
                    return $node->attr($attribute);
                });
                return $rv;
        }

        throw new \InvalidArgumentException("type can only be 'text', 'html', or 'attr', '" . $type . "' given");
    }

    /**
     * @param Crawler $crawler The crawler to use.
     * @param string $filter The filter to use.
     * @param string $type text, html, or attr.
     * @param string $attribute The name of the attribute, if $type is attr.
     * @return string|NULL The value in the first node that matched the filter,
     *     or null if no node matched the filter.
     */
    public static function getFromCrawler(Crawler $crawler, $filter, $type = "text", $attribute = null)
    {
        $filteredCrawler = $crawler->filter($filter);
        if ($filteredCrawler->count() === 0) {
            return null;
        }

        switch ($type) {
            case "text":
                return $filteredCrawler->text();
            case "html":
                return $filteredCrawler->html();
            case "attr":
                return $filteredCrawler->attr($attribute);
        }

        throw new \InvalidArgumentException("type can only be 'text', 'html', or 'attr', '" . $type . "' given");
    }
}
