<?php
namespace Topalis\App\Validator;

use Symfony\Component\HttpFoundation\File\File;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\Constraints\FileValidator;
use Topalis\App\Exception\IpaFileException;
use Topalis\App\Ios\IpaFile;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class IpaValidator extends FileValidator
{
    public function validate($value, Constraint $constraint)
    {
        parent::validate($value, $constraint);
        if ($this->context->getViolations()->count() > 0) {
            return;
        }

        /** @var $constraint Ipa */
        if (!$value instanceof File) {
            $this->context->addViolation($constraint->noFileUploadMessage);

            return;
        }

        try {
            $ipa = new IpaFile($value->getPathname());
            $ipa->getInfo();
        } catch (\Exception $e) {
            $this->context->addViolation($constraint->message, array("%msg%" => $e->getMessage()));
        }
    }
}
