<?php
namespace Topalis\ConfigGroup\Controller;

use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Topalis\ConfigGroup\Entity\ConfigGroup;

/**
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @Route("/admin/group/{group}/section/ios")
 */
class ProfileSectionController extends Controller
{
    /**
     * @Route("", name="admin_group_section_ios")
     * @param ConfigGroup $group
     * @Template()
     *
     * @return array
     */
    public function indexAction(ConfigGroup $group)
    {
        $payloads = array();
        $builder = $this->get("config_group.form.profile");
        if ($group->isCustomProfile()) {
            foreach ($builder->all() as $child) {
                if ("exchange" !== $child->getName() && "save" !== $child->getName()) {
                    $builder->remove($child->getName());
                }
            }
            $profile = $this->get("profile.factory")->fromXML($group->getProfile());
            foreach ($profile->getPayloads() as $payload) {
                $payloads[] = $payload->getType();
            }
        }

        $form = $builder->getForm();

        $uploadForm = $this->get("config_group.form.profile_upload")->getForm();
        $uploadHandler = $this->get("config_group.form.profile_upload.handler");
        $uploadProcess = $uploadHandler->process($uploadForm, $group);
        $formHandler = $this->get("config_group.form.profile.handler");
        $process = $formHandler->process($form, $group);
        if ($uploadProcess || $process) {
            $this->get("session")->getFlashBag()->add(
                "success",
                $this->get("translator")->trans("admin.group.saveSuccess")
            );

            return $this->redirect(
                $this->generateUrl("admin_group_section_ios", array("group" => $group->getId()))
            );
        }

        return array(
            "group"       => $group,
            "form"        => $form->createView(),
            "upload_form" => $uploadForm->createView(),
            "payloads"    => $payloads,
        );
    }

    /**
     * @Route("/download", name="admin_group_section_profile_download")
     *
     * @param ConfigGroup $group
     *
     * @return \Symfony\Component\HttpFoundation\Response
     * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
     */
    public function downloadAction(ConfigGroup $group)
    {
        if (null === $group->getProfile()) {
            throw new NotFoundHttpException();
        }
        $certificateManager = $this->get("certificate.manager");
        $serverCert = $certificateManager->loadServerCert();
        $response = new Response(
            $certificateManager->sign($group->getProfile(), $serverCert[0], $serverCert[1])
        );
        $disposition = $response->headers->makeDisposition(
            ResponseHeaderBag::DISPOSITION_ATTACHMENT,
            $group->getName() . ".mobileconfig"
        );

        $response->headers->set("Content-Disposition", $disposition);

        return $response;
    }
}
