<?php
namespace Topalis\ConfigGroup\Form\Handler;

use Symfony\Component\Form\FormError;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Validator\Constraints\NotBlank;
use Topalis\Certificate\CertificateManager;
use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\ConfigGroup\Entity\ConfigGroupEntityManager;
use Topalis\Profile\AndroidConfigurationProfile;
use Topalis\Profile\ConfigurationProfile;
use Topalis\Profile\Payload\Android\WifiPayload;
use Topalis\Profile\ProfileFactory;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class AndroidProfileFormHandler
{
    /**
     * @var \Symfony\Component\HttpFoundation\Request
     */
    protected $request;

    /**
     * @var \Topalis\ConfigGroup\Entity\ConfigGroupEntityManager
     */
    protected $manager;
    /**
     * @var \Topalis\Profile\ProfileFactory
     */
    private $profileFactory;
    /**
     * @var \Topalis\Certificate\CertificateManager
     */
    private $certificateManager;

    public function __construct(Request $request, ConfigGroupEntityManager $manager,
        ProfileFactory $profileFactory, CertificateManager $certificateManager)
    {
        $this->request = $request;
        $this->manager = $manager;
        $this->profileFactory = $profileFactory;
        $this->certificateManager = $certificateManager;
    }

    public function process(FormInterface $form, ConfigGroup $group)
    {
        if (null !== $profile = $group->getAndroidProfile()) {
            $profile = $this->profileFactory->fromXML($profile);
        } else {
            $profile = new AndroidConfigurationProfile();
            $profile->setIdentifier($this->profileFactory->getDeviceProfileIdentifier());
            $profile->setOrganization($this->certificateManager->getCAOrg());
        }
        $form->setData($profile);
        $form->handleRequest($this->request);
        if ($form->has("wifi")) {
            foreach ($form->get("wifi")->all() as $child) {
                /** @var WifiPayload $payload */
                $payload = $child->getData();
                if (null === $child->get("encryption_type")->getData()) {
                    $payload->setPassword(null);
                } else {
                    if (strlen($payload->getPassword()) == 0) {
                        $notBlank = new NotBlank();
                        $child->get("password")->addError(new FormError($notBlank->message));
                    }
                }
            }
        }

        if ($form->get("save")->isClicked() && $form->isValid()) {
            /** @var $profile ConfigurationProfile */
            $profile = $form->getData();
            $group->setAndroidProfile($profile->toXML());
            $this->onSuccess($group);

            return true;
        }

        return false;
    }

    protected function onSuccess(ConfigGroup $group)
    {
        $this->manager->save($group);
    }
}
