<?php
namespace Topalis\ConfigGroup\Form\Handler;

use Symfony\Component\DependencyInjection\ContainerAware;
use Topalis\ConfigGroup\Entity\ConfigGroup;
use Symfony\Component\Form\FormInterface;

/**
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class ConfigGroupRemoveFormHandler extends ContainerAware
{
    /**
     * @return \Topalis\ConfigGroup\Entity\ConfigGroupEntityManager
     */
    private function getConfigGroupEntityManager()
    {
        return $this->container->get("config_group.entity.manager.config_group");
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     * @return \Symfony\Component\BrowserKit\Request
     */
    private function getRequest()
    {
        return $this->container->get("request");
    }

    /**
     * @return \Topalis\User\Entity\UserEntityManager
     */
    private function getUserEntityManager()
    {
        return $this->container->get("user.entity.manager.user");
    }

    public function process(FormInterface $form, ConfigGroup $group)
    {
        $request = $this->getRequest();
        if ($request->isMethod("POST")) {
            $hasUsers = $form->has("userAction");
            $hasDevices = $form->has("deviceAction");
            if (!$hasUsers && !$hasDevices) {
                $this->onSuccess($group);
                return true;
            }

            $form->submit($request);
            if ($form->isValid()) {
                if ($hasDevices) {
                    $deviceEm = $this->getDeviceEntityManager();
                    $deviceAction = $form->get("deviceAction")->getData();
                    if ($deviceAction == "delete") {
                        foreach ($group->getDevices() as $device) {
                            if ($device->isDeleted()) {
                                continue;
                            }
                            $deviceEm->delete($device, false);
                        }
                    } else {
                        $newDeviceGroup = $form->get("deviceConfigGroup")->getData();
                        foreach ($group->getDevices() as $device) {
                            if ($device->isDeleted()) {
                                continue;
                            }
                            $device->setConfigGroup($newDeviceGroup);
                            $deviceEm->save($device, false);
                        }
                    }
                    $deviceEm->flush();
                }

                if ($hasUsers) {
                    $userEm = $this->getUserEntityManager();
                    $userAction = $form->get("userAction")->getData();
                    if ($userAction == "delete") {
                        foreach ($group->getUsers() as $user) {
                            $userEm->delete($user, false);
                        }
                    } else {
                        $newGroupUser = $form->get("userConfigGroup")->getData();
                        foreach ($group->getUsers() as $user) {
                            $user->setConfigGroup($newGroupUser);
                            $userEm->save($user, false);
                        }
                    }
                    $userEm->flush();
                }

                $this->onSuccess($group);
                return true;
            }
        }

        return false;
    }

    private function onSuccess(ConfigGroup $group)
    {
        $this->getConfigGroupEntityManager()->delete($group);
    }
}
