<?php
namespace Topalis\ConfigGroup\Form\Handler;

use Symfony\Component\Form\FormInterface;
use Symfony\Component\HttpFoundation\Request;
use Topalis\Certificate\Util\CertUtil;
use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\ConfigGroup\Entity\ConfigGroupEntityManager;
use Topalis\Util\ArrayUtil;
use Symfony\Component\HttpFoundation\File\UploadedFile;

/**
 *
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class ProfileUploadFormHandler
{
    /**
     * @var \Symfony\Component\HttpFoundation\Request
     */
    protected $request;

    /**
     * @var \Topalis\ConfigGroup\Entity\ConfigGroupEntityManager
     */
    protected $manager;

    public function __construct(Request $request, ConfigGroupEntityManager $manager)
    {
        $this->request = $request;
        $this->manager = $manager;
    }

    public function process(FormInterface $form, ConfigGroup $group)
    {
        $form->setData($group);
        $params = $this->request->request->get($form->getName());
        if (!empty($params) && isset($params['upload'])) {
            $form->handleRequest($this->request);
        }
        if ($form->get("upload")->isClicked() && $form->isValid()) {
            $this->upload($group, $form->get("profile")->getData());
            $this->onSuccess($group);

            return true;
        }

        return false;
    }

    public function upload(ConfigGroup $group, UploadedFile $file)
    {
        $content = file_get_contents($file->getPathname());
        $decoded = CertUtil::decodePKCS7($content);
        $signedData = ArrayUtil::get($decoded, "data");
        if (strlen($signedData) > 0) {
            $content = $signedData;
        }
        $group->setCustomProfile(true);
        $group->setCustomProfileName($file->getClientOriginalName());
        $group->setProfile($content);
    }

    protected function onSuccess(ConfigGroup $user)
    {
        $this->manager->save($user);
    }
}
