<?php
namespace Topalis\ConfigGroup\Form\Handler;

use Symfony\Component\Form\FormInterface;
use Symfony\Component\HttpFoundation\Request;
use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\ConfigGroup\Entity\ConfigGroupEntityManager;
use Topalis\WinPhone\MS\MSWlanProfile;
use Topalis\WinPhone\Entity\MSWlanConfig;
use Symfony\Component\Form\FormError;
use Symfony\Component\Validator\Constraints\NotBlank;

/**
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class WinPhoneFormHandler
{
    /**
     * @var \Symfony\Component\HttpFoundation\Request
     */
    private $request;

    /**
     * @var \Topalis\ConfigGroup\Entity\ConfigGroupEntityManager
     */
    private $manager;

    public function __construct(Request $request, ConfigGroupEntityManager $manager)
    {
        $this->request = $request;
        $this->manager = $manager;
    }

    public function process(FormInterface $form, ConfigGroup $group)
    {
        $passCodeForm = $form->get("pass_code");
        $passCodeData = $group->getWinPhonePassCode();
        if ($passCodeData) {
            $passCodeForm->setData(array($passCodeData));
        }

        $restrictionsForm = $form->get("restrictions");
        $restrictionsData = $group->getWinPhoneRestrictions();
        if ($restrictionsData) {
            $restrictionsForm->setData(array($restrictionsData));
        }

        $wlanForm = $form->get("wifi");
        $wlanData = array();
        $wlanConfigs = $group->getMSWlanConfigs();
        foreach ($wlanConfigs as $wlanConfig) {
            $wlanProfile = MSWlanProfile::fromXml($wlanConfig->getProfile());
            $wlanData[] = $wlanProfile;
        }
        $wlanForm->setData($wlanData);

        $exchangeForm = $form->get("exchange");
        $exchangeForm->setData($group->getWinPhoneExchangeConfig());

        $params = $this->request->request->get($form->getName());
        if (isset($params['save'])) {
            $form->handleRequest($this->request);
            if (!$form->isValid()) {
                return false;
            }

            $passCodeData = $passCodeForm->getData();
            if (isset($passCodeData[0])) {
                $group->setWinPhonePassCode($passCodeData[0]);
            } else {
                $group->setWinPhonePassCode(null);
            }

            $restrictionsData = $restrictionsForm->getData();
            if (isset($restrictionsData[0])) {
                $group->setWinPhoneRestrictions($restrictionsData[0]);
            } else {
                $group->setWinPhoneRestrictions(null);
            }

            $this->handleWlan($wlanForm, $group);

            $exchangeData = $exchangeForm->getData();
            $group->setWinPhoneExchangeConfig($exchangeData);

            if ($form->isValid()) {
                $this->onSuccess($group);
                return true;
            } else {
                $this->manager->refresh($group);
            }
        }

        return false;
    }

    private function handleWlan(FormInterface $form, ConfigGroup $group)
    {
        $curWlanConfigs = array();
        foreach ($group->getMSWlanConfigs() as $wlanConfig) {
            $curWlanConfigs[$wlanConfig->getSsid()] = $wlanConfig;
        }

        $valid = true;
        foreach ($form->all() as $wlanForm) {
            $wlanProfile = $wlanForm->getData();
            if ($wlanProfile->getEncryption() != "none" && !$wlanProfile->getKeyMaterial()) {
                $notBlank = new NotBlank();
                $wlanForm->get("password")->addError(new FormError($notBlank->message));
            }

            if ($wlanForm->isValid()) {
                $ssid = $wlanProfile->getSsid();
                if (isset($curWlanConfigs[$ssid])) {
                    $curWlanConfigs[$ssid]->setProfile($wlanProfile->toXml());
                    unset($curWlanConfigs[$ssid]);
                } else {
                    $wlanConfig = new MSWlanConfig();
                    $wlanConfig->setSsid($ssid);
                    $wlanConfig->setProfile($wlanProfile->toXml());
                    $group->addMSWlanConfig($wlanConfig);
                }
            } else {
                $valid = false;
            }
        }

        if (!$valid) {
            return;
        }

        foreach ($curWlanConfigs as $wlanConfig) {
            $group->removeMSWlanConfig($wlanConfig);
        }
    }

    private function onSuccess(ConfigGroup $group)
    {
        $this->manager->save($group);
    }
}
