<?php
namespace Topalis\ConfigGroup\Form\Transformer\WinPhone;

use Symfony\Component\Form\DataTransformerInterface;
use Topalis\ConfigGroup\Form\Transformer\WinPhone\Util\WinPhoneTransformerUtil;

/**
 * Transformer for Windows Phone restrictions configuration.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class WinPhoneRestrictionsTransformer implements DataTransformerInterface
{
    private static $keys = array(
        "requireDeviceEncryption",
        "allowStore",
        "allowCamera",
        "allowScreenCapture",
        "allowVoiceRecording",
        "allowSaveAsOfOfficeFiles",
        "allowCortana",
        "allowCopyPaste",
        "allowSyncMySettings",
        "allowWiFi",
        "allowAutoConnectToWiFiSenseHotspots",
        "allowInternetSharing",
        "allowManualWiFiConfiguration",
        "allowWiFiHotspotReporting",
        "allowNFC",
        "allowBluetooth",
        "allowCellularDataRoaming",
        "allowVPNOverCellular",
        "allowVPNRoamingOverCellular",
        "allowBrowser",
        "allowUSBConnection",
        "allowStorageCard",
        "allowTelemetry" => 2,
        "allowLocation",
        "allowSearchToUseLocation",
        "safeSearchPermissions" => 1,
        "allowStoringImagesFromVisionSearch",
        "allowManualRootCertificateInstallation",
        "allowDeveloperUnlock",
        "allowMicrosoftAccountConnection",
        "allowAddingNonMicrosoftAccountsManually",
    );

    public function transform($value)
    {
        if ($value) {
            foreach (self::$keys as $key => $val) {
                if (is_int($key)) {
                    WinPhoneTransformerUtil::addOrReplaceBool($value, $val);
                } else if (is_bool($val)) {
                    WinPhoneTransformerUtil::addOrReplaceBool($value, $key, 1, $val);
                } else if (!isset($value[$key])) {
                    $value[$key] = $val;
                }
            }
        }

        return $value;
    }

    public function reverseTransform($value)
    {
        foreach (array_keys($value) as $key) {
            if ($key == "safeSearchPermissions" || $key == "allowTelemetry") {
                continue;
            } else {
                WinPhoneTransformerUtil::revReplaceBool($value, $key);
            }
        }

        return $value;
    }
}
