<?php
namespace Topalis\ConfigGroup\Form\Type\Android;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Validator\Constraints\NotBlank;
use Topalis\Form\Validator\Constraint\UniqueInCollection;

/**
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class WifiType extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $options; // unused

        $prefixGeneral = "admin.group.profile.wifi.";

        $builder
            ->add("ssid", "text", array(
                "label" => $prefixGeneral . "ssid",
                "constraints" => array(
                    new NotBlank(),
                    new UniqueInCollection(),
                ),
                "disabled" => true,
            ))
            ->add("isHidden", "checkbox", array(
                "label" => $prefixGeneral . "hidden_network",
                "disabled" => true,
            ))
            ->add("encryption_type", "choice", array(
                "label"       => $prefixGeneral . "encryption_type",
                "empty_value" => $prefixGeneral . "encryption_type.none",
                "attr"        => array("class" => "wifi_encryption_type"),
                "choices"     => array(
                    "WEP" => "WEP",
                    "WPA" => "WPA",
                ),
                "disabled" => true,
            ))
            ->add("password", "password", array(
                "label" => $prefixGeneral . "password",
                "always_empty" => false,
                "disabled" => true,
            ))
            ->add("proxy_type", "choice", array(
                "label"       => $prefixGeneral . "proxy_type",
                "empty_value" => $prefixGeneral . "proxy_type.none",
                "attr"        => array("class" => "wifi_proxy_type"),
                "choices"     => array(
                    "Manual" => $prefixGeneral . "proxy_type.manual",
                ),
                "disabled" => true,
            ))
            ->add("proxyHost", "text", array(
                "label" => $prefixGeneral . "proxy_server",
                "disabled" => true,
            ))
            ->add("proxyPort", "number", array(
                "label" => $prefixGeneral . "proxy_server_port",
                "disabled" => true,
            ))
        ;
    }

    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(array(
            "data_class" => 'Topalis\Profile\Payload\Android\WifiPayload',
            "required"    => false,
        ));
    }

    public function getName()
    {
        return "wifi";
    }
}
