<?php
namespace Topalis\ConfigGroup\Form\Type;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Validator\Constraints as Assert;
use Topalis\ConfigGroup\Form\Transformer\DeviceTimeoutTransformer;

/**
 * @author Florian Pfitzer <florian.pfitzer@to.com>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class ConfigGroupForm extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $options; // unused
        $builder
            ->add("name", "text", array(
                "label" => "admin.group.name",
                "max_length" => 255,
            ))
            ->add("comment", "text", array(
                "label" => "admin.group.comment",
                "max_length" => 255,
                "required" => false,
            ))
            ->add(
                $builder->create("deviceTimeout", "integer", array(
                    "label" => "admin.group.deviceTimeout",
                    "description" => "admin.group.deviceTimeoutDescr",
                    "descriptionBlock" => false,
                    "constraints" => new Assert\Range(array("min" => 0)),
                    "required" => false,
                ))->addViewTransformer(new DeviceTimeoutTransformer())
            )
        ;
    }

    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(array(
            "data_class" => 'Topalis\ConfigGroup\Entity\ConfigGroup'
        ));
    }

    public function getName()
    {
        return "config_group";
    }
}
