<?php
namespace Topalis\ConfigGroup\Form\Type\Ios;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Validator\Constraints\NotBlank;
use Topalis\Form\Validator\Constraint\UniqueInCollection;

/**
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class WifiType extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $options; // unused
        $prefix = "admin.group.profile.wifi.";
        $builder
            ->add("ssid", "text", array(
                "label"       => $prefix . "ssid",
                "constraints" => array(
                    new NotBlank(),
                    new UniqueInCollection(),
                ),
                "disabled" => true,
            ))
            ->add("hidden_network", "checkbox", array(
                "label" => $prefix . "hidden_network",
                "disabled" => true,
            ))
            ->add("auto_join", "checkbox", array(
                "label" => $prefix . "auto_join",
                "disabled" => true,
            ))
            ->add("encryption_type", "choice", array(
                "label"       => $prefix . "encryption_type",
                "empty_value" => $prefix . "encryption_type.none",
                "attr"        => array("class" => "wifi_encryption_type"),
                "choices"     => array(
                    "WEP" => "WEP",
                    "WPA" => "WPA",
                    "Any" => $prefix . "encryption_type.any"
                ),
                "disabled" => true,
            ))
            ->add("password", "password", array(
                "label"        => $prefix . "password",
                "always_empty" => false,
                "disabled" => true,
            ))
            ->add("hotspot", "checkbox", array(
                "label"   => $prefix . "hotspot",
                "os"      => array("iOS" => ">=7.0"),
                "disabled" => true,
            ))
            ->add("proxy_type", "choice", array(
                "label"       => $prefix . "proxy_type",
                "empty_value" => $prefix . "proxy_type.none",
                "attr"        => array("class" => "wifi_proxy_type"),
                "choices"     => array(
                    "Manual" => $prefix . "proxy_type.manual",
                    "Auto"   => $prefix . "proxy_type.auto"
                ),
                "disabled" => true,
            ))
            ->add("proxy_server", "text", array(
                "label" => $prefix . "proxy_server",
                "disabled" => true,
            ))
            ->add("proxy_server_port", "number", array(
                "label" => $prefix . "proxy_server_port",
                "disabled" => true,
            ))
            ->add("proxy_username", "text", array(
                "label" => $prefix . "proxy_username",
                "disabled" => true,
            ))
            ->add("proxy_password", "text", array(
                "label" => $prefix . "proxy_password",
                "disabled" => true,
            ))
            ->add("proxy_pac_url", "text", array(
                "label" => $prefix . "proxy_pac_url",
                "disabled" => true,
            ))
            ->add("proxy_pac_fallback_allowed", "checkbox", array(
                "label" => $prefix . "proxy_pac_fallback_allowed",
                "os"      => array("iOS" => ">=7.0"),
                "disabled" => true,
            ));

    }

    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(array(
            "data_class" => 'Topalis\Profile\Payload\WifiPayload',
            "required"   => false
        ));
    }

    public function getName()
    {
        return "wifi";
    }
}
