<?php
namespace Topalis\ConfigGroup\Form\Type\WinPhone;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Validator\Constraints\Range;
use Topalis\ConfigGroup\Form\Transformer\WinPhone\WinPhonePassCodeTransformer;

/**
 *
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class WinPhonePassCodeType extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $options; // unused
        $prefix = "admin.group.profile.passcode.";
        $prefixWin = "admin.group.winphone.passcode.";

        $minPasswordLength = array();
        for ($i = 4; $i <= 16; $i++) {
            $minPasswordLength[$i] = $i;
        }

        $builder
            ->add("allowSimpleDevicePassword", "checkbox", array(
                "label" => $prefix."allowSimple",
                "description" => $prefix."allowSimpleDescr"
            ))
            ->add("alphanumericDevicePasswordRequired", "checkbox", array(
                "label" => $prefixWin."allowAlphanumeric",
                "description" => $prefixWin."allowAlphanumericDescr"
            ))
            ->add("minDevicePasswordComplexCharacters", "choice", array(
                "label" => $prefixWin."minComplexChars",
                "description" => $prefixWin."minComplexCharsDescr",
                "choices" => array(
                    1 => 1,
                    2 => 2,
                    3 => 3,
                    4 => 4
                )
            ))
            ->add("minDevicePasswordLength", "choice", array(
                "label" => $prefix."minLength",
                "description" => $prefix."minLengthDescr",
                "choices" => $minPasswordLength
            ))
            ->add("devicePasswordExpiration", "number", array(
                "label" => $prefix."maxPINAgeInDays",
                "description" => $prefix."maxPINAgeInDaysDescr",
                "constraints" => new Range(array(
                    "min" => 0,
                    "max" => 730
                ))
            ))
            ->add("maxInactivityTimeDeviceLock", "number", array(
                "label" => $prefixWin."maxInactivity",
                "description" => $prefix."maxInactivityDescr",
                "constraints" => new Range(array(
                    "min" => 0,
                    "max" => 999
                ))
            ))
            ->add("devicePasswordHistory", "number", array(
                "label" => $prefix."pinHistory",
                "description" => $prefix."pinHistoryDescr",
                "constraints" => new Range(array(
                    "min" => 0,
                    "max" => 50
                ))
            ))
            ->add("disableGracePeriod", "checkbox", array(
                "label" => $prefixWin. "disableGracePeriod",
                "os" => array("WinPhone" => ">=8.1")
            ))
            ->add("maxDevicePasswordFailedAttempts", "number", array(
                "label" => $prefixWin."maxFailedAttempts",
                "description" => $prefix."maxFailedAttemptsDescr",
                "constraints" => new Range(array(
                    "min" => 0,
                    "max" => 999
                ))
            ))
            ->addViewTransformer(new WinPhonePassCodeTransformer())
        ;
    }

    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(array(
            "required" => false,
        ));
    }

    public function getName()
    {
        return "code";
    }
}
