<?php
namespace Topalis\ConfigGroup\Form\Type\WinPhone;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Validator\Constraints\NotBlank;
use Topalis\ConfigGroup\Form\Transformer\WinPhone\WinPhoneConnectionModeTransformer;
use Symfony\Component\Validator\Constraints\Length;
use Topalis\ConfigGroup\Form\Transformer\WinPhone\WinPhoneWifiTransformer;
use Topalis\Form\Validator\Constraint\UniqueInCollection;

/**
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class WinPhoneWifiType extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $options; // unused
        $prefix = "admin.group.profile.wifi.";
        $builder
            ->add("ssid", "text", array(
                "label"       => $prefix . "ssid",
                "constraints" => array(
                    new NotBlank(),
                    new UniqueInCollection(),
                ),
                "disabled" => true,
            ))
            ->add("nonBroadcast", "checkbox", array(
                "label" => $prefix . "hidden_network"
            ))
            // for some reason connectionMode MUST be set to "manual"
            /*
            ->add($builder
                ->create("connectionMode", "checkbox", array(
                    "label" => $prefix . "auto_join"
                ))->addModelTransformer(new WinPhoneConnectionModeTransformer())
            )
            */
            ->add("authentication", "choice", array(
                "label"       => $prefix . "encryption_type",
                "attr"        => array("class" => "wifi_encryption_type"),
                "choices"     => array(
                    "open" => $prefix . "encryption_type.none",
                    // "shared" => "WEP",
                    "WPAPSK" => "WPA",
                    "WPA2PSK" => "WPA2"
                ),
                "disabled" => true,
            ))
            ->add("password", "password", array(
                "label"        => $prefix . "password",
                "property_path" => "keyMaterial",
                "always_empty" => false,
                "constraints" => new Length(array("min" => 8, "max" => 63)),
                "disabled" => true,
            ))
            ->addViewTransformer(new WinPhoneWifiTransformer())
            ;

    }

    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(array(
            "data_class" => 'Topalis\WinPhone\MS\MSWlanProfile',
            "required"   => false
        ));
    }

    public function getName()
    {
        return "wifi";
    }
}
