<?php
namespace Topalis\ConfigGroup\Validator;

use Symfony\Component\HttpFoundation\File\File;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Topalis\Certificate\Util\CertUtil;
use Topalis\Profile\ConfigurationProfile;
use Topalis\Util\ArrayUtil;

/**
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class ConfigProfileValidator extends ConstraintValidator
{
    public function validate($value, Constraint $constraint)
    {
        /** @var $constraint ConfigProfile */
        if (null === $value || '' === $value) {
            return;
        }

        if (!$value instanceof File) {
            return;
        }

        if (count($this->context->getViolations()) > 0) {
            return;
        }

        $content = file_get_contents($value->getPathname());
        $decoded = CertUtil::decodePKCS7($content);
        $signedData = ArrayUtil::get($decoded, "data");
        if (strlen($signedData) > 0) {
            $content = $signedData;
        }

        $profile = new ConfigurationProfile($content);
        if (null !== $profile->get("EncryptedPayloadContent")) {
            $this->context->addViolation($constraint->encryptedProfileMessage);
            return;
        }
    }
}
