<?php
namespace Topalis\Configuration\Client;

use Buzz\Client\Curl;
use Buzz\Message\RequestInterface;
use Topalis\Configuration\AuralisConfig;

/**
 * Extend curl buzz curl client with auralis proxy config
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class AuralisCurl extends Curl
{
    public function __construct(AuralisConfig $config)
    {
        parent::__construct();
        $this->resetTimeouts();
        // disable IPv6, see #AUR-484
        $this->setOption(CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        $this->loadConfig($config);
    }

    /**
     * Loads proxy configuration from the specified AuralisConfig.
     *
     * @param AuralisConfig $config The AuralisConfig to load.
     */
    public function loadConfig(AuralisConfig $config)
    {
        // clear old values
        $this->setOption(CURLOPT_PROXY, null);
        $this->setOption(CURLOPT_PROXYAUTH, null);
        $this->setOption(CURLOPT_PROXYUSERPWD, null);

        $proxy = $config->getHttpProxy();
        if (strlen($proxy) > 0) {
            $this->setOption(CURLOPT_PROXY, $proxy);
            $proxyUser = $config->getHttpProxyUser();
            $proxyPassword = $config->getHttpProxyPassword();
            if (strlen($proxyUser) > 0 && strlen($proxyPassword) > 0) {
                $this->setOption(CURLOPT_PROXYAUTH, CURLAUTH_BASIC);
                $this->setOption(CURLOPT_PROXYUSERPWD, $proxyUser . ":" . $proxyPassword);
            }
        }
    }

    /**
     * Sets the timeout and the connect timeout to their initial values (30
     * seconds and 10 seconds).
     */
    public function resetTimeouts()
    {
        $this->setTimeout(30);
        $this->setConnectTimeout(10);
    }

    /**
     * Sets the number of seconds to wait while trying to connect. Use 0 to wait
     * indefinitely.
     *
     * @param int $connectTimeout The number of seconds to set.
     */
    public function setConnectTimeout($connectTimeout)
    {
        $this->setOption(CURLOPT_CONNECTTIMEOUT, $connectTimeout);
    }

}
