<?php
namespace Topalis\Configuration;

/**
 * This interface describes methods for accessing configurations
 * with key value pairs.
 *
 * @author Dominic Tubach
 *
 */
interface Configuration
{
    /**
     * Clears all properties from the configuration.
     */
    public function clear();

    /**
     * @param string $key A property key.
     * @return boolean true if a property with the specified key exists, false
     *   otherwise.
     */
    public function containsProperty($key);

    /**
     * @return string[string] An associative array with all properties.
     */
    public function getProperties();

    /**
     * @param string $key The key to get the property for.
     * @param string $default The value returned if no property with the
     *   specified key is set.
     * @return string returns the property with the specified key,
     *   or the specified default value if no property with the specified key is
     *   set.
     */
    public function getProperty($key, $default = null);

    /**
     * Removes the property with the specified key.
     *
     * @param string $key The key of the property to remove.
     * @return boolean true if the configuration has been changed, false
     *   otherwise.
     */
    public function removeProperty($key);

    /**
     * Sets the property with the specified key to the specified value.
     *
     * @param string $key The key to set the property for.
     * @param string $value The value to be set.
     * @return boolean true if the configuration has been changed, false
     *   otherwise.
     */
    public function setProperty($key, $value);
}
