<?php
namespace Topalis\Configuration\Controller\Section;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Component\HttpFoundation\Request;
use Topalis\Configuration\Form\AdvancedConfigurationForm;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @Route("/admin/config/advanced")
 */
class AdvancedConfigSectionController extends Controller
{
    /**
     * @Route("", name="admin_config_advanced")
     * @Template()
     *
     * @param Request $request The request.
     */
    public function indexAction(Request $request)
    {
        $config = $this->get("config.auralis");
        $apache = $this->get("apache");
        $asfApacheMod = $this->get("asf.apache_mod");
        $asfModConfig = $asfApacheMod->getConfig();

        $form = $this->createForm(new AdvancedConfigurationForm(), array(
            "port" => array(
                "primaryPort" => $config->getPrimaryPort(),
                "secondaryPort" => $config->getSecondaryPort(),
            ),
            "minWbxmlLen" => $asfModConfig->getMinWbxmlLen(),
            "itunesStore" => strtoupper($config->getItunesAppStore()),
            "googlePlayStoreLang" => $config->getGooglePlayStoreLang(),
            "appInstallAttempts" => $config->getAppInstallAttempts(),
        ));
        $form->handleRequest($request);

        if ($form->isValid()) {
            $data = $form->getData();

            $config->setPrimaryPort($data['port']['primaryPort']);
            $config->setSecondaryPort($data['port']['secondaryPort']);

            $asfModConfig->setMinWbxmlLen($data['minWbxmlLen']);
            $asfApacheMod->enable($asfModConfig);

            $config->setItunesAppStore(strtolower($data['itunesStore']));
            $config->setGooglePlayStoreLang($data['googlePlayStoreLang']);
            $config->setAppInstallAttempts($data['appInstallAttempts']);

            $this->get("session")->getFlashBag()->add("success",
                $this->get("translator")->trans("admin.global.saveSuccess")
            );

            return $this->redirect($this->generateUrl("admin_config_advanced"));
        }

        return array(
            "primaryPort"    => $apache->getDefine("auralis_primary_port"),
            "secondaryPort"  => $apache->getDefine("auralis_secondary_port"),
            "form"           => $form->createView(),
            "sections"       => $this->container->getParameter("config.sections"),
            "active_section" => "advanced",
        );
    }
}
