<?php
namespace Topalis\Configuration\Controller\Section;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Component\HttpFoundation\Request;
use Topalis\Admin\Entity\Admin;
use Topalis\Configuration\Form\AdminNetworkConfigurationForm;

/**
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @Route("/admin/config/network")
 */
class NetworkConfigController extends Controller
{
    /**
     * @Route("", name="admin_config_network")
     * @Template()
     *
     * @param Request $request The request.
     */
    public function indexAction(Request $request)
    {
        $systemConfig = $this->get("system.systemconfig");
        $networkConfigSupported = $systemConfig->isNetworkConfigSupported();
        $timeConfigSupported = $systemConfig->isTimeConfigSupported();

        $config = $this->get("config.auralis");

        $storedData = array(
            "apacheBasic" => array(
                "hostname"  => $config->getHostname(),
                "allowFrom" => $config->getAllowFrom(),
            ),
            "httpProxy" => array(
                "host"      => $config->getHttpProxy(),
                "user"      => $config->getHttpProxyUser(),
                "password"  => $config->getHttpProxyPassword(),
            ),
            "adminPassword" => null,
        );

        if ($networkConfigSupported) {
            $storedData['interface'] = array(
                "bootProto" => $systemConfig->getNetwork()->getBootProto(),
                "ipAddress" => $systemConfig->getNetwork()->getIpAddress(),
                "netmask"   => $systemConfig->getNetwork()->getNetmask(),
                "gateway"   => $systemConfig->getNetwork()->getGateway(),
                "dns1"      => $systemConfig->getNetwork()->getDNS1(),
                "dns2"      => $systemConfig->getNetwork()->getDNS2(),
            );
        }
        if ($timeConfigSupported) {
            $storedData['time'] = array(
                "ntpServers" => implode(", ", $systemConfig->getNTPServers()),
                "timeZone"   => $systemConfig->getTimeZone(),
            );
        }
        $formBuilder = $this->get("form.factory")->createBuilder(
            new AdminNetworkConfigurationForm(), $storedData, array(
                "interface" => $networkConfigSupported,
                "time"      => $timeConfigSupported
            )
        );

        // disable admin password if admin exists
        if ($this->adminExists()) {
            $formBuilder->remove("adminPassword");
        }
        $form = $formBuilder->getForm();

        $form->handleRequest($request);
        if ($form->isValid()) {
            // Admin
            if (!$this->adminExists()) {
                $admin = new Admin();
                $admin
                    ->setLogin("admin")
                    ->setLevel(0)
                    ->setRealname("Initial admin")
                    ->setPlainPassword($form->get("adminPassword")->getData())
                ;
                $this->get("admin.entity.manager.admin")->save($admin);
            }
            $data = $form->getData();

            $config->setHostname($data['apacheBasic']['hostname']);
            $config->setAllowFrom($data['apacheBasic']['allowFrom']);
            $config->setHttpProxy($data['httpProxy']['host']);
            $config->setHttpProxyPassword($data['httpProxy']['password']);
            $config->setHttpProxyUser($data['httpProxy']['user']);
            if ($networkConfigSupported) {
                $interfaceData = $data['interface'];
                $systemConfig->getNetwork()->setBootProto($interfaceData['bootProto']);
                if ($interfaceData['bootProto'] == "static") {
                    $systemConfig->getNetwork()->setIpAddress($interfaceData['ipAddress']);
                    $systemConfig->getNetwork()->setNetmask($interfaceData['netmask']);
                    $systemConfig->getNetwork()->setGateway($interfaceData['gateway']);
                    $systemConfig->getNetwork()->setDNS1($interfaceData['dns1']);
                    $systemConfig->getNetwork()->setDNS2($interfaceData['dns2']);
                }
            }
            if ($timeConfigSupported) {
                $systemConfig->setNTPServers(explode(",", $data['time']['ntpServers']));
                $systemConfig->setTimeZone($data['time']['timeZone']);
            }

            $this->get("session")->getFlashBag()->add("success",
                $this->get("translator")->trans("admin.global.saveSuccess")
            );

            return $this->redirect($this->generateUrl("admin_config_network"));
        }

        return array(
            "embed"          => $request->attributes->get("embed", false),
            "form"           => $form->createView(),
            "sections"       => $this->container->getParameter("config.sections"),
            "active_section" => "network",
        );
    }

    protected function adminExists()
    {
        return $this->get("db")->exists("admins", array("login" => "admin"));
    }
}
