<?php
namespace Topalis\Configuration\Exception;

/**
 * Exception used when a configuration restore is not possible because the
 * application version that created the backup is different from the version
 * of the application the restore shall happen.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class VersionMismatchException extends \Exception
{
    private $currentVersion;
    private $backupVersion;

    public function __construct($currentVersion, $backupVersion)
    {
        $this->currentVersion = $currentVersion;
        $this->backupVersion = $backupVersion;

        if ($backupVersion === null) {
            $backupVersion = "unknown";
        }
        $message = "The version that created the backup ($backupVersion)
        differs from the current version ($currentVersion)";
        parent::__construct($message);
    }

    public function getCurrentVersion()
    {
        return $this->currentVersion;
    }

    public function getBackupVersion()
    {
        return $this->backupVersion;
    }

}
