<?php
namespace Topalis\Configuration;

class LdapConfig
{
    public $hostname;
    public $port;
    public $security;
    public $bindName;
    public $bindPassword;
    public $baseDn;
    public $filter;

    /**
     * Sets the base dn. If a DNS name is given, it is converted into an LDAP
     * dn.
     *
     * @param string $baseDn
     *
     * @return self
     */
    public function setBaseDn($baseDn)
    {
        if (strlen($baseDn) > 0 && stripos($baseDn, "=") === false) {
            $split = explode(".", $baseDn);
            $baseDn = "dc=" . join(",dc=", $split);
        }
        $this->baseDn = $baseDn;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getBaseDn()
    {
        return $this->baseDn;
    }

    /**
     * @param mixed $bindName
     *
     * @return self
     */
    public function setBindName($bindName)
    {
        $this->bindName = $bindName;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getBindName()
    {
        return $this->bindName;
    }

    /**
     * @param mixed $bindPassword
     *
     * @return self
     */
    public function setBindPassword($bindPassword)
    {
        $this->bindPassword = $bindPassword;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getBindPassword()
    {
        return $this->bindPassword;
    }

    /**
     * @param mixed $filter
     *
     * @return self
     */
    public function setFilter($filter)
    {
        $this->filter = $filter;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getFilter()
    {
        return $this->filter;
    }

    /**
     * @param mixed $hostname
     *
     * @return self
     */
    public function setHostname($hostname)
    {
        $this->hostname = $hostname;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getHostname()
    {
        return $this->hostname;
    }

    /**
     * @param mixed $port
     *
     * @return self
     */
    public function setPort($port)
    {
        $this->port = $port;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getPort()
    {
        return $this->port;
    }

    /**
     * @param mixed $security
     *
     * @return self
     */
    public function setSecurity($security)
    {
        $this->security = $security;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getSecurity()
    {
        return $this->security;
    }
}
