<?php
namespace Topalis\Dashboard\System;

/**
 * gather information about memory
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class Memory extends SystemInformation
{
    protected $memTotal = 0;
    protected $memFree = 0;
    protected $swapTotal = 0;
    protected $swapFree = 0;

    public function __construct()
    {
        $mem = $this->readFile("/proc/meminfo");
        $lines = preg_split("/\n/", $mem, -1, PREG_SPLIT_NO_EMPTY);
        foreach ($lines as $line) {
            if (preg_match("=^MemTotal:\s+(.*?)kB=i", $line, $matches)) {
                $this->memTotal = $matches[1] * 1024;
            }
            if (preg_match("=^MemFree:\s+(.*?)kB=i", $line, $matches)) {
                $this->memFree = $matches[1] * 1024;
            }
            if (preg_match("=^SwapTotal:\s+(.*?)kB=i", $line, $matches)) {
                $this->swapTotal = $matches[1] * 1024;
            }
            if (preg_match("=^SwapFree:\s+(.*?)kB=i", $line, $matches)) {
                $this->swapFree = $matches[1] * 1024;
            }
        }
    }

    public function getFree()
    {
        return $this->memFree;
    }

    public function getTotal()
    {
        return $this->memTotal;
    }

    public function getUsed()
    {
        return $this->memTotal - $this->memFree;
    }

    public function getPercentage()
    {
        return number_format($this->getUsed() / $this->getTotal() * 100, 2, ".", "");
    }

    public function getSwapTotal()
    {
        return $this->swapTotal;
    }

    public function getSwapFree()
    {
        return $this->swapFree;
    }

    public function getSwapUsed()
    {
        return $this->swapTotal - $this->swapFree;
    }
}
