<?php
namespace Topalis\Database\Query;

use Doctrine\ORM\Query\AST\Functions\FunctionNode;
use Doctrine\ORM\Query\Lexer;

/**
 * Usage: COUNTIF(expr1)
 * COUNTIF() returns the COUNT of rows where expr1 is true by internally
 * constructing a case statment so that the expression
 * COUNT(IF(expr, 1, NULL) is evaluated.
 *
 * @author  Florian Pfitzer<pfitzer@topalis.com>
 */
class CountIf extends FunctionNode
{
    private $expr;

    public function parse(\Doctrine\ORM\Query\Parser $parser)
    {
        $parser->match(Lexer::T_IDENTIFIER);
        $parser->match(Lexer::T_OPEN_PARENTHESIS);
        $this->expr = $parser->ConditionalExpression();
        $parser->match(Lexer::T_CLOSE_PARENTHESIS);
    }

    public function getSql(\Doctrine\ORM\Query\SqlWalker $sqlWalker)
    {
        return sprintf(
            "COUNT(IF(%s, 1, NULL))",
            $sqlWalker->walkConditionalExpression($this->expr)
        );
    }
}
