<?php
namespace Topalis\Database\Query;

use Doctrine\ORM\Query\AST\Functions\FunctionNode;
use Doctrine\ORM\Query\Lexer;

/**
 * DIVFunction ::= "DIV" "(" ArithmeticPrimary "," ArithmeticPrimary ")"
 * DIV(N, M) is transformed to: N DIV M
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class DIV extends FunctionNode
{
    private $n;
    private $m;

    public function parse(\Doctrine\ORM\Query\Parser $parser)
    {
        $parser->match(Lexer::T_IDENTIFIER);
        $parser->match(Lexer::T_OPEN_PARENTHESIS);
        $this->n = $parser->ArithmeticExpression();
        $parser->match(Lexer::T_COMMA);
        $this->m = $parser->ArithmeticExpression();
        $parser->match(Lexer::T_CLOSE_PARENTHESIS);
    }

    public function getSql(\Doctrine\ORM\Query\SqlWalker $sqlWalker)
    {
        return $this->n->dispatch($sqlWalker) . " DIV " . $this->m->dispatch($sqlWalker);
    }
}
