<?php
namespace Topalis\Device\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputArgument;

/**
 * This command requests device information for all or specified devices.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class RequestDeviceInfoCommand extends ContainerAwareCommand
{
    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->getContainer()->get("device.entity.manager.device");
    }

    /**
     * @return \Topalis\Device\MDM\MdmFacadeInterface
     */
    private function getMdmFacade()
    {
        return $this->getContainer()->get("device.mdm.facade");
    }

    protected function configure()
    {
        $this
        ->setName("device:requestinfo")
        ->setDescription("Request device information for all or the specified devices.")
        ->setHelp("Request device information for all or the specified devices.")
        ->addOption("id", null, InputOption::VALUE_IS_ARRAY | InputArgument::OPTIONAL,
            "The device id to request device information for.");
    }

    protected function execute(\Symfony\Component\Console\Input\InputInterface $input,
        \Symfony\Component\Console\Output\OutputInterface $output)
    {
        $ids = $input->getOption("id");

        $mdmFacade = $this->getMdmFacade();
        if (count($ids) > 0) {
            $em = $this->getDeviceEntityManager();
            foreach ($ids as $id) {
                $device = $em->find($id);
                if ($device === null) {
                    $output->writeln("A device with the id '" . $id . "' does not exist.");
                } else if (!$device->isManaged()) {
                    $output->writeln("Cannot request device information from the device with the id '" . $id
                        . "': the device is not a managed device.");
                } else if (!$device->isActive()) {
                    $output->writeln("Cannot request device information from the device with the id '" . $id
                        . "': the device is not active, anymore.");
                } else {
                    $mdmFacade->requestDeviceInfo($device);
                }
            }
        } else {
            $mdmFacade->requestDeviceInfos();
        }
    }
}
