<?php
namespace Topalis\Device\DependencyInjection\Compiler;

use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;

/**
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class DeviceTypePass implements CompilerPassInterface
{
    public function process(ContainerBuilder $container)
    {
        if (!$container->has("device.type.factory")) {
            return;
        }

        // Builds an array with service IDs as values and tag aliases as keys
        $types = new \SplPriorityQueue();
        foreach ($container->findTaggedServiceIds('topalis.device.type') as $serviceId => $tag) {
            $id = $tag[0]['id'];
            $alias = isset($tag[0]['alias'])
                ? $tag[0]['alias']
                : $serviceId;
            $order = isset($tag[0]['order']) ? $tag[0]['order'] : 0;

            $types->insert(array("id" => $id, "alias" => $alias, "serviceId" => $serviceId), $order);
        }

        $typeList = array();
        foreach ($types as $data) {
            $typeList[$data['id']] = array("alias" => $data['alias'], "serviceId" => $data['serviceId']);
        }

        $container->getDefinition("device.type.factory")->replaceArgument(1, $typeList);
    }
}
