<?php
namespace Topalis\Device;

use Topalis\Device\Entity\Device;

/**
 * Class containing helper functions regarding devices.
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class DeviceUtils
{
    /**
     * Converts the device information of the specified device into and array.
     * The corresponding user name is included.
     *
     * @param Device $device The device to get the device info array.
     * @return array the device information
     */
    public static function parseDeviceInformation(Device $device)
    {
        static $keys = array(
            "DeviceName",
            "Vendor",
            "ProductName",
            "ModelName",
            "Model",
            "OSVersion",
            "BuildVersion",
            "SerialNumber",
            "IMEI",
            "UDID",
            "PhoneNumber",
            "SIMCarrierNetwork",
            "WiFiMAC",
            "BluetoothMAC",
            "IsRoaming",
            "DeviceCapacity",
            "AvailableDeviceCapacity",
            "BatteryLevel",
            "CellularTechnology",
            "MEID",
            "ModemFirmwareVersion",
            "ICCID",
            "CurrentCarrierNetwork",
            "SIMCarrierNetwork",
            "SubscriberCarrierNetwork",
            "CarrierSettingsVersion",
            "VoiceRoamingEnabled",
            "DataRoamingEnabled",
            "SubscriberMCC",
            "SubscriberMNC",
            "CurrentMCC",
            "CurrentMNC",
            // Android
            "WiFiIp",
            "WiFiSSID",
            "ExternalCapacity",
            "AvailableExternalCapacity",
            "VoiceMailNumber",
            "AdbEnabled",
            "InstallNonMarketApps",
            "AppVersion",
            "PasscodeSufficient",
            "DeviceEncrypted",
        );

        $rv = array(
            "User" => $device->getUser() ? $device->getUser()->getLogin() : null,
            "ConfigGroup" => $device->getConfigGroup() ? $device->getConfigGroup()->getName() : null
        );
        $deviceInfo = $device->getType()->getDeviceInfo($device);
        if ($deviceInfo !== null) {
            foreach ($keys as $key) {
                $value = $deviceInfo->get($key);
                if (is_array($value)) {
                    $value = json_encode($value);
                }
                $rv[$key] = (string) $value;
            }
        } else {
            foreach ($keys as $key) {
                $rv[$key] = null;
            }
        }

        return $rv;
    }
}
