<?php

namespace Topalis\Device\Entity;

use Doctrine\ORM\Mapping as ORM;

/**
 * AccessCertStat
 *
 * @ORM\Table(name="access_cert_stat", indexes={@ORM\Index(name="idx_access_cert_stat_traffic", columns={"traffic"})})
 * @ORM\Entity
 */
class AccessCertStat
{
    /**
     * @var \DateTime
     *
     * @ORM\Column(name="last_tstamp", type="datetime", nullable=false,
     *     options={"version"=true, "comment"="Timestamp of last access"})
     */
    protected $lastTstamp;

    /**
     * @var string
     *
     * @ORM\Column(name="last_ip", type="string", length=50, nullable=true,
     *     options={"comment"="Remote IP of last access"})
     */
    protected $lastIp;

    /**
     * @var integer
     *
     * @ORM\Column(name="traffic", type="bigint", nullable=false,
     *     options={"unsigned"=true, "comment"="Total traffic in bytes"})
     */
    protected $traffic;

    /**
     * @var \Topalis\Certificate\Entity\ClientCertificate
     *
     * @ORM\ManyToOne(targetEntity="Topalis\Certificate\Entity\ClientCertificate")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="cert_id", referencedColumnName="id", nullable=false)
     * })
     * @ORM\Id
     */
    protected $cert;

    /**
     * @var \Topalis\Device\Entity\Device
     *
     * @ORM\ManyToOne(targetEntity="Topalis\Device\Entity\Device")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="device_id", referencedColumnName="id", nullable=false)
     * })
     */
    protected $device;

    /**
     * Set lastTstamp
     *
     * @param \DateTime $lastTstamp
     * @return AccessCertStat
     */
    public function setLastTstamp(\DateTime $lastTstamp)
    {
        $this->lastTstamp = $lastTstamp;

        return $this;
    }

    /**
     * Get lastTstamp
     *
     * @return \DateTime
     */
    public function getLastTstamp()
    {
        return $this->lastTstamp;
    }

    /**
     * Set lastIp
     *
     * @param string $lastIp
     * @return AccessCertStat
     */
    public function setLastIp($lastIp)
    {
        $this->lastIp = $lastIp;

        return $this;
    }

    /**
     * Get lastIp
     *
     * @return string
     */
    public function getLastIp()
    {
        return $this->lastIp;
    }

    /**
     * Set traffic
     *
     * @param integer $traffic
     * @return AccessCertStat
     */
    public function setTraffic($traffic)
    {
        $this->traffic = $traffic;

        return $this;
    }

    /**
     * Get traffic
     *
     * @return integer
     */
    public function getTraffic()
    {
        return $this->traffic;
    }

    /**
     * Set cert
     *
     * @param \Topalis\Certificate\Entity\ClientCertificate $cert
     * @return AccessCertStat
     */
    public function setCert(\Topalis\Certificate\Entity\ClientCertificate $cert = null)
    {
        $this->cert = $cert;

        return $this;
    }

    /**
     * Get cert
     *
     * @return \Topalis\Certificate\Entity\ClientCertificate
     */
    public function getCert()
    {
        return $this->cert;
    }

    /**
     * Set device
     *
     * @param \Topalis\Device\Entity\Device $device
     * @return AccessCertStat
     */
    public function setDevice(\Topalis\Device\Entity\Device $device = null)
    {
        $this->device = $device;

        return $this;
    }

    /**
     * Get device
     *
     * @return \Topalis\Device\Entity\Device
     */
    public function getDevice()
    {
        return $this->device;
    }
}
