<?php
namespace Topalis\Device\EventListener;

use Doctrine\ORM\Event\PreUpdateEventArgs;
use Topalis\Device\Entity\Device;

/**
 * This listener asserts that a device is not moved from a non-Exchange group
 * into an Exchange group, and vice versa.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class DeviceGroupListener
{
    public function preUpdate(PreUpdateEventArgs $eventArgs)
    {
        $device = $eventArgs->getEntity();
        if (!$device instanceof Device) {
            return;
        }

        if ($device->isDeleted()) {
            return;
        }

        $changeSet = $eventArgs->getEntityChangeSet();
        if (!isset($changeSet['configGroup'])) {
            return;
        }

        if ($changeSet['configGroup'][0]->isNoExchange() != $changeSet['configGroup'][1]->isNoExchange()) {
            throw new \Exception(
                "A device in a non-Exchange group cannot be moved to an Exchange group, and vice versa");
        }
    }
}
