<?php
namespace Topalis\Device\Export;

use PHPExcel;
use PHPExcel_IOFactory;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Symfony\Component\HttpFoundation\StreamedResponse;
use Topalis\Device\DeviceUtils;
use Topalis\Device\Entity\Device;

/**
 * Export devices as excel file using phpexcel (http://phpexcel.codeplex.com/)
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class ExcelDeviceExporter implements DeviceExporter
{
    /**
     * @var string Excel Version (2007 or 5 (BIFF) format)
     */
    protected $format;

    public function __construct($format = 2007)
    {
        $this->format = $format;
    }
    /**
     * @param Device[] $devices
     *
     * @return Response
     */
    public function export(array $devices)
    {
        $excel = new PHPExcel();
        $sheet = $excel->getActiveSheet();
        $row = 1;
        foreach ($this->getData($devices) as $info) {
            $column = 0;
            foreach ($info as $value) {
                $sheet->getCellByColumnAndRow($column, $row)
                    ->setValueExplicit($value, \PHPExcel_Cell_DataType::TYPE_STRING);
                $column++;
            }
            $row++;
        }

        switch ($this->format) {
            case "csv":
                $format = "CSV";
                $mimeType = "text/comma-separated-values";
                $fileName = "auralis-devices.csv";
                break;
            case "5":
                $format = "Excel5";
                $mimeType = "application/vnd.ms-excel";
                $fileName = "auralis-devices.xls";
                break;
            case "2007":
            default:
                $format = "Excel2007";
                $mimeType = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet";
                $fileName = "auralis-devices.xlsx";
                break;
        }
        $column = $sheet->getHighestColumn();
        $sheet->getStyle("A1:".$column."1")->getFont()->setBold(true);

        // autosize
        $columnIndex = \PHPExcel_Cell::columnIndexFromString($column);
        for ($i = 1; $i <= $columnIndex; $i++) {
            $sheet->getColumnDimensionByColumn($i)->setAutoSize(true);
        }
        $sheet->calculateColumnWidths();
        $excel->getDefaultStyle()->getNumberFormat()->setFormatCode(\PHPExcel_Style_NumberFormat::FORMAT_TEXT);

        $sheet->freezePane("A2");

        $writer = PHPExcel_IOFactory::createWriter($excel, $format);
        $response = new StreamedResponse();
        $response->setStatusCode(200);
        $response->headers->set("Content-Type", $mimeType);
        $disposition = $response->headers->makeDisposition(
            ResponseHeaderBag::DISPOSITION_ATTACHMENT,
            $fileName
        );
        $response->headers->set("Content-Disposition", $disposition);
        $response->setCallback(function () use ($writer) {
            $writer->save("php://output");
        });

        return $response;
    }

    protected function getData(array $devices)
    {
        if (empty($devices)) {
            return array();
        }
        $data = array(array_keys(DeviceUtils::parseDeviceInformation(reset($devices)))); // header
        foreach ($devices as $device) {
            $data[] = DeviceUtils::parseDeviceInformation($device);
        }

        return $data;
    }
}
