<?php
namespace Topalis\Device\MDM;

use Symfony\Component\DependencyInjection\ContainerAware;

/**
 * Default MDM facade that will call device specific MDM facades.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class DefaultMdmFacade extends ContainerAware implements MdmFacadeInterface
{
    /**
     * @return MdmFacadeInterface[]
     */
    protected function getAllMdmFacades()
    {
        return $this->container->get("device.mdm.facade.container")->getAllMdmFacades();
    }

    /**
     * @param \Topalis\Device\Entity\Device $device The device to get the MDM
     *    facade for.
     * @return MdmFacadeInterface
     */
    protected function getMdmFacadeForDevice(\Topalis\Device\Entity\Device $device)
    {
        return $this->container->get("device.mdm.facade.container")->getMdmFacade($device);
    }

    public function getCommands(\Topalis\Device\Entity\Device $device, $limit = 0, array $orderBy = null)
    {
        return $this->getMdmFacadeForDevice($device)->getCommands($device, $limit, $orderBy);
    }

    public function installMailAccount(\Topalis\Device\Entity\MailAccount $mailAccount)
    {
        $this->getMdmFacadeForDevice($mailAccount->getDevice())->installMailAccount($mailAccount);
    }

    public function removeMailAccount(\Topalis\Device\Entity\MailAccount $mailAccount)
    {
        $this->getMdmFacadeForDevice($mailAccount->getDevice())->removeMailAccount($mailAccount);
    }

    public function requestDeviceInfo(\Topalis\Device\Entity\Device $device)
    {
        $this->getMdmFacadeForDevice($device)->requestDeviceInfo($device);
    }

    public function requestDeviceInfos()
    {
        foreach ($this->getAllMdmFacades() as $mdmFacade) {
            $mdmFacade->requestDeviceInfos();
        }
    }

    public function resendCommand(\Topalis\Device\Entity\Device $device, $commandId)
    {
        $this->getMdmFacadeForDevice($device)->resendCommand($device, $commandId);
    }

    public function lockDevice(\Topalis\Device\Entity\Device $device, $message = null, $phoneNumber = null)
    {
        $this->getMdmFacadeForDevice($device)->lockDevice($device, $message, $phoneNumber);
    }

    public function eraseDevice(\Topalis\Device\Entity\Device $device)
    {
        $this->getMdmFacadeForDevice($device)->eraseDevice($device);
    }

    public function wipeDevice(\Topalis\Device\Entity\Device $device)
    {
        $this->getMdmFacadeForDevice($device)->wipeDevice($device);
    }

    public function updateConfigGroup(\Topalis\ConfigGroup\Entity\ConfigGroup $configGroup,
        \Doctrine\ORM\UnitOfWork $uow)
    {
        $changeSet = $uow->getEntityChangeSet($configGroup);
        if (count($changeSet) == 1 && isset($changeSet['name'])) {
            // only name changed, no update needed
            return;
        }

        foreach ($this->getAllMdmFacades() as $mdmFacade) {
            $mdmFacade->updateConfigGroup($configGroup, $uow);
        }
    }

    public function updateDevice(\Topalis\Device\Entity\Device $device, \Doctrine\ORM\UnitOfWork $uow)
    {
        $this->getMdmFacadeForDevice($device)->updateDevice($device, $uow);
    }

    public function addWebClip(\Topalis\WebClip\Entity\WebClipInterface $webClip)
    {
        foreach ($this->getAllMdmFacades() as $mdmFacade) {
            $mdmFacade->addWebClip($webClip);
        }
    }

    public function updateWebClip(
        \Topalis\WebClip\Entity\WebClipInterface $webClip,
        array $removedGroups, array $addedGroups, \Doctrine\ORM\UnitOfWork $uow)
    {
        foreach ($this->getAllMdmFacades() as $mdmFacade) {
            $mdmFacade->updateWebClip($webClip, $removedGroups, $addedGroups, $uow);
        }
    }

    public function removeWebClip(\Topalis\WebClip\Entity\WebClipInterface $webClip)
    {
        foreach ($this->getAllMdmFacades() as $mdmFacade) {
            $mdmFacade->removeWebClip($webClip);
        }
    }

    public function addWifi(\Topalis\Wifi\Entity\WifiConfig $wifiConfig)
    {
        foreach ($this->getAllMdmFacades() as $mdmFacade) {
            $mdmFacade->addWifi($wifiConfig);
        }
    }

    public function updateWifi(\Topalis\Wifi\Entity\WifiConfig $wifiConfig, array $removedGroups,
        array $addedGroups, array $changeSet)
    {
        foreach ($this->getAllMdmFacades() as $mdmFacade) {
            $mdmFacade->updateWifi($wifiConfig, $removedGroups, $addedGroups, $changeSet);
        }
    }

    public function removeWifi(\Topalis\Wifi\Entity\WifiConfig $wifiConfig)
    {
        foreach ($this->getAllMdmFacades() as $mdmFacade) {
            $mdmFacade->removeWifi($wifiConfig);
        }
    }
}
