<?php
namespace Topalis\Device\MDM;

use Symfony\Component\DependencyInjection\ContainerAware;

/**
 * Class that contains the different MDM facades for the different devices OSs.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class MdmFacadeContainer extends ContainerAware
{
    private $mdmFacades = array();

    private $mdmFacadesUnique = array();

    /**
     *
     * @param \Topalis\Device\Entity\Device $device
     * @return MdmFacadeInterface
     */
    public function getMdmFacade(\Topalis\Device\Entity\Device $device)
    {
        return isset($this->mdmFacades[$device->getOs()]) ? $this->mdmFacades[$device->getOs()] : new DummyMdmFacade();
    }

    /**
     * @return array Array of MDM facades.
     */
    public function getAllMdmFacades()
    {
        return $this->mdmFacadesUnique;
    }

    public function registerMdmFacade($os, MdmFacadeInterface $mdmFacade)
    {
        $this->mdmFacades[$os] = $mdmFacade;
        if (!in_array($mdmFacade, $this->mdmFacadesUnique)) {
            $this->mdmFacadesUnique[] = $mdmFacade;
        }
    }
}
