<?php
namespace Topalis\Device\MDM;

/**
 * Interface for a facade for MDM.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
interface MdmFacadeInterface
{
    /**
     * The MDM commands for the specified device.
     *
     * @param \Topalis\Device\Entity\Device $device
     * @param int $limit
     * @param array $orderBy
     * @return array Array of \Topalis\Device\MDM\MdmCommandInterface objects.
     */
    public function getCommands(\Topalis\Device\Entity\Device $device, $limit = 0, array $orderBy = null);

    public function installMailAccount(\Topalis\Device\Entity\MailAccount $mailAccount);

    public function removeMailAccount(\Topalis\Device\Entity\MailAccount $mailAccount);

    public function requestDeviceInfo(\Topalis\Device\Entity\Device $device);

    public function requestDeviceInfos();

    /**
     * Sends the MDM command with the specified id again. A new command will be
     * created so that the old command stays in the history.
     *
     * @param \Topalis\Device\Entity\Device $device
     * @param int $commandId
     */
    public function resendCommand(\Topalis\Device\Entity\Device $device, $commandId);

    /**
     * @param \Topalis\Device\Entity\Device $device      The device to be locked.
     * @param string                        $message     Message that shall be shown to the user.
     * @param string                        $phoneNumber Phone number that shall be shown to the user.
     */
    public function lockDevice(\Topalis\Device\Entity\Device $device, $message = null, $phoneNumber = null);

    public function eraseDevice(\Topalis\Device\Entity\Device $device);

    public function wipeDevice(\Topalis\Device\Entity\Device $device);

    public function updateConfigGroup(\Topalis\ConfigGroup\Entity\ConfigGroup $configGroup,
        \Doctrine\ORM\UnitOfWork $uow);

    public function updateDevice(\Topalis\Device\Entity\Device $device, \Doctrine\ORM\UnitOfWork $uow);

    public function addWebClip(\Topalis\WebClip\Entity\WebClipInterface $webClip);

    public function updateWebClip(
        \Topalis\WebClip\Entity\WebClipInterface $webClip,
        array $removedGroups,
        array $addedGroups,
        \Doctrine\ORM\UnitOfWork $uow
    );

    public function removeWebClip(\Topalis\WebClip\Entity\WebClipInterface $webClip);

    public function addWifi(\Topalis\Wifi\Entity\WifiConfig $wifiConfig);

    public function updateWifi(\Topalis\Wifi\Entity\WifiConfig $wifiConfig, array $removedGroups,
        array $addedGroups, array $changeSet);

    public function removeWifi(\Topalis\Wifi\Entity\WifiConfig $wifiConfig);
}
