<?php
namespace Topalis\Device\Type;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Topalis\Device\Entity\Device;

/**
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class DeviceTypeFactory
{
    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    /**
     * @var array
     */
    private $types = array();

    /**
     *
     * @var array
     */
    private $idMap = array();

    public function __construct(ContainerInterface $container, array $types)
    {
        $this->container = $container;

        foreach ($types as $id => $data) {
            $this->types[$data['alias']] = $data['serviceId'];
            $this->idMap[$id] = $data['alias'];
        }
    }

    public function hasType($name)
    {
        return isset($this->types[$name]);
    }

    public function hasTypeById($id)
    {
        return isset($this->idMap[$id]);
    }

    /**
     * @param string $name
     *
     * @return DeviceTypeInterface
     * @throws \InvalidArgumentException
     */
    public function getType($name)
    {
        if (!$this->hasType($name)) {
            throw new \InvalidArgumentException(sprintf("unknown device type %s", $name));
        }

        $type = $this->types[$name];
        if (is_string($type)) {
            // service id
            $type = $this->container->get($type);
            $this->types[$name] = $type; // cache for later usage
        }
        return $type;
    }

    public function getTypeById($id)
    {
        if (!$this->hasTypeById($id)) {
            throw new \InvalidArgumentException(sprintf("unknown device type id %s", $id));
        }

        return $this->getType($this->idMap[$id]);
    }

    /**
     * @return DeviceTypeInterface[]
     */
    public function getTypes()
    {
        $types = array();
        foreach (array_keys($this->types) as $type) {
            $types[] = $this->getType($type);
        }

        return $types;
    }
}
