<?php
namespace Topalis\Form\DependencyInjection\Compiler;

use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;

/**
 * Use StorageDecorator on all oneup filesystems
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class UploaderPass implements CompilerPassInterface
{
    public function process(ContainerBuilder $container)
    {
        foreach ($container->getParameter("oneup_uploader.controllers") as $name => $controller) {
            $controllerService = $container->getDefinition($controller[0]);
            // get storage service
            $storage = $controllerService->getArgument(1);

            $decoratorClass = $container->getParameter("form.upload.storage.decorator.class");
            $storageDecorator = $container->register("oneup_uploader.storage_decorator." . $name, $decoratorClass)
                ->addArgument($storage)
            ;
            $controllerService->replaceArgument(1, $storageDecorator);
        }
    }
}
