<?php
namespace Topalis\Form\Extension\EventListener;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\Form\Exception\UnexpectedTypeException;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;

/**
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class CollectionLimitListener implements EventSubscriberInterface
{
    /**
     * @var integer
     */
    private $limit;

    public function __construct($limit)
    {
        $this->limit = $limit;
    }

    public function preSubmit(FormEvent $event)
    {
        $data = $event->getData();
        if (null === $data) {
            return;
        }

        if (!is_array($data)) {
            throw new UnexpectedTypeException($data, "array");
        }

        if (count($data) > $this->limit) {
            $data = array(reset($data));
            $event->setData($data);
        }
    }

    public static function getSubscribedEvents()
    {
        return array(
            FormEvents::PRE_SUBMIT => "preSubmit",
            FormEvents::PRE_SET_DATA => "preSubmit",
        );
    }
}
