<?php
namespace Topalis\Form\Extension\Transformer;

use Oneup\UploaderBundle\Uploader\Storage\FilesystemOrphanageStorage;
use Oneup\UploaderBundle\Uploader\Storage\OrphanageStorageInterface;
use Symfony\Component\Finder\SplFileInfo;
use Symfony\Component\Form\DataTransformerInterface;
use Symfony\Component\HttpFoundation\File\File;
use Topalis\Util\StringUtil;

/**
 * Transform uploaded files into
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class UploaderTransformer implements DataTransformerInterface
{
    /**
     * @var OrphanageStorageInterface|FilesystemOrphanageStorage
     */
    private $storage;

    /**
     * @var boolean
     */
    private $multiple;

    /**
     * @var string
     */
    private $uploadDir;

    public function __construct(OrphanageStorageInterface $storage, $uploadDir, $multiple)
    {
        $this->storage = $storage;
        $this->multiple = $multiple;
        $this->uploadDir = $uploadDir;
    }

    public function transform($value)
    {
        if (null === $value) {
            return;
        }
        $value = (array) $value;
        $files = array();
        foreach ($value as $file) {
            if (is_string($file) && StringUtil::startsWith(realpath($file), realpath($this->uploadDir))) {
                $files[] = new File($file);
            } else if ($file instanceof File) {
                $files[] = $file;
            }
        }

        return $files;
    }

    public function reverseTransform($value)
    {
        if (null === $value) {
            return;
        }

        /** @var SplFileInfo[] $files */
        $files = $this->storage->getFiles();
        $uploads = array();
        $uploadedFiles = array();

        $value = (array) $value;
        // old files
        foreach ($value as $file) {
            if (is_string($file) && realpath($file) && realpath($this->uploadDir)
                && StringUtil::startsWith(realpath($file), realpath($this->uploadDir))) {
                $uploadedFiles[] = new File($file);
            } else if ($file instanceof File) {
                $uploadedFiles[] = $file;
            }
        }

        // new files
        foreach ($files as $file) {
            if (in_array($file->getFilename(), $value)) {
                $uploads[] = $file;
            }
        }

        if (!empty($uploads)) {
            if (!$this->multiple) {
                $uploads = array(reset($uploads));
            }
            $uploadedFiles = array_merge($uploadedFiles, $this->storage->uploadFiles($uploads));
        }
        if (!$this->multiple) {
            return reset($uploadedFiles);
        }

        return $uploadedFiles;
    }
}
