<?php
namespace Topalis\Form\Extension\Type;

use Oneup\UploaderBundle\Uploader\Orphanage\OrphanageManager;
use Oneup\UploaderBundle\Uploader\Storage\OrphanageStorageInterface;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Topalis\Form\Extension\Transformer\UploaderTransformer;

/**
 * Use jquery blueimp fileupload for uploading files via ajax
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class JqueryFileUploadType extends AbstractType
{
    /**
     * @var OrphanageManager
     */
    private $orphanageManager;

    /**
     * @var string
     */
    private $kernelDir;

    public function __construct(OrphanageManager $orphanageManager, $kernelDir)
    {
        $this->orphanageManager = $orphanageManager;
        $this->kernelDir = $kernelDir;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        /** @var OrphanageStorageInterface $storage */
        $storage = $this->orphanageManager->get($options['endpoint']);
        $uploadDir = sprintf("%s/../web/uploads/%s", $this->kernelDir, $options['endpoint']);

        $builder->addModelTransformer(new UploaderTransformer($storage, $uploadDir, $options['multiple']));
    }

    public function buildView(FormView $view, FormInterface $form, array $options)
    {
        $form; // unused

        $view->vars = array_replace($view->vars, array(
            "endpoint" => $options['endpoint'],
            "multiple" => $options['multiple'],
        ));
    }

    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(array(
            "compound" => false,
            "multiple" => false,
            "data_class" => null,
        ));
        $resolver->setRequired(array("endpoint"));
    }

    public function getParent()
    {
        return "form";
    }

    public function getName()
    {
        return "jquery_file_upload";
    }
}
