<?php
namespace Topalis\Form\Validator\Constraint;

use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Exception\UnexpectedTypeException;

/**
 * Validate a list of ips
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class IPListValidator extends ConstraintValidator
{
    public function validate($value, Constraint $constraint)
    {
        /** @var $constraint IPList */
        if (null === $value || '' === $value) {
            return;
        }

        if (!is_scalar($value) && !(is_object($value) && method_exists($value, '__toString'))) {
            throw new UnexpectedTypeException($value, 'string');
        }

        $ips = preg_split('/\s*,\s*/', trim($value), null, PREG_SPLIT_NO_EMPTY);
        foreach ($ips as $ip) {
            list($ip, $mask) = explode("/", $ip) + array(null, null);

            if (!filter_var($ip, FILTER_VALIDATE_IP)) {
                $this->context->addViolation($constraint->message, array("%ip%" => $ip), $value);
            }

            if (empty($mask)) {
                return;
            }

            $imask = intval($mask);
            if ($imask < 0 || $imask > 128
                || (filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4) && $imask > 32)
                || $mask != (string) $imask) {
                $this->context->addViolation($constraint->ipMaskMessage, array("%ip%" => $ip), $value);
            }
        }
    }
}
