<?php
namespace Topalis\Form\Validator\Constraint;

use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Exception\UnexpectedTypeException;

/**
 * Validate Passwords
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class PasswordValidator extends ConstraintValidator
{
    public function validate($value, Constraint $constraint)
    {
        /** @var $constraint Password */

        if (null === $value || '' === $value) {
            return;
        }

        if (!is_scalar($value) && !(is_object($value) && method_exists($value, '__toString'))) {
            throw new UnexpectedTypeException($value, 'string');
        }

        $stringValue = (string) $value;
        if (strlen($stringValue) < $constraint->length) {
            $this->context->addViolation($constraint->lengthMessage, array("%length%" => $constraint->length), $value);
        }

        if ($constraint->capital && !preg_match('/[A-ZAÖÜ]/', $stringValue)) {
            $this->context->addViolation($constraint->capitalMessage, array(), $value);
        }

        if ($constraint->lowercase && !preg_match('/[a-zäöüß]/', $stringValue)) {
            $this->context->addViolation($constraint->lowercaseMessage, array(), $value);
        }

        if ($constraint->digit && !preg_match('/\d/', $stringValue)) {
            $this->context->addViolation($constraint->digitMessage, array(), $value);
        }

        if ($constraint->specialChar
            && !preg_match('/[\!\"\#\$\%\&`\(\)\*\+\/,\.:;\<=\>\?\@\[\]\\\^\_\']/', $stringValue)) {
            $this->context->addViolation($constraint->specialCharMessage, array(), $value);
        }
    }

}
