<?php
namespace Topalis\Gcm\Exception;

/**
 * Thrown if the GCM server returns an HTTP status code that doesn't match 2xx.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class GcmHttpException extends GcmException
{
    /**
     *
     * @var \Buzz\Message\Response
     */
    private $response;

    public function __construct(\Buzz\Message\Response $response)
    {
        $status = $response->getStatusCode();
        $message = $response->getReasonPhrase();

        if (strlen($message) == 0) {
            if ($status == 400) {
                $message = "Invalid JSON";
            } else if ($status == 401) {
                $message = "Authentication failed";
            } else if ($status >= 500 && $status <= 599) {
                $message = "Internal server error";
            } else {
                $message = "Unexpected status code '" . $status . "'";
            }
        }

        parent::__construct($message, $status);
    }

    public function getContent()
    {
        return $this->response->getContent();
    }

    public function getHeader($name, $glue = "\r\n")
    {
        return $this->response->getHeader($name, $glue);
    }

    public function getHeaders()
    {
        return $this->response->getHeaders();
    }

    public function getStatusCode()
    {
        return $this->response->getStatusCode();
    }
}
