<?php
namespace Topalis\Generator\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Topalis\Profile\Annotation\Payload\Key;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class GeneratePayloadCommand extends ContainerAwareCommand
{
    protected function configure()
    {
        $this
            ->setName("generate:payload")
            ->setDescription("Adds payload methods from annotation config")
            ->addArgument("class", InputArgument::REQUIRED, "class name for payload")
        ;
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $output; // unused

        $class = $input->getArgument("class");
        $reader = $this->getContainer()->get("annotation_reader");
        $reflection = new \ReflectionClass($class);
        $annotations = $reader->getClassAnnotations($reflection);

        foreach ($annotations as $annotation) {
            if (!$annotation instanceof Key) {
                continue;
            }
            $propertyName = $annotation->getName();
            $methodName = ucfirst($annotation->getName());
            $varName = lcfirst($annotation->getName());
            $setter = "set".$methodName;
            $getter = "get".$methodName;
            if ("boolean" === $annotation->getType()) {
                $getter = "is".$methodName;
            }

            if (!$reflection->hasMethod($setter)) {
                $methodBody = array(
                    "public function ".$setter."(\$".$varName.")",
                    "{",
                    "    \$this->setValue(\"".$propertyName."\", \$".$varName.", self::TYPE_".strtoupper($annotation->getType()).");",
                    "}",
                );
                echo implode("\n", $methodBody)."\n\n";
            }
            if (!$reflection->hasMethod($getter)) {
                $methodBody = array(
                    "public function ".$getter."()",
                    "{",
                    "    return \$this->getValue(\"".$propertyName."\"".(null !== $annotation->getDefault() ? ", ".$annotation->getDefault() : '').");",
                    "}",
                );
                echo implode("\n", $methodBody)."\n\n";
            }
        }
    }

}
