<?php
namespace Topalis\Kernel\Assets\Asset;

use Assetic\Asset\FileAsset;
use Assetic\Util\VarUtils;

/**
 * managing dynamic imports
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class DynamicAsset extends FileAsset
{

    protected $source;

    public function __construct(
        $source,
        $filters = array(),
        $sourceRoot = null,
        $sourcePath = null,
        array $vars = array()
    )
    {
        parent::__construct($source, $filters, $sourceRoot, $sourcePath, $vars);
        $this->source = $source;
    }

    public function getLastModified()
    {
        $source = VarUtils::resolve($this->source, $this->getVars(), $this->getValues());

        if (!is_file($source)) {
            throw new \RuntimeException(sprintf('The source file "%s" does not exist.', $source));
        }

        $times = array(filemtime($source));

        $content = file_get_contents($source);
        $imports = array();

        preg_match_all("=@import(.*?);=si", $content, $imports);

        if (isset($imports[1])) {
            foreach ($imports[1] as $import) {
                $import = trim($import);
                if (substr($import, 0, 1) == '"') {
                    $import = substr($import, 1, -1);
                }
                $path = dirname($source) . "/" . $import;

                $pattern = '/\w+\/\.\.\//';
                while (preg_match($pattern, $path)) {
                    $path = preg_replace($pattern, '', $path);
                }

                if (file_exists($path)) {
                    $times[] = filemtime($path);
                }
            }
        }

        return max($times);

    }

}
