<?php
namespace Topalis\Kernel\EventListener;

use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Topalis\Util\StringUtil;

/**
 * Set route data in session for redirects to previous page.
 * @see http://www.fractalizer.ru/frpost_658/symfony2-how-redirect-user-to-a-previous-page-correctly/
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class RouteListener
{
    /**
     * @var \Symfony\Component\Routing\RouterInterface
     */
    private $router;

    public function __construct(\Symfony\Component\Routing\RouterInterface $router)
    {
        $this->router = $router;
    }

    public function onKernelRequest(GetResponseEvent $event)
    {
        if ($event->getRequestType() !== \Symfony\Component\HttpKernel\HttpKernel::MASTER_REQUEST) {
            return;
        }

        $request = $event->getRequest();
        $session = $request->getSession();

        $routeParams = $this->router->match($request->getPathInfo());
        $routeName = $routeParams['_route'];
        $format = isset($routeParams['_format']) ? $routeParams['_format'] : null;
        if ($routeName[0] == '_' || StringUtil::startsWith($routeName, "fos_") || $routeName == "admin_login"
            || $format == "json") {
            return;
        }

        unset($routeParams['_route']);
        $routeData = array(
            "name" => $routeName,
            "params" => $routeParams
        );

        // Skipping duplicates
        $thisRoute = $session->get("thisRoute");
        if ($thisRoute == $routeData && $session->has("lastRoute")) {
            return;
        }

        if ($thisRoute !== null) {
            $session->set("lastRoute", $thisRoute);
        }
        $session->set("thisRoute", $routeData);
    }
}
