<?php
namespace Topalis\Kernel\Http;

use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Redirect with a delay. For example to use when apache reloads
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class RedirectDelayResponse extends RedirectResponse
{
    // avoid useless method override warning
    // @codingStandardsIgnoreStart
    public function __construct($url, $status = 201, $headers = array())
    {
        parent::__construct($url, $status, $headers);
    }
    // @codingStandardsIgnoreEnd

    /**
     * Sets the redirect target of this response.
     *
     * @param string $url The URL to redirect to
     *
     * @throws \InvalidArgumentException
     * @return RedirectResponse The current response.
     */
    public function setTargetUrl($url)
    {
        if (empty($url)) {
            throw new \InvalidArgumentException('Cannot redirect to an empty URL.');
        }

        $this->targetUrl = $url;

        // the javascript redirect is required for chrome when the server
        // certificate is changed
        $this->setContent(
            sprintf('<!DOCTYPE html>
<html>
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
        <meta http-equiv="refresh" content="2;url=%1$s" />
        <script type="text/javascript">
            setTimeout(function() { location.href="%1$s"; }, 2000);
        </script>
        <title>Redirecting to %1$s</title>
    </head>
    <body>
        Redirecting to <a href="%1$s">%1$s</a>.
    </body>
</html>', htmlspecialchars($url, ENT_QUOTES, 'UTF-8')));

        return $this;
    }

}
