<?php
namespace Topalis\Kernel\Request\ParamConverter;

use Doctrine\ORM\Mapping\MappingException;
use Doctrine\Bundle\DoctrineBundle\Registry;
use Sensio\Bundle\FrameworkExtraBundle\Request\ParamConverter\DoctrineParamConverter;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\ConfigurationInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Find doctrine entities for parameters.
 * e.g.: group_id: Entity: group, attribute id
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class ArrayParamConverter extends DoctrineParamConverter
{
    public function apply(Request $request, ConfigurationInterface $configuration)
    {
        $class = $configuration->getClass();
        $options = $this->getOptions($configuration);
        $paramName = $configuration->getName();
        if (is_object($request->attributes->get($paramName))) {
            return true;
        }
        try {
            $metadata = $this->registry->getManager($options['entity_manager'])
                ->getClassMetadata($configuration->getClass());
        } catch (MappingException $e) {
            return false;
        }

        $criteria = array();
        foreach ($request->attributes->all() as $attribute => $value) {
            if (0 === strpos($attribute, $paramName."_")) {
                $field = substr($attribute, strlen($paramName."_"));
                if ($metadata->hasField($field)) {
                    $criteria[$field] = $value;
                }
            }
        }
        if (empty($criteria)) {
            return false;
        }
        $object = $this->registry->getRepository($class, $options['entity_manager'])->findOneBy($criteria);
        if (!$object) {
            return false;
        }
        $request->attributes->set($configuration->getName(), $object);

        return true;

    }

    /**
     * {@inheritdoc}
     * @see Sensio\Bundle\FrameworkExtraBundle\Request\ParamConverter.DoctrineParamConverter::supports()
     */
    public function supports(ConfigurationInterface $configuration)
    {
        if (null === $this->registry) {
            return false;
        }

        if (null === $configuration->getClass()) {
            return false;
        }

        $options = $this->getOptions($configuration);

        // Doctrine Entity?
        try {
            $this->registry->getManager($options['entity_manager'])->getClassMetadata($configuration->getClass());

            return true;
        } catch (MappingException $e) {
            return false;
        }
    }
}
