<?php
namespace Topalis\Kernel\Routing;

use Symfony\Component\Config\Loader\Loader;
use Symfony\Component\Config\Loader\LoaderResolverInterface;
use Symfony\Component\Routing\RouteCollection;

/**
 * This Loader loads routes from resources that have been added to the loader
 * before.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class RegisteredResourcesLoader extends Loader
{
    private $resources = array();

    /**
     * Adds a resource to the loader.
     *
     * @param mixed $resource A resource
     * @param string|null $type The resource type or null if unknown
     */
    public function addResource($resource, $type)
    {
        $this->resources[] = array("resource" => $resource, "type" => $type);
    }

    public function load($resource, $type = null)
    {
        $collection = new RouteCollection();
        foreach ($this->resources as $resource) {
            $importedRoutes = $this->import($resource['resource'], $resource['type']);
            $collection->addCollection($importedRoutes);
        }

        return $collection;
    }

    public function supports($resource, $type = null)
    {
        return $resource === "." && $type === "registered_resources";
    }
}
