<?php
namespace Topalis\Kernel\Security;

use Symfony\Component\Security\Core\Exception\UnsupportedUserException;
use Symfony\Component\Security\Core\Exception\UsernameNotFoundException;
use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Component\Security\Core\User\UserProviderInterface;
use Topalis\Admin\Entity\Admin;
use Topalis\Admin\Entity\AdminEntityManager;

/**
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class AdminProvider implements UserProviderInterface
{
    /**
     * @var \Topalis\Admin\Entity\AdminEntityManager
     */
    private $manager;
    /**
     * @var \Topalis\Database\MySqlDb
     */
    private $db;

    public function __construct(\Topalis\Database\MySqlDb $db, AdminEntityManager $manager)
    {
        $this->db = $db;
        $this->manager = $manager;
    }

    /**
     * Loads the user for the given username.
     * This method must throw UsernameNotFoundException if the user is not
     * found.
     *
     * @param string $username The username
     *
     * @return UserInterface
     * @see UsernameNotFoundException
     * @throws UsernameNotFoundException if the user is not found
     */
    public function loadUserByUsername($username)
    {
        $user = $this->manager->createQueryBuilder("a")
            ->where("a.level != :monitor")
            ->andWhere("a.login = :username")
            ->setParameter("monitor", Admin::LEVEL_MONITOR)
            ->setParameter("username", $username)
            ->getQuery()->getOneOrNullResult();
        if (null === $user) {
            throw new UsernameNotFoundException();
        }

        return $user;
    }

    public function refreshUser(UserInterface $device)
    {
        if (!$device instanceof Admin) {
            throw new UnsupportedUserException();
        }
        return $this->manager->findById($device->getId());
    }

    /**
     * Whether this provider supports the given user class
     *
     * @param string $class
     *
     * @return boolean
     */
    public function supportsClass($class)
    {
        return $class == "Topalis\Admin\Entity\Admin";
    }
}
