<?php
namespace Topalis\Kernel\Twig\Extension;

use Symfony\Component\Config\FileLocator;

/**
 * @author Florian Pfitzer <florian.pfitzer@to.com>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class HelpExtension extends \Twig_Extension
{
    /**
     * @var \Symfony\Component\HttpFoundation\Request
     */
    private $request;

    /**
     * @param \Symfony\Component\HttpFoundation\Request $request
     */
    public function setRequest(\Symfony\Component\HttpFoundation\Request $request = null)
    {
        $this->request = $request;
    }

    /**
     * @return \Symfony\Component\HttpFoundation\Request
     */
    public function getRequest()
    {
        if ($this->request === null) {
            $this->request = \Symfony\Component\HttpFoundation\Request::createFromGlobals();
        }
        return $this->request;
    }

    protected function getManualPath()
    {
        $locale = $this->getRequest()->getLocale();
        $fileLocator = new FileLocator(__DIR__."/../../Resources/public/help");
        try {
            $filename = "manual_" . $locale . ".pdf";
            $fileLocator->locate($filename);
        } catch (\InvalidArgumentException $e) {
            // file is not found
            try {
                $filename = "manual.pdf";
                $fileLocator->locate($filename);
            } catch (\InvalidArgumentException $e) {
                // file is not found;
                return null;
            }
        }

        return "/bundles/topaliskernel/help/" . $filename;
    }

    public function getGlobals()
    {
        return array(
            "manualPath" => $this->getManualPath()
        );
    }

    /**
     * Returns the name of the extension.
     * @return string The extension name
     */
    public function getName()
    {
        return "help";
    }
}
