<?php
namespace Topalis\Kernel\Twig\Extension;

use Topalis\Menu\Menu;
use Symfony\Component\Security\Core\SecurityContextInterface;
use Topalis\Util\Process;

/**
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class MenuExtension extends \Twig_Extension
{
    /**
     * @var \Symfony\Component\Security\Core\SecurityContextInterface
     */
    private $securityContext;

    /**
     * @param \Symfony\Component\Security\Core\SecurityContextInterface $securityContext
     */
    public function setSecurityContext(SecurityContextInterface $securityContext)
    {
        $this->securityContext = $securityContext;
    }

    /**
     * @return \Symfony\Component\Security\Core\SecurityContextInterface
     */
    public function getSecurityContext()
    {
        return $this->securityContext;
    }

    protected function getMenu()
    {
        // Menu defintion
        $menu = new Menu();

        $menu
            ->add("admin_dashboard", "admin.dashboard.title", "icon-dashboard")
        ;

        $menu->addDivider();
        $menu->addGroup("admin.menu.user")
            ->add("admin_group", "admin.group.title", "icon-list-alt")
            ->add("admin_user", "admin.users.title", "icon-user")
            ->add("admin_device", "admin.devices.title", "icon-phone")
        ;

        if ($this->getSecurityContext()->getToken() && $this->getSecurityContext()->isGranted("ROLE_ADMIN")) {
            $menu->addDivider();
            $menu->add("admin_reverse_proxy", "admin.reverseProxy.title", "icon-globe");
            $menu->add("admin_web_clip", "admin.webClip.title", "icon-globe");
            $menu->add("admin_app_ios_managed", "admin.app.title", "icon-th");
            $menu->add("admin_wifi_config", "admin.wifiConfig.title", "icon-signal");

            $menu->addDivider();
            $menu->addGroup("admin.menu.configuration");
            $menu
                ->add("admin_config_network", "admin.global.title", "icon-wrench")
                ->add("admin_admin", "admin.admins.title", "icon-star")
            ;

            $menu->addDivider();
            $menu
                ->add("admin_backup", "admin.backup.title", "icon-refresh")
                ->add("admin_maintenance", "admin.maintenance.title", "icon-hdd")
                ->add("admin_log", "admin.log.title", "icon-comment");
            /*
             * ugly workaround to show update entry only if updates in the way
             * performed by the UpdateBundle are possible
             * TODO there should be a way to add menu entries through a bundle
             */
            if (class_exists("Topalis\Update\UpdateManager")
                && Process::commandExists(\Topalis\Update\UpdateManager::UPDATE_COMMAND)) {
                $menu->add("admin_update", "admin.update.title", "icon-upload");
            }
            $menu->add("admin_license", "admin.license.title", "icon-briefcase");
        }

        $menu->addDivider();
        $menu->add("admin_about", "admin.about.title", "icon-info-sign");

        return $menu;
    }

    public function getGlobals()
    {
        return array(
            "menu" => $this->getMenu(),
        );
    }

    /**
     * Returns the name of the extension.
     *
     * @return string The extension name
     */
    public function getName()
    {
        return "menu";
    }
}
