<?php
namespace Topalis\Kernel\Twig\Extension;

use Symfony\Component\Translation\Translator;

/**
 * Provide translate functions to templates
 * trans("foo.bar")
 * "foo.bar"|trans
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class TranslateExtension extends \Twig_Extension
{
    /**
     * @var \Symfony\Component\Translation\Translator
     */
    protected $translator;

    public function __construct(Translator $translator)
    {
        $this->translator = $translator;
    }
    public function getFunctions()
    {
        return array(
            new \Twig_SimpleFunction("trans", array($this, "translate")),
            new \Twig_SimpleFunction("transJ", array($this, "translateJSON"), array("is_safe" => array("html"))),
        );
    }

    public function getFilters()
    {
        return array(
            new \Twig_SimpleFilter("trans", array($this, "translate")),
            new \Twig_SimpleFilter("transJ", array($this, "translateJSON"), array("is_safe" => array("html"))),
            new \Twig_SimpleFilter("transchoice", array($this, "transchoice")),
        );
    }

    public function translate($message, array $arguments = array(), $domain = null, $locale = null)
    {
        if (null === $domain) {
            $domain = 'messages';
        }

        return $this->translator->trans($message, $arguments, $domain, $locale);
    }

    public function translateJSON($id, array $parameters = array(), $domain = 'messages', $locale = null)
    {
        return "'" . str_replace("'", "\'", $this->translate($id, $parameters, $domain, $locale)) . "'";
    }

    public function transchoice($message, $count, array $arguments = array(), $domain = null, $locale = null)
    {
        if (null === $domain) {
            $domain = 'messages';
        }

        $arguments = array_merge(array('%count%' => $count), $arguments);
        return $this->translator->transChoice($message, $count, $arguments, $domain, $locale);
    }

    /**
     * Returns the name of the extension.
     *
     * @return string The extension name
     */
    public function getName()
    {
        return "trans";
    }
}
