<?php
namespace Topalis\License;

use Symfony\Component\DependencyInjection\ContainerAware;

class LicenseManager extends ContainerAware
{
    /**
     * @var \Topalis\License\License
     */
    private $license;

    /**
     * @var string
     */
    private $licenseFilename;

    /**
     * @return \Topalis\Apache\Apache
     */
    private function getApache()
    {
        return $this->container->get("apache");
    }

    /**
     * @return \Topalis\ActiveSyncFilter\Apache\ASFApacheMod
     */
    private function getASFApacheMod()
    {
        return $this->container->get("asf.apache_mod");
    }

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * public because License uses it
     * @return \Topalis\Configuration\AuralisConfig
     */
    public function getConfig()
    {
        return $this->container->get("config.auralis");
    }

    /**
     * public because License uses it
     * @return \Topalis\Database\MySqlDb
     */
    public function getDB()
    {
        return $this->container->get("db");
    }

    /**
     * @param string $content
     * @return \Topalis\License\License
     */
    public function createLicense($content)
    {
        return new License($this, $content);
    }

    /**
     * @return \Topalis\License\License
     */
    public function &getLicense()
    {
        if ($this->license === null) {
            if (is_file($this->getLicenseFilename())) {
                $content = file_get_contents($this->getLicenseFilename());
            } else {
                $content = null;
            }
            $this->license = $this->createLicense($content);
        }
        return $this->license;
    }

    /**
     * @param \Topalis\License\License $license
     */
    public function setLicense(\Topalis\License\License $license)
    {
        $wasValid = $license->isValid();
        $licenseExists = is_file($this->getLicenseFilename());
        $license->save($this->getLicenseFilename());
        if (!$licenseExists) {
            /*
             * License file newly created so it's required to set permissions.
             * If the file aready existed the current process may not be allowed
             * to do this.
             */
            chmod($this->getLicenseFilename(), 0600);
        }
        $this->license = $license;
        if (!$wasValid && $this->license->isValid()) {
            /*
             * create revocation list to ensure that certificates
             * that were revoked because of an invalid license are
             * removed from the revocation list.
             */
            $this->getCertificateManager()->enableCreateRevocationList();
            // reenable features that were disabled because of an
            // invalid license.
            if (!$this->getASFApacheMod()->isEnabled()) {
                $this->getASFApacheMod()->enable();
            }
            $this->getApache()->setReverseProxyEnabled(true);
        }
    }

    /**
     * @return string
     */
    private function getLicenseFilename()
    {
        return $this->licenseFilename;
    }

    /**
     * @param string $licenseFilename
     */
    public function setLicenseFilename($licenseFilename)
    {
        $this->licenseFilename = $licenseFilename;
    }

    /**
     * public because License uses it
     * @return \Topalis\Kernel\TimeProviderInterface
     */
    public function getTimeProvider()
    {
        return $this->container->get("kernel.time_provider");
    }
}
