<?php
namespace Topalis\Log;

use Topalis\Util\Process;

/**
 * Class for filtering log files
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class FilteredLog implements LogInterface
{
    protected $log;
    protected $filter;

    protected $content;
    protected $highlight = false;

    public function __construct(LogInterface $log, $filter, $highlight = false)
    {
        $this->log = $log;
        $this->filter = $filter;
        $this->highlight = $highlight;
    }

    public function getFilterCommand()
    {
        $filter = $this->filter;
        if ($this->highlight) {
            $filter = addcslashes($filter, "()\"");
            $filter = addcslashes($filter, "\""); // triple escape for double quotes
            $filter = addcslashes($filter, "\"");

            return sprintf("sed -rn \"s/(%s)/<span class='highlight'>\\1<\/span>/ip\"", $filter);
        }
        $filter = addcslashes($filter, "'");

        return sprintf("grep '%s'", $filter);
    }

    public function getLines()
    {
        $cmd = sprintf("%s '%s' | wc", $this->getFilterCommand(), $this->log->getRealPath());

        $output = "";
        Process::exec($cmd, $output);
        $output = explode(" ", trim($output));

        return $output[0];
    }

    public function getContentFromLine($line)
    {
        $output = "";
        $cmd = sprintf("%s '%s' | tail -n +%s ", $this->getFilterCommand(), $this->log->getRealPath(), $line);
        Process::exec($cmd, $output);

        return $output;
    }

    public function getContent()
    {
        if (null !== $this->content) {
            return $this->content;
        }

        $output = "";
        $cmd = sprintf("%s '%s'", $this->getFilterCommand(), $this->log->getRealPath());
        Process::exec($cmd, $output);

        return $this->content = $output;
    }

    public function getId()
    {
        return $this->log->getId();
    }

    public function getName()
    {
        return $this->log->getName();
    }

    public function setName($name)
    {
        $this->log->setName($name);

        return $this;
    }
}
