<?php
namespace Topalis\Log;

/**
 * This class is used to manage the WebDAV access to log files.
 *
 * @author Andreas Hügel <andreas.huegel@topalis.com>
 * @author Frederick Roth <frederick.roth@topalis.com>
 * @author Fritz Elfert <fritz.elfert@topalis.com>
 * @author Holger Widmann <holger.widmann@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class WebDavManager
{
    /**
     * @var \Topalis\Admin\AdminManager
     */
    private $adminManager;

    /**
     * @var \Topalis\Apache\Apache
     */
    private $apache;

    /**
     * @var \Topalis\Configuration\AuralisConfig
     */
    private $config;

    /**
     * @return \Topalis\Admin\AdminManager
     */
    protected function getAdminManager()
    {
        return $this->adminManager;
    }

    /**
     * @param \Topalis\Admin\AdminManager $adminManager
     */
    public function setAdminManager(\Topalis\Admin\AdminManager $adminManager)
    {
        $this->adminManager = $adminManager;
        $adminManager->addListener(new WebDavAdminListener($this));
    }

    /**
     * @return \Topalis\Apache\Apache
     */
    protected function getApache()
    {
        return $this->apache;
    }

    /**
     * @param \Topalis\Apache\Apache $apache
     */
    public function setApache(\Topalis\Apache\Apache $apache)
    {
        $this->apache = $apache;
    }

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    public function getConfig()
    {
        return $this->config;
    }

    /**
     * @param \Topalis\Configuration\AuralisConfig $config
     */
    public function setConfig(\Topalis\Configuration\AuralisConfig $config)
    {
        $this->config = $config;
    }

    /**
     * Fetch WebDAV status.
     * @return boolean true, if WebDAV access is enabled.
     */
    public function isEnabled()
    {
        return $this->getConfig()->isDavEnabled();
    }

    /**
     * Generate content for htpasswd file from admins table and
     * enable or disable WebDAV access in apache.
     *
     * @throws \Exception On error.
     */
    public function generateHtpasswd()
    {
        $pwcontent = "";
        if ($this->isEnabled()) {
            $admins = $this->getAdminManager()->getAdmins();
            foreach ($admins as $a) {
                if ($a->davhash) {
                    $pwcontent .= $a->login.':'.$a->davhash."\n";
                }
            }
            $this->getApache()->enableDavAccess($pwcontent);
        } else {
            $this->getApache()->disableDavAccess();
        }
    }

    /**
     * Enable or disable DAV access for log files.
     * @param boolean $enabled true = enable, false = disable
     * @throws \Exception On error.
     */
    public function setEnabled($enabled)
    {
        $this->getConfig()->setDavEnabled($enabled);
        $this->generateHtpasswd();
    }
}
