<?php
namespace Topalis\MDM\Request;

use Topalis\MDM\Request\AbstractMdmRequest;

/**
 * This class represents the InstallApplication request.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class InstallApplicationRequest extends AbstractMdmRequest
{
    const REQUEST_TYPE = "InstallApplication";

    const MGM_FLAG_REMOVE = 1;
    const MGM_FLAG_PREVENT_BACKUP = 4;

    /**
     *
     * @return integer|NULL
     */
    public function getITunesStoreID()
    {
        return $this->getValue("iTunesStoreID");
    }

    /**
     *
     * @param integer|NULL $iTunesStoreID
     */
    public function setITunesStoreID($iTunesStoreID)
    {
        $this->setValue("iTunesStoreID", $iTunesStoreID, static::TYPE_INTEGER);
    }

    /**
     *
     * @return string|NULL
     */
    public function getIdentifier()
    {
        return $this->getValue("Identifier");
    }

    /**
     *
     * @param string|NULL $identifier
     */
    public function setIdentifier($identifier)
    {
        $this->setValue("Identifier", $identifier, self::TYPE_STRING);
    }

    /**
     * Android only
     * @return string|Null
     */
    public function getName()
    {
        return $this->getValue("Name");
    }

    /**
     * android only
     * @param string|Null $name
     */
    public function setName($name)
    {
        $this->setValue("Name", $name, self::TYPE_STRING);
    }

    /**
     *
     * @return array|NULL
     */
    public function getOptions()
    {
        return $this->getValue("Options");
    }

    /**
     *
     * @param string $key
     * @return mixed|NULL
     */
    public function getOption($key)
    {
        $options = $this->get("Options");
        return isset($options[$key]) ? $options[$key]->getValue() : null;
    }

    /**
     *
     * @param array|NULL $options
     */
    public function setOptions(array $options = null)
    {
        $this->setValue("Options", $options, self::TYPE_DICTIONARY);
    }

    /**
     *
     * @param string $key
     * @param mixed|NULL $value
     * @param int $type
     */
    public function setOption($key, $value, $type)
    {
        $options = $this->getOptions();
        if ($value === null) {
            unset($options[$key]);
            if (empty($options)) {
                $options = null;
            }
        } else {
            $options[$key] = $this->convertToCFType($value, $type);
        }
        $this->setOptions($options);
    }

    /**
     *
     * @return boolean|NULL
     */
    public function isNotManaged()
    {
        return $this->getOption("NotManaged");
    }

    /**
     *
     * @param boolean|NULL $notManaged
     */
    public function setNotManaged($notManaged)
    {
        $this->setOption("NotManaged", $notManaged, self::TYPE_BOOLEAN);
    }

    /**
     *
     * @return integer|NULL
     */
    public function getPurchaseMethod()
    {
        return $this->getOption("PurchaseMethod");
    }

    /**
     *
     * @param integer|NULL $purchaseMethod
     */
    public function setPurchaseMethod($purchaseMethod)
    {
        $this->setOption("PurchaseMethod", $purchaseMethod, self::TYPE_INTEGER);
    }

    /**
     *
     * @return string|NULL
     */
    public function getManifestURL()
    {
        return $this->getValue("ManifestURL");
    }

    /**
     *
     * @param string|NULL $manifestURL
     */
    public function setManifestURL($manifestURL)
    {
        $this->setValue("ManifestURL", $manifestURL, self::TYPE_STRING);
    }

    /**
     *
     * @return integer|NULL
     */
    public function getManagementFlags()
    {
        return $this->getValue("ManagementFlags");
    }

    /**
     *
     * @param integer $managementFlags
     */
    public function setManagementFlags($managementFlags)
    {
        $this->setValue("ManagementFlags", $managementFlags, self::TYPE_INTEGER);
    }

    /**
     *
     * @return array|NULL
     */
    public function getConfiguration()
    {
        return $this->getValue("Configuration");
    }

    /**
     *
     * @param array|NULL $configuration
     */
    public function setConfiguration(array $configuration = null)
    {
        $this->setValue("Configuration", $configuration, self::TYPE_DICTIONARY);
    }

    /**
     *
     * @return array|NULL
     */
    public function getAttributes()
    {
        return $this->getValue("Attributes");
    }

    /**
     *
     * @param array|NULL $attributes
     */
    public function setAttributes(array $attributes = null)
    {
        $this->setValue("Attributes", $attributes, self::TYPE_DICTIONARY);
    }
}
