<?php
namespace Topalis\Mail\Command;

use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;

class SendAdminMailCommand extends ContainerAwareCommand
{
    /**
     * @return \Topalis\Mail\Mailer
     */
    private function getMailer()
    {
        return $this->getContainer()->get("mailer");
    }

    protected function configure()
    {
        $this
            ->setName("mail:send_admin")
            ->setDescription("Sends an email to the configured addresses.")
            ->setHelp("Sends an email to the configured addresses.")
            ->addArgument("subject", InputArgument::REQUIRED, "The subject of the email.")
            ->addArgument("message", InputArgument::OPTIONAL,
                "The message of the email. If not defined it is read from stdin.")
            ->addOption("disable-exceptions", null, InputOption::VALUE_NONE,
                "Disables printing of exceptions.")
            ->addOption("cron", null, InputOption::VALUE_NONE,
                "Prints the message if the mailer is not configured. Does nothing if no message was read.")
        ;
    }

    protected function execute(\Symfony\Component\Console\Input\InputInterface $input,
        \Symfony\Component\Console\Output\OutputInterface $output)
    {
        $output; // unused
        try {
            $subject = $input->getArgument("subject");
            $message = $input->getArgument("message");
            $disableExceptions = $input->getOption("disable-exceptions");
            $cron = $input->getOption("cron");

            if ($message === null) {
                while (($line = fgets(STDIN)) !== false) {
                    $message[] = $line;
                }

                if ($message === null) {
                    if ($cron) {
                        return;
                    }

                    throw new \RuntimeException("Could not read message from stdin.");
                }
            }

            $result = $this->sendAdminMail($subject, $message);
            if (!$result) {
                if ($cron) {
                    $output->write($message);
                    if ($result === false) {
                        // mailer is not configured
                        return 0;
                    }
                }

                return 1;
            }
        } catch (\Exception $e) {
            if ($cron) {
                $output->write($message);
            } else if (!$disableExceptions) {
                throw $e;
            }

            return 70;
        }
    }

    /**
     * Sends message to configured admins with specified subject and text.
     *
     * @param string $subject The email subject.
     * @param string $message The email message.
     * @return int|boolean The number of recipients who were accepted for delivery,
     *     or false if there is no mail configuration.
     */
    private function sendAdminMail($subject, $message)
    {
        $mailer = $this->getMailer();
        if ($mailer->isConfigured()) {
            return $mailer->sendAdminMessage($subject, $message);
        }
        return false;
    }
}
