<?php
namespace Topalis\Rollout\Controller;

use Symfony\Component\Security\Core\SecurityContext;
use Topalis\Kernel\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Topalis\Certificate\Util\CertUtil;
use Symfony\Component\HttpFoundation\Response;

/**
 * Class to create the deployment form.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 * @Route("", service="rollout.controller.index")
 */
class IndexController extends Controller
{
    /**
     * @return \Symfony\Component\Form\Extension\Csrf\CsrfProvider\CsrfProviderInterface
     */
    protected function getCsrfProvider()
    {
        return $this->container->get("form.csrf_provider");
    }

    /**
     * @Route("/cacert.cer", name="cacert_der", methods="GET")
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function caCertDerAction()
    {
        $content = CertUtil::pem2der(file_get_contents($this->getConfig()->getCAChain()));
        $response = new Response($content);
        $response->headers->set("Content-Type", "application/x-x509-ca-cert");
        return $response;
    }

    /**
     * @Template()
     * @Route("/login", name="rollout_login")
     *
     * @return array
     */
    public function loginAction()
    {
        $lastUsername = $this->getRequest()->getSession()->get(SecurityContext::LAST_USERNAME);
        if ($this->getRequest()->attributes->has(SecurityContext::AUTHENTICATION_ERROR)) {
            $error = $this->getRequest()->attributes->get(SecurityContext::AUTHENTICATION_ERROR);
        } else {
            $error = $this->getRequest()->getSession()->get(SecurityContext::AUTHENTICATION_ERROR);
        }

        if ($error) {
            $error = "Bad credentials";
            $this->getRequest()->getSession()->clear();
        }
        $csrfToken = $this->getCsrfProvider()->generateCsrfToken('authenticate');

        return array(
            'lastUsername' => $lastUsername,
            'error'         => $error,
            'csrf_token'    => $csrfToken,
        );
    }

    /**
     * @Route("/login_check", name="rollout_security_check")
     */
    public function securityCheckAction()
    {
        // The security layer will intercept this request
    }
}
