<?php
namespace Topalis\SystemMonitor\Command;

use Topalis\Dashboard\System;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;

class MonitorDiskSpaceCommand extends ContainerAwareCommand
{
    const OUT_OF_SPACE_SUBJECT = "Alert: running out of disk space";
    const OUT_OF_SPACE_TEXT = <<<EOF
Running out of disk space "%s (%d%% used)" on %s.
EOF;

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    private function getConfig()
    {
        return $this->getContainer()->get("config.auralis");
    }

    /**
     * @return \Topalis\Mail\Mailer
     */
    private function getMailer()
    {
        return $this->getContainer()->get("mailer");
    }

    protected function configure()
    {
        $this
        ->setName("monitor:disk_space")
        ->setDescription("Monitors the remaining disk space.")
        ->setHelp("Monitors the remaining disk space.");
    }

    protected function execute(\Symfony\Component\Console\Input\InputInterface $input,
        \Symfony\Component\Console\Output\OutputInterface $output)
    {
        $input; // unused

        $this->checkDiskSpace($output);
    }

    protected function checkDiskSpace(\Symfony\Component\Console\Output\OutputInterface $output)
    {
        $hostname = $this->getConfig()->getHostname();
        $system = new System();
        $filesystem = $system->getFilesystem();
        foreach ($filesystem->getDisks() as $disk) {
            if ($disk->getUse() >= 90) {
                $subject = self::OUT_OF_SPACE_SUBJECT;
                $message = sprintf(self::OUT_OF_SPACE_TEXT, $disk->getName(), $disk->getUse(), $hostname);
                try {
                    $this->sendAdminMail($subject, $message);
                } catch (\Exception $e) {
                    if ($output instanceof \Symfony\Component\Console\Output\ConsoleOutput) {
                        $output = $output->getErrorOutput();
                    }
                    $output->writeln("Sending email failed: " . $e->getMessage());
                    $output->writeln("Subject: " . $subject);
                    $output->writeln("Message: " . $message);
                }
            }
        }
    }

    protected function sendAdminMail($subject, $message)
    {
        $mailer = $this->getMailer();
        if ($mailer->isConfigured()) {
            return $mailer->sendAdminMessage($subject, $message);
        }
        return false;
    }

}
