<?php
namespace Topalis\Update\Command;

use Topalis\Profile\ConfigurationProfile;
use Topalis\Profile\ProfileFactory;

/**
 * This class contains methods to perform the update.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class UpdateCommandHelper
{
    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    public function __construct(\Symfony\Component\DependencyInjection\ContainerInterface $container)
    {
        $this->container = $container;
    }

    /**
     * @return \Topalis\ConfigGroup\Entity\ConfigGroupEntityManager
     */
    private function getConfigGroupEntityManager()
    {
        return $this->container->get("config_group.entity.manager.config_group");
    }

    /**
     * @return \Topalis\MDM\MdmAdminInterface
     */
    private function getMdmAdmin()
    {
        return $this->container->get("mdm.admin");
    }

    /**
     * @return \Topalis\ReverseProxy\Entity\ReverseProxyUrlEntityManager
     */
    private function getReverseProxyUrlEntityManager()
    {
        return $this->container->get("reverse_proxy.entity.manager.url");
    }

    public function installSecureIntranetProfiles()
    {
        $installWebClipProfile = !$this->getReverseProxyUrlEntityManager()->exists();
        $mdmAdmin = $this->getMdmAdmin();
        foreach ($mdmAdmin->getActiveManagedDevices() as $device) {
            $mdmAdmin->installDeviceProfile($device);
            $mdmAdmin->installSecureIntranetProfile($device);
            if ($installWebClipProfile) {
                // now we have always a WebClip profile.
                // in 2.0 it was only available if there were WebClips configured.
                $mdmAdmin->installWebClipProfile($device, false);
            }
        }
    }

    public function removeOldDeviceProfiles()
    {
        $mdmAdmin = $this->getMdmAdmin();
        $configGroupManager = $this->getConfigGroupEntityManager();
        foreach ($configGroupManager->findAll() as $group) {
            $profile = new ConfigurationProfile($group->getProfile());
            if ($profile->getIdentifier() == ProfileFactory::getDeviceProfileIdentifier()) {
                continue;
            }

            foreach ($group->getDevices() as $device) {
                if ($device->isActive() && $device->isManaged()) {
                    $mdmAdmin->removeProfile($device, $profile->getIdentifier());
                }
            }
        }
    }
}
