<?php
namespace Topalis\Update\Command;

/**
 * This class contains methods to perform the update to version 2.2.1
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class UpdateHelper020201
{
    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    public function __construct(\Symfony\Component\DependencyInjection\ContainerInterface $container)
    {
        $this->container = $container;
    }

    /**
     *
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    private function getConfig()
    {
        return $this->container->get("config.auralis");
    }

    /**
     * Regnerate the server certificate because of the OpenSSL heartbleed bug
     * #AUR-408. But do it only if it has been signed by the auralis CA.
     *
     * @return boolean true if a new server certificate has been created, false
     *     otherwise.
     */
    public function regenerateServerCert()
    {
        $serverCert = $this->getCertificateManager()->getServerCert();

        $caCert = $this->getCertificateManager()->getCACertificate();
        // check if server cert is signed by auralis CA
        if ($serverCert->getSignerKeyId() != $caCert->getKeyId()) {
            return false;
        }

        $config = $this->getConfig();
        $serial = $config->getCertNextSerial();
        $cert = $serverCert->export($serial);
        $config->setServerCert($cert);

        $ca = $this->getCertificateManager()->loadCACert();
        $caCert = null;
        openssl_x509_export($ca[0], $caCert);
        array_unshift($ca[2], $caCert);
        $config->setServerChain(serialize($ca[2]));
        $this->getCertificateManager()->exportServerCert();
        // decrease serial such that we won't use the same serial again
        $config->setCertNextSerial($serial - 1);

        return true;
    }
}
