<?php
namespace Topalis\Update\Command;

use Topalis\Apache\Apache;
use Topalis\Util\FileSystemUtil;

/**
 * This class contains methods to perform the update to version 2.2.1
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class UpdateHelper020202
{
    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    public function __construct(\Symfony\Component\DependencyInjection\ContainerInterface $container)
    {
        $this->container = $container;
    }

    /**
     * @return \Topalis\Apache\Apache;
     */
    private function getApache()
    {
        return $this->container->get("apache");
    }

    /**
     * @return \Topalis\ReverseProxy\Entity\ReverseProxyUrlEntityManager
     */
    private function getReverseProxyUrlEntityManager()
    {
        return $this->container->get("reverse_proxy.entity.manager.url");
    }

    /**
     * Remove Apache config files and icons of removed reverse proxy URLs
     * #AUR-415.
     */
    public function cleanReverseProxyUrls()
    {
        $em = $this->getReverseProxyUrlEntityManager();
        $apache = $this->getApache();
        $path = $apache->getConfigPath(Apache::REVERSE_PROXY_DIR);

        // remove Apache config files
        foreach (new \DirectoryIterator($path) as $fileInfo) {
            if (!$fileInfo->isFile()) {
                continue;
            }

            $filename = $fileInfo->getFilename();
            if (!preg_match("/proxy_([0-9]+)\.conf/", $filename, $matches)) {
                continue;
            }

            $id = $matches[1];

            if ($em->find($id) !== null) {
                continue;
            }

            $apache->removeReverseProxy($id);
        }

        // remove icons
        $uploadPath = $this->container->getParameter("kernel.upload_dir") . "/reverse_proxy/";
        if (!is_dir($uploadPath)) {
            return;
        }

        foreach (new \DirectoryIterator($uploadPath) as $fileInfo) {
            $filename = $fileInfo->getFilename();
            if (!$fileInfo->isFile() || strtolower(substr($filename, -4)) != ".png") {
                continue;
            }

            if ($em->exists(("entity.icon = :filename"), array("filename" => $filename))) {
                continue;
            }

            FileSystemUtil::tryUnlinkFile($fileInfo->getPathname());
        }
    }
}
