<?php
namespace Topalis\Update\Command;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Topalis\MDM\Entity\MdmCommand;

/**
 * This class contains methods to perform the update to version 2.4.1
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class UpdateHelper020401
{
    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }

    /**
     * In 2.4.0 wrong device profiles have been installed on iOS devices so we
     * reinstall the device profile on all iOS devices.
     */
    public function fixDeviceProfile()
    {
        $typeFactory = $this->container->get("device.type.factory");
        $iosType = $typeFactory->getType("ios");

        $deviceEm = $this->container->get("device.entity.manager.device");
        $mdmCommandEm = $this->container->get("mdm.entity.manager.mdm_command");

        $devices = $deviceEm->findBy(array(
            "os" => $iosType->getId(),
            "disabled" => 0,
        ));

        $mdmAdmin = $this->container->get("mdm.admin");
        $mdmQueue = $this->container->get("mdm.queue");
        foreach ($devices as $device) {
            if (!$device->isManaged() || !$device->isActive()) {
                continue;
            }

            $mdmCommands = $mdmCommandEm->findBy(array(
                "device" => $device,
                "info" => "Device profile",
                "requestType" => "InstallProfile",
                "status" => array(
                    MdmCommand::STATUS_NEW,
                    MdmCommand::STATUS_QUEUED,
                    MdmCommand::STATUS_NOT_NOW
                )
            ));
            foreach ($mdmCommands as $mdmCommand) {
                $mdmQueue->cancelCommand($mdmCommand);
            }

            $mdmAdmin->installDeviceProfile($device);
        }
    }
}
