<?php
namespace Topalis\Update\Command;

use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * This class contains methods to perform the update to version 2.4.2
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class UpdateHelper020402
{
    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }

    /**
     * On group change the WiFis and apps weren't updated on Android and iOS so
     * we do it here for all devices, since we didn't now if a group change has
     * been performed.
     */
    public function fixConfigGroupChange()
    {
        $deviceEm = $this->container->get("device.entity.manager.device");
        $typeFactory = $this->container->get("device.type.factory");
        $androidType = $typeFactory->getType("android");
        $iosType = $typeFactory->getType("ios");

        $devices = $deviceEm->findBy(array(
            "os" => array($androidType->getId(), $iosType->getId()),
            "disabled" => 0,
        ));

        $mdmAdmin = $this->container->get("mdm.admin");
        foreach ($devices as $device) {
            if (!$device->isManaged() || !$device->isActive()) {
                continue;
            }

            $mdmAdmin->installWifiProfile($device);
            $mdmAdmin->installAppsForDevice($device);
            $mdmAdmin->removeUnassignedAppsFromDevice($device);
        }
    }
}
