<?php
namespace Topalis\User\EventListener;

use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Event\PostFlushEventArgs;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Topalis\Device\Entity\Device;
use Topalis\Device\Entity\MailAccount;
use Topalis\User\Entity\User;

/**
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class MailAccountEventListener
{

    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    protected $deleting = false;
    protected $toDelete = array();

    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }

    public function postUpdate(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if (!$entity instanceof User) {
            return;
        }

        $em = $eventArgs->getEntityManager();

        $uow = $eventArgs->getEntityManager()->getUnitOfWork();
        $changeSet = $uow->getEntityChangeSet($entity);

        if (!isset($changeSet['email'])) {
            return;
        }

        $devices = $entity->getDevices();
        $oldEmail = $changeSet['email'][0];
        $newEmail = $changeSet['email'][1];

        foreach ($devices as $device) {
            if (!$device->isDeleted()) {
                $this->updateDevice($device, $entity, $oldEmail, $newEmail);
                $em->persist($device);
            }
        }
        $em->flush();
    }

    public function updateDevice(Device $device, User $user, $oldEmail, $newEmail)
    {
        /** @var $device Device */
        $mailAccounts = $device->getMailAccounts();
        if (empty($mailAccounts)) {
            return;
        }

        $newAccountAlreadyExists = false;
        foreach ($mailAccounts as $mailAccount) {
            if ($mailAccount->getEmail() == $oldEmail) {
                $removeMailAccount = $mailAccount;
                $device->removeMailAccount($removeMailAccount);
                continue;
            }
            if ($mailAccount->getEmail() == $newEmail) {
                $newAccountAlreadyExists = true;
                continue;
            }
            if ($newAccountAlreadyExists && isset($removeMailAccount)) {
                break;
            }
        }
        if (!$newAccountAlreadyExists) {
            $account = new MailAccount();
            $account
                ->setEmail($newEmail)
                ->setDevice($device)
                ->setUser($user->getLogin());
            $device->addMailAccount($account);
        }
    }

    public function postFlush(PostFlushEventArgs $eventArgs)
    {
        if ($this->deleting) {
            return;
        }

        if (empty($this->toDelete)) {
            return;
        }

        $this->deleting = true;
        foreach ($this->toDelete as $key => $entity) {
            unset($this->toDelete[$key]);
            $eventArgs->getEntityManager()->remove($entity);
        }
        $eventArgs->getEntityManager()->flush();
        $this->deleting = false;
    }
}
