<?php
namespace Topalis\User\EventListener;

use Doctrine\ORM\Event\LifecycleEventArgs;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Topalis\Device\Entity\DeviceEntityManager;
use Topalis\User\Entity\User;

/**
 *
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class RemoveUserEventListener
{
    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }

    /**
     * @return DeviceEntityManager
     */
    protected function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    public function preRemove(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if (!$entity instanceof User) {
            return;
        }

        $this->getDeviceEntityManager()->disableDeleteFilter();
        foreach ($entity->getDevices() as $device) {
            $device->setUser(null);
            if ($device->isDeleted()) {
                $this->getDeviceEntityManager()->save($device);
            } else {
                $this->getDeviceEntityManager()->delete($device);
            }
        }
        $this->getDeviceEntityManager()->enableDeleteFilter();
    }
}
