<?php
namespace Topalis\User\Ldap;

use Topalis\Configuration\LdapConfig;
use Toyota\Component\Ldap\Core\Node;

/**
 * An LdapManager allows to access an LDAP directory to get users that can be
 * imported into auralis.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
interface LdapManagerInterface
{
    /**
     * Connects and binds to the Ldap connection.
     */
    public function bind();

    /**
     * @return \Topalis\Configuration\LdapConfig The LDAP configuration.
     */
    public function getLdapConfig();

    /**
     *
     * @param string $baseDn Base distinguished name to search in.
     *    (Default = configured dn)
     * @param boolean $inDepth Whether to search through all subtree depth.
     * @param int $limit The maximum number of users returned. 0 means no
     *     limit.
     * @return array Array of users containing the following keys: login, name,
     *     email, ou
     */
    public function getUsers($baseDn = null, $inDepth = false, $limit = 0);

    /**
     *
     * @param \Toyota\Component\Ldap\Core\Node $node Base node to search in.
     *    (Default = root node)
     * @return null|array Array containing a tree of organizations and
     *    organizational users that have at least one user. Returns null if
     *    there are no users.
     *    The tree has the following structure:
     *    array(2) {
     *      'dn' => <dn>
     *      'children' =>
     *      NULL|array(n) {
     *          '<o>|<ou>' =>
     *          array(2) {
     *              'dn' => <dn>
     *              'children' => NULL|array(m)
     *          }
     *          ...
     *      }
     *   }
     */
    public function getOuTree(Node $node = null);

    /**
     *
     * @param string $baseDn Base distinguished name to search in.
     *    (Default = configured dn)
     * @param boolean $inDepth Whether to search through all subtree depth.
     * @return boolean true if there's at least one user, false otherwise.
     */
    public function hasUsers($baseDn, $inDepth = false);

    /**
     * @return true if the settings required for an LDAP connection are
     *     provided.
     */
    public function isConfigured();

    /**
     * Sets the configuration to use.
     *
     * @param \Topalis\Configuration\LdapConfig $ldapConfig The configuration to set.
     */
    public function setLdapConfig(LdapConfig $ldapConfig);

    /**
     * The function users are filtered by. The function may return true to pass
     * users, or false to filter them out.
     *
     * @param callback $userFilterFunction The function to set.
     */
    public function setUserFilterFunction($userFilterFunction);
}
