<?php
namespace Topalis\WebClip\Controller;

use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\Kernel\Controller;
use Topalis\WebClip\Entity\WebClip;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @Route("/admin/group/{group}/section/web-clips")
 */
class ConfigGroupController extends Controller
{
    /**
     * @Route("", name="admin_web_clip_group")
     * @Template()
     *
     * @param ConfigGroup $group
     *
     * @return array
     */
    public function indexAction(ConfigGroup $group)
    {
        $webClips = $this->getWebClipEnityManager()->findAll();

        return array(
            "webClips" => $webClips,
            "group"    => $group,
        );
    }

    /**
     * @Route("/{webClip}/enable", name="admin_web_clip_group_enable")
     *
     * @param ConfigGroup $group
     * @param WebClip     $webClip
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function enable(ConfigGroup $group, WebClip $webClip)
    {
        $webClip->addConfigGroup($group);
        $this->getWebClipEnityManager()->save($webClip);

        return $this->redirect(
            $this->generateUrl("admin_web_clip_group", array("group" => $group->getId()))
        );
    }

    /**
     * @Route("/{webClip}/disable", name="admin_web_clip_group_disable")
     *
     * @param ConfigGroup $group
     * @param WebClip     $webClip
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function disable(ConfigGroup $group, WebClip $webClip)
    {
        $webClip->removeConfigGroup($group);
        $this->getWebClipEnityManager()->save($webClip);

        return $this->redirect(
            $this->generateUrl("admin_web_clip_group", array("group" => $group->getId()))
        );
    }


    /**
     * @return \Topalis\WebClip\Entity\WebClipEntityManager
     */
    protected function getWebClipEnityManager()
    {
        return $this->container->get("web_clip.entity.manager.web_clip");
    }
}
