<?php
namespace Topalis\WebClip\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Symfony\Component\HttpFoundation\File\File;
use Topalis\ConfigGroup\Entity\ConfigGroup;

/**
 * @ORM\MappedSuperclass()
 * @UniqueEntity(fields={"name"})
 */
abstract class AbstractWebClip implements WebClipInterface
{
    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="integer", nullable=false, options={"unsigned"=true})
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    protected $id;

    /**
     * @var string|File
     *
     * @ORM\Column(name="icon", type="string", length=4095, nullable=false)
     */
    protected $icon;

    /**
     * @var string
     *
     * @ORM\Column(name="name", type="string", length=255, nullable=false, unique=true)
     */
    protected $name;

    /**
     * @var boolean
     *
     * @ORM\Column(name="removable", type="boolean", nullable=false)
     */
    protected $removable = false;

    /**
     * @var boolean
     *
     * @ORM\Column(name="precomposed", type="boolean", nullable=false)
     */
    protected $precomposed = false;

    /**
     * @var boolean
     *
     * @ORM\Column(name="fullscreen", type="boolean", nullable=false)
     */
    protected $fullscreen = false;

    /**
     * @var ArrayCollection
     *
     * @ORM\ManyToMany(
     *  targetEntity="Topalis\ConfigGroup\Entity\ConfigGroup",
     *  inversedBy="webClips",
     *  cascade={"persist"}
     * )
     */
    protected $configGroups;

    /**
     * Indicates if and icon is added to the home screen.
     *
     * @var boolean
     *
     * @ORM\Column(name="visible", type="boolean", nullable=false,
     *     options={"default"=true})
     */
    protected $visible = true;

    public function __construct()
    {
        $this->configGroups = new ArrayCollection();
    }

    public function setId($id)
    {
        $this->id = $id;

        return $this;
    }

    public function getId()
    {
        return $this->id;
    }

    public function setIcon($icon)
    {
        $this->icon = $icon;

        return $this;
    }

    public function getIcon()
    {
        return $this->icon;
    }

    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    public function getName()
    {
        return $this->name;
    }

    public function setRemovable($removable)
    {
        $this->removable = $removable;

        return $this;
    }

    public function isRemovable()
    {
        return $this->removable;
    }

    public function setPrecomposed($precomposed)
    {
        $this->precomposed = $precomposed;

        return $this;
    }

    public function isPrecomposed()
    {
        return $this->precomposed;
    }

    public function setFullscreen($fullscreen)
    {
        $this->fullscreen = $fullscreen;

        return $this;
    }

    public function isFullscreen()
    {
        return $this->fullscreen;
    }

    public function setConfigGroups(array $configGroups)
    {
        $this->configGroups->clear();

        foreach ($configGroups as $group) {
            $this->addConfigGroup($group);
        }

        return $this;
    }

    public function getConfigGroups()
    {
        return $this->configGroups;
    }

    public function getUploadDir()
    {
        return "web_clips";
    }

    public function setVisible($visible)
    {
        $this->visible = $visible;

        return $this;
    }

    public function isVisible()
    {
        return $this->visible;
    }
}
