<?php
namespace Topalis\Wifi\Controller;

use Topalis\Kernel\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Topalis\Wifi\Entity\WifiConfig;
use Topalis\Wifi\Form\WifiConfigForm;
use Topalis\Wifi\Form\Handler\WifiConfigFormHandler;

/**
 * Controller for wifi configurations.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 * @Route("/admin/wifi", service="wifi_config.controller")
 */
class WifiConfigController extends Controller
{
    /**
     * @return \Topalis\Database\EntityManager
     */
    private function getWifiConfigEntityManager()
    {
        return $this->container->get("wifi_config.entity.manager.wifi_config");
    }

    /**
     * @Route("", name="admin_wifi_config")
     * @Template()
     * @return array
     */
    public function indexAction()
    {
        $wifiConfigs = $this->getWifiConfigEntityManager()->findAll();

        return array(
            "wifiConfigs" => $wifiConfigs,
        );
    }

    /**
     * @Route("/add", name="admin_wifi_config_add")
     * @Template()
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|\Symfony\Component\HttpFoundation\Response
     */
    public function addAction()
    {
        $wifiConfig = new WifiConfig();
        return $this->forward("wifi_config.controller:editAction", array("wifiConfig" => $wifiConfig));
    }

    /**
     * @Route("/{wifiConfig}/edit", name="admin_wifi_config_edit")
     * @Template
     *
     * @param WifiConfig $wifiConfig
     *
     * @throws \Topalis\Kernel\Exception\Http\NotFoundException
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|\Symfony\Component\HttpFoundation\Response
     */
    public function editAction(WifiConfig $wifiConfig)
    {
        $form = $this->createForm(new WifiConfigForm($wifiConfig));
        $formHandler = new WifiConfigFormHandler($this->getRequest());

        if ($formHandler->process($form)) {
            try {
                $this->getWifiConfigEntityManager()->save($formHandler->getWifiConfigEntity());
                $this->addSuccess($this->tr("admin.wifiConfig.saveSuccess"));
            } catch (\Exception $e) {
                $this->getLogger()->error("Error saving WiFi config", array("exception" => $e));
                $msg = $this->tr("admin.wifiConfig.error.save", array("%errmsg%" => $e->getMessage()));
                $this->addError($msg);
            }
            return $this->redirect($this->generateUrl("admin_wifi_config"));
        }

        return array(
            "form" => $form->createView(),
            "wifiConfig" => $wifiConfig,
        );
    }

    /**
     * @Route("/{wifiConfig}/delete", name="admin_wifi_config_delete")
     *
     * @param WifiConfig $wifiConfig
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function deleteAction(WifiConfig $wifiConfig)
    {
        try {
            $this->getWifiConfigEntityManager()->delete($wifiConfig);
            $this->addSuccess($this->tr("admin.wifiConfig.deleteSuccess"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error deleting WiFi config with id " . $wifiConfig->getId(),
                array("exception" => $e));
            $msg = $this->tr(
                "admin.wifiConfig.error.delete",
                array("%name%" => $wifiConfig->getName(), "%errmsg%" => $e->getMessage())
            );
            $this->addError($msg);
        }

        return $this->redirect($this->generateUrl("admin_wifi_config"));
    }

}
