<?php
namespace Topalis\Wifi\EventListener;

use Doctrine\ORM\Event\LifecycleEventArgs;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Topalis\Wifi\Entity\WifiConfig;

/**
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class WifiConfigListener
{
    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    /**
     * @param ContainerInterface $container
     */
    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }

    /**
     * @return \Topalis\Device\MDM\MdmFacadeInterface
     */
    private function getMdmFacade()
    {
        return $this->container->get("device.mdm.facade");
    }

    public function preRemove(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if (!$entity instanceof WifiConfig) {
            return;
        }

        foreach ($entity->getConfigGroups() as $configGroup) {
            $configGroup->removeWifiConfig($entity);
            $eventArgs->getEntityManager()->persist($configGroup);
        }
        $eventArgs->getEntityManager()->flush();
        $this->getMdmFacade()->removeWifi($entity);
    }

    public function postPersist(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if (!$entity instanceof WifiConfig) {
            return;
        }

        $this->getMdmFacade()->addWifi($entity);
    }

    public function postUpdate(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if (!$entity instanceof WifiConfig) {
            return;
        }

        $configGroups = $entity->getConfigGroups();
        // we have to get the changeSet here because it propably will be cleared by a facade
        $changeSet = $eventArgs->getEntityManager()->getUnitOfWork()->getEntityChangeSet($entity);
        $this->getMdmFacade()->updateWifi($entity, $configGroups->getDeleteDiff(),
            $configGroups->getInsertDiff(), $changeSet);
    }
}
