<?php
namespace Topalis\Wifi\Form\Type;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Validator\Constraints as Assert;
use Topalis\Wifi\Entity\WifiConfig;

/**
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class WifiConfigType extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $options; // unused
        $prefix = "admin.wifiConfig.";
        $builder
            ->add("name", "text", array(
                "label"       => $prefix . "name",
                "max_length"  => 255,
                "constraints" => array(
                    new Assert\NotBlank(),
                    new Assert\Length(array("max" => 255)),
                )
            ))
            ->add("ssid", "text", array(
                "label"       => $prefix . "ssid",
                "max_length"  => 32,
                "constraints" => array(
                    new Assert\NotBlank(),
                    new Assert\Length(array("max" => 32)),
                ),
            ))
            ->add("hidden", "checkbox", array(
                "label" => $prefix . "hiddenNetwork",
                "required" => false
            ))
            ->add("autoJoin", "checkbox", array(
                "label"    => $prefix . "autoJoin",
                "os"       => array(
                    "iOS",
                    "WinPhone" => ">=8.1",
                ),
                "required" => false
            ))
            ->add("encryptionType", "choice", array(
                "label"       => $prefix . "encryptionType",
                "choices"     => array(
                    WifiConfig::ENCRYPTION_TYPE_NONE
                        => $prefix . "encryptionType." . WifiConfig::ENCRYPTION_TYPE_NONE,
                    WifiConfig::ENCRYPTION_TYPE_WPAPSK
                        => $prefix . "encryptionType." . WifiConfig::ENCRYPTION_TYPE_WPAPSK,
                    WifiConfig::ENCRYPTION_TYPE_WPA2PSK
                        => $prefix . "encryptionType." . WifiConfig::ENCRYPTION_TYPE_WPA2PSK,
                ),
            ))
            ->add("encryptionKey", "password", array(
                "label"        => $prefix . "password",
                "always_empty" => false,
                "max_length"   => 63,
                "constraints"  => new Assert\Length(array("min" => 8, "max" => 63))
            ))
            ->add("hotspot", "checkbox", array(
                "label"    => $prefix . "hotspot",
                "os"       => array("iOS" => ">=7.0"),
                "required" => false
            ))
            ->add("proxyType", "choice", array(
                "label"   => $prefix . "proxyType",
                "choices" => array(
                    "None"   => $prefix . "proxyType.none",
                    "Manual" => $prefix . "proxyType.manual",
                    "Auto"   => $prefix . "proxyType.auto"
                ),
            ))
            ->add("proxyHost", "text", array(
                "label" => $prefix . "proxyHost",
                "max_length"  => 255,
                "constraints" => new Assert\Length(array("max" => 255)),
            ))
            ->add("proxyPort", "network_port", array(
                "label" => $prefix . "proxyPort",
            ))
            ->add("proxyUsername", "text", array(
                "label" => $prefix . "proxyUsername",
                "max_length"  => 255,
                "constraints" => new Assert\Length(array("max" => 255)),
                "os"      => array("iOS"),
            ))
            ->add("proxyPassword", "password", array(
                "label" => $prefix . "proxyPassword",
                "max_length"  => 255,
                "constraints" => new Assert\Length(array("max" => 255)),
                "os"      => array("iOS"),
            ))
            ->add("proxyPacUrl", "text", array(
                "label" => $prefix . "proxyPacUrl",
                "description" => $prefix . "proxyPacUrlDescr",
                "max_length"  => 255,
                "constraints" => new Assert\Length(array("max" => 255)),
                "os"      => array("iOS"),
            ))
            ->add("proxyPacFallbackAllowed", "checkbox", array(
                "label" => $prefix . "proxyPacFallbackAllowed",
                "os"      => array("iOS" => ">=7"),
                "required" => false
            ))
            ->add("configGroups", "configgroup_select", array(
                "label" => "admin.wifiConfig.groups",
                "descriptionBlock" => true,
                "description" => "admin.wifiConfig.groupsDescription",
                "required" => false,
            ))
            ;

    }

    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(array(
            "data_class" => 'Topalis\Wifi\Entity\WifiConfig'
        ));
    }

    public function getName()
    {
        return "wifiConfig";
    }
}
