<?php
namespace Topalis\WinPhone\Controller;

use Topalis\Kernel\Controller;
use Topalis\WinPhone\Discovery\DiscoveryServer;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Component\HttpFoundation\Response;
use Topalis\WinPhone\Soap\SoapResponse;

/**
 * This controller provides the discovery service for Windows Phone.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 * @Route("/EnrollmentServer/Discovery.svc", service="winphone.controller.discovery")
 */
class DiscoveryController extends Controller
{
    public function getSoapServer()
    {
        $server = new DiscoveryServer();
        $config = $this->getConfig();
        $enrollmentServiceUrl = "https://" . $config->getHostname() . ":" . $config->getSecondaryPort()
            . "/EnrollmentServer/DeviceEnrollment.svc";
        $server->setEnrollmentServiceUrl($enrollmentServiceUrl);
        $certificatePolicyServiceUrl = "https://" . $config->getHostname() . ":" . $config->getSecondaryPort()
            . "/EnrollmentServer/CertificatePolicy.svc";
        $server->setEnrollmentPolicyServiceUrl($certificatePolicyServiceUrl);
        return $server;
    }

    /**
     * @Route("", name="win_discover_index", methods="GET")
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function indexAction()
    {
        $response = new Response();
        $response->headers->set("Pragma", "no-cache");
        $response->headers->set("Content-Type", "text/html");
        $response->headers->set("Content-Length", 0);
        return $response;
    }

    /**
     * @Route("", name="win_discover", methods="POST")
     *
     * @return \Topalis\WinPhone\Soap\SoapResponse
     */
    public function discoverAction()
    {
        $server = $this->getSoapServer();
        $content = $server->handle($this->getRequest()->getContent());
        $response = new SoapResponse($content);
        $response->headers->set("Content-Length", strlen($content));

        return $response;
    }
}
