<?php
namespace Topalis\WinPhone\Controller;

use Topalis\Kernel\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Component\HttpFoundation\Response;
use Topalis\WinPhone\OMA\SyncML\Entity\XmlConverter;
use Topalis\WinPhone\OMA\SyncML\Handler\SyncMLHandler;
use Topalis\WinPhone\OMA\SyncML\SyncMLSession;
use Topalis\Device\Entity\AccessLog;

/**
 * This controller provides the OMA device management service for Windows Phone.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 * @Route("/omadm/winphone", service="winphone.controller.omadm")
 */
class OmaDMController extends Controller
{
    /**
     *
     * @return \Topalis\WinPhone\OMA\SyncML\Handler\SyncMLHandler
     */
    private function getSyncMLHandler()
    {
        $syncMLCommandHandler = new SyncMLHandler();
        $syncMLCommandHandler->setContainer($this->container);
        return $syncMLCommandHandler;
    }

    /**
     * @Route("", name="win_omadm", methods="POST")
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function omaDMAction()
    {
        // set language of admin mails to default; ignore locale sent by client
        $this->getTranslator()->setLocale("");

        $syncML = XmlConverter::fromXml($this->getRequest()->getContent());
        $syncMLSession = new SyncMLSession($syncML);

        $this->getDoctrine()->getConnection()->beginTransaction();
        try {
            $syncMLResponse = $this->getSyncMLHandler()->handleSyncMLSession($syncMLSession);
            $content = $syncMLResponse->toXml();
            $response = new Response($content);
            $response->headers->set("Content-Type", "application/vnd.syncml.dm+xml");
            $response->headers->set("Content-Length", strlen($content));

            $this->getDoctrine()->getConnection()->commit();
        } catch (\Exception $e) {
            $this->getDoctrine()->getConnection()->rollback();
            throw $e;
        }

        $device = $syncMLSession->getDevice();
        if ($device !== null) {
            try {
                // log device access
                $accessLog = new AccessLog();
                $tstamp = $this->getTimeProvider()->getDateTime();
                $accessLog->setTstamp($tstamp);
                $accessLog->setCert($device->getMdmCert());
                $accessLog->setDevice($device);
                $accessLog->setIp($this->getRequest()->getClientIp());
                $accessLog->setStatus(200);
                $accessLog->setTraffic($response->headers->get("Content-Length"));
                $this->container->get("device.entity.manager.access_log")->save($accessLog);
            } catch (\Exception $e) {
                $this->getLogger()->error("Could not store access data", array("exception" => $e));
            }
        }

        return $response;
    }
}
