<?php
namespace Topalis\WinPhone\Entity;

use Topalis\Database\EntityManager;
use Doctrine\Common\Persistence\ObjectManager;

/**
 * Entity manager for SyncMLCommand.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class SyncMLCommandEntityManager extends EntityManager
{
    public function __construct(ObjectManager $em)
    {
        parent::__construct($em, 'Topalis\WinPhone\Entity\SyncMLCommand');
    }

    /**
     *
     * @param \Topalis\Device\Entity\Device $device The device to get the
     *     commands for.
     * @param array $statusList Only return commands with one of the specified
     *     status. If not specified all commands are returned.
     * @return array Array with SyncMLCommand objects for the specified device.
     */
    public function findForDevice(\Topalis\Device\Entity\Device $device, array $statusList = null)
    {
        $qb = $this->createQueryBuilder("c")
            ->where("c.device = :device")
            ->setParameter("device", $device);

        if ($statusList !== null) {
            $where = array();
            $count = count($statusList);
            for ($i = 0; $i < $count; $i++) {
                $where[] = "c.status = :status" . $i;
                $qb->setParameter("status" . $i, $statusList[$i]);
            }
            $qb->andWhere(implode(" OR ", $where));
        }

        return $qb->getQuery()->getResult();
    }
}
