<?php
namespace Topalis\WinPhone\OMA\Provisioning;

class CertificateStoreProvider extends ConfigurationServiceProvider
{
    public function __construct()
    {
        parent::__construct("CertificateStore");
    }

    public function getCertificate($path)
    {
        $split = explode("/", $path);

        $characteristic = $this;
        foreach ($split as $type) {
            $characteristic = $characteristic->getFirstCharacteristic($type);
            if ($characteristic === null) {
                return null;
            }
        }

        foreach ($characteristic->getCharacteristics() as $charistic) {
            if ($charistic instanceof CertificateCharacteristic) {
                return $charistic;
            }
        }

        return null;
    }

    public function removeCertificate($path)
    {
        $split = explode("/", $path);

        $characteristics = array();
        $characteristic = $this;
        foreach ($split as $type) {
            $characteristic = $characteristic->getFirstCharacteristic($type);
            if ($characteristic === null) {
                return;
            }
            $characteristics[] = $characteristic;
        }

        $count = count($characteristics);
        $i = $count - 1;

        // remove certificate
        $charistics = $characteristics[$i]->getCharacteristics();
        foreach ($charistics as $key => $charistic) {
            if ($charistic instanceof CertificateCharacteristic) {
                $charistics->remove($key);
                break;
            }
        }

        // recursively remove empty characteristics
        for ($i; $i >= 1; $i--) {
            if ($characteristics[$i]->getCharacteristics()->isEmpty()) {
                $characteristics[$i-1]->removeCharacteristic($characteristics[$i]);
            }
        }
    }

    public function setCertificate($path, CertificateCharacteristic $certificate,
        array $addtionalCharacteristics = array())
    {
        $split = explode("/", $path);

        $characteristic = $this;
        foreach ($split as $type) {
            $nextCharacteristic = $characteristic->getFirstCharacteristic($type);
            if ($nextCharacteristic === null) {
                $nextCharacteristic = new Characteristic($type);
                $characteristic->addCharacteristic($nextCharacteristic);
            }
            $characteristic = $nextCharacteristic;
        }
        $characteristic->addCharacteristic($certificate);
        foreach ($addtionalCharacteristics as $charistic) {
            $characteristic->addCharacteristic($charistic);
        }
    }
}
