<?php
namespace Topalis\WinPhone\OMA\Provisioning;

class CharacteristicFactory
{
    private $inCertStore = false;

    public function fromDOMElement(\DOMElement $elem,
        $defaultClass = "Topalis\\WinPhone\\OMA\\Provisioning\\Characteristic")
    {
        if ($elem->tagName != "characteristic") {
            throw new \InvalidArgumentException("The tag name is not 'characteristic'");
        }

        $type = $elem->getAttribute("type");
        if (!$type) {
            throw new \InvalidArgumentException("Attribute 'type' is missing");
        }

        $isInCertStore = $this->inCertStore;

        $characteristic = null;
        switch ($type) {
            case "APPADDR":
                $characteristic = new AppAddrCharacteristic();
                break;
            case "APPAUTH":
                $characteristic = new AppAuthCharacteristic();
                break;
            case "APPLICATION":
                $characteristic = new ApplicationProvider();
                break;
            case "CERTIFICATE":
                $characteristic = new CertificateCharacteristic();
                break;
            case "CertificateStore":
                $this->inCertStore = true;
                $characteristic = new CertificateStoreProvider();
                break;
            case "DMClient":
                $characteristic = new DMClientProvider();
                break;
            case "Provider":
                $characteristic = new ProviderCharacteristic(null);
                break;
            case "Registry":
                $characteristic = new RegistryProvider();
                break;
            default:
                if ($this->inCertStore) {
                    $isCert = false;
                    foreach ($elem->childNodes as $node) {
                        if ($node->nodeType != XML_ELEMENT_NODE || $node->tagName != "parm") {
                            continue;
                        }

                        if ($node->getAttribute("name") == "EncodedCertificate") {
                            $isCert = true;
                            break;
                        }
                    }
                    if ($isCert) {
                        $characteristic = new CertificateCharacteristic();
                        $characteristic->setType($type);
                    }
                }

                if ($characteristic === null) {
                    $characteristic = new $defaultClass($type);
                }
                break;
        }

        foreach ($elem->childNodes as $node) {
            if ($node->nodeType != XML_ELEMENT_NODE) {
                continue;
            }

            if ($node->tagName == "characteristic") {
                $characteristic->addCharacteristic($this->fromDOMElement($node));
            } else if ($node->tagName == "parm") {
                $characteristic->addParm(Parm::fromDOMElement($node));
            }
        }

        $this->inCertStore = $isInCertStore;

        return $characteristic;
    }
}
