<?php
namespace Topalis\WinPhone\OMA\Provisioning;

class RegistryProvider extends ConfigurationServiceProvider
{
    public function __construct()
    {
        parent::__construct("Registry");
    }

    public function getValue($path)
    {
        $split = $this->splitPath($path);
        $type = $split[0];
        $name = $split[1];

        $characteristic = $this->getFirstCharacteristic($type);
        return $characteristic ? $characteristic->getParmValue($name) : null;
    }

    public function setValue($path, $value, $datatype)
    {
        $split = $this->splitPath($path);
        $type = $split[0];
        $name = $split[1];

        $characteristic = $this->getFirstCharacteristic($type);
        if ($characteristic === null) {
            if ($value === null) {
                return;
            }
            $characteristic = new Characteristic($type);
            $this->addCharacteristic($characteristic);
        }

        $characteristic->setParmValue($name, $value, $datatype);
        if ($characteristic->getParms()->isEmpty()) {
            $this->removeCharacteristic($characteristic);
        }
    }

    private function splitPath($path)
    {
        $pos = strrpos($path, "\\");
        if ($pos === false) {
            throw new \InvalidArgumentException("Invalid registry path");
        }

        $type = substr($path, 0, $pos);
        $name = substr($path, $pos + 1);

        if (strlen($type) == 0 || strlen($name) == 0) {
            throw new \InvalidArgumentException("Invalid registry path");
        }

        $split = array($type, $name);
        return $split;
    }
}
