<?php
namespace Topalis\WinPhone\OMA\Provisioning;

use Doctrine\Common\Collections\ArrayCollection;

/**
 * PHP representation of a wap-provisioningdoc (not complete; as far as required).
 *
 * @see http://msdn.microsoft.com/en-us/library/bb737517.aspx
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class WapProvisioningDoc
{
    /**
     * @var \Doctrine\Common\Collections\ArrayCollection
     */
    private $serviceProviders;

    /**
     * @var string
     */
    private $version;

    public static function fromXml($xml)
    {
        $doc = new \DOMDocument();
        $doc->loadXML($xml);
        $root = $doc->documentElement;
        if ($root === null || $root->tagName != "wap-provisioningdoc") {
            throw new \InvalidArgumentException("The given XML is not a wap-provisioningdoc");
        }

        $wapDoc = new WapProvisioningDoc();

        if ($root->hasAttribute("version")) {
            $wapDoc->setVersion($root->getAttribute("version"));
        }

        $serviceProviderFactory = new ConfigurationServiceProviderFactory();
        foreach ($root->childNodes as $node) {
            if ($node->nodeType != XML_ELEMENT_NODE) {
                continue;
            }

            if ($node->tagName == "characteristic") {
                $wapDoc->addConfigurationServiceProvider($serviceProviderFactory->fromDOMElement($node));
            }
        }

        return $wapDoc;
    }

    public function __construct($version = null)
    {
        $this->serviceProviders = new ArrayCollection();
        $this->setVersion($version);
    }

    public function addConfigurationServiceProvider(ConfigurationServiceProvider $serviceProvider)
    {
        $this->serviceProviders[] = $serviceProvider;
    }

    public function clearConfigurationServiceProviders($type = null)
    {
        if ($type === null) {
            $this->serviceProviders->clear();
        } else {
            foreach ($this->serviceProviders as $key => $serviceProvider) {
                if ($serviceProvider->getType() == $type) {
                    $this->serviceProviders->remove($key);
                }
            }
        }
    }

    /**
     * @param string $type The type of the Configuration Service Provider to
     *     get, null to get all Providers.
     * @return \Doctrine\Common\Collections\ArrayCollection
     */
    public function getConfigurationServiceProviders($type = null)
    {
        if ($type === null) {
            return $this->serviceProviders;
        }

        $serviceProviders = new ArrayCollection();
        foreach ($this->serviceProviders as $serviceProvider) {
            if ($serviceProvider->getType() == $type) {
                $serviceProviders[] = $serviceProvider;
            }
        }
        return $serviceProviders;
    }

    /**
     * @param string $type
     * @return ConfigurationServiceProvider|NULL
     */
    public function getFirstConfigurationServiceProvider($type)
    {
        foreach ($this->serviceProviders as $serviceProvider) {
            if ($serviceProvider->getType() == $type) {
                return $serviceProvider;
            }
        }

        return null;
    }

    public function removeConfigurationServiceProvider(ConfigurationServiceProvider $serviceProvider)
    {
        return $this->serviceProviders->removeElement($serviceProvider);
    }

    public function getVersion()
    {
        return $this->version;
    }

    public function setVersion($version)
    {
        $this->version = $version;
    }

    public function toXml($formatted = false, $xmlDeclaration = true)
    {
        $doc = new \DOMDocument();
        $root = $doc->createElement("wap-provisioningdoc");
        $doc->appendChild($root);

        if ($this->getVersion() !== null) {
            $root->setAttribute("version", $this->getVersion());
        }

        foreach ($this->serviceProviders as $serviceProvider) {
            $serviceProvider->appendToDOM($root);
        }

        if ($formatted) {
            $doc->formatOutput = true;
        } else {
            $doc->preserveWhiteSpace = false;
        }

        if ($xmlDeclaration) {
            return $doc->saveXML();
        }

        return $doc->saveXML($doc->documentElement);
    }
}
