<?php
namespace Topalis\WinPhone\OMA\SyncML\Entity;

class Alert extends AbstractConcreteCommand
{
    /**
     *
     * @var string
     */
    protected $data;

    /*
     * User interaction alert codes (1100-1199)
     */

    /**
     * Display message to provide information to the user.
     * @var int
     */
    const DISPLAY = 1100;

    /**
     * Display message and ask for confirmation.
     * @var int
     */
    const CONFIRM_OR_REJECT = 1101;

    /**
     * Display message and allow user to type in a text string.
     * @var int
     */
    const TEXT_INPUT = 1102;

    /**
     * The user is presented a set of choices from which she is allowed to
     * select only one.
     * @var int
     */
    const SINGLE_CHOICE = 1103;

    /**
     * The user is presented a set of choices from which she is allowed to
     * select one or more.
     * @var int
     */
    const MULTIPLE_CHOICE = 1104;

    // 1105-1199 Reserved for future SynML usage.

    /*
     * Device management session alert codes (1200-1220)
     */

    /**
     * Specifies a server-initiated device management session.
     * @var int
     */
    const SERVER_INITIATED_MGMT = 1200;

    /**
     * Specifies a client-initiated device management session.
     * @var int
     */
    const CLIENT_INITIATED_MGMT = 1201;

    // 1202-1220 Reserved for future SyncML usage.

    /*
     * Special device management alert codes (1222-1299)
     */

    /**
     * Specifies a request for the next message in the package.
     * @var int
     */
    const NEXT_MESSAGE = 1222;

    /**
     * Informs the recipient taht the sender wishes to abort the device
     * management session.
     * @var int
     */
    const SESSION_ABORT = 1223;

    /**
     * Informs the server that an event has occurred in the client.
     * @var int
     */
    const CLIENT_EVENT = 1224;

    /**
     * End of Data for chunked object not received.
     * @var int
     */
    const NO_END_OF_DATA = 1225;

    /**
     * Generic client generated alert with or without reference to a
     * Management Object.
     * @var int
     */
    const GENERIC_ALERT = 1226;

    // 1227-1299 Reserved for future SyncML usage.

    /**
     *
     * @var string
     */
    protected $correlator;

    /**
     *
     * @return string
     */
    public function getData()
    {
        return $this->data;
    }

    /**
     *
     * @param string $data
     */
    public function setData($data)
    {
        $this->data = $data;
        return $this;
    }

    /**
     *
     * @return string
     */
    public function getCorrelator()
    {
        return $this->correlator;
    }

    /**
     *
     * @param string $correlator
     */
    public function setCorrelator($correlator)
    {
        $this->correlator = $correlator;
        return $this;
    }
}
