<?php
namespace Topalis\WinPhone\OMA\SyncML\Entity\ConfigurationServiceProvider;

use Topalis\Profile\UUIDGenerator;

/**
 * ActiveSync configuration service provider.
 *
 * @see Windows Phone 8 Enterprise Device Management Protocol
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class ActiveSyncCSP extends AbstractConfigurationServiceProvider
{
    /**
     *
     * @var string
     */
    private $guID;

    /**
     *
     * @param string $cmdID
     * @param string $guID The global unique identifier of the ActiveSync
     *    account. Every ActiveSync account on a device must have its own GUID.
     *    If not specified an GUID is generated.
     */
    public function __construct($cmdID = null, $guID = null)
    {
        parent::__construct($cmdID);
        if ($guID === null) {
            $this->generateGuID();
        } else {
            $this->setGuID($guID);
        }
        foreach ($this->getDefaults() as $locURI => $data) {
            $this->setData($locURI, $data);
        }
    }

    /**
     * Generates and sets a new GUID.
     *
     * @return string The new GUID.
     */
    public function generateGuID()
    {
        $uuidGenerator = new UUIDGenerator();
        $guID = $uuidGenerator->generateUUID();
        $this->setGuID($guID);
        return $guID;
    }

    /**
     * @return string The global unique identifier of the ActiveSync account.
     *    Every ActiveSync account on a device must have its own GUID.
     */
    public function getGuID()
    {
        return $this->guID;
    }

    /**
     * @param string $guID The global unique identifier of the ActiveSync
     *    account. Every ActiveSync account on a device must have its own GUID.
     */
    public function setGuID($guID)
    {
        $this->guID = $guID;
        $locURIPrefix = "./Vendor/MSFT/ActiveSync/Accounts/%7B" . $guID . "%7D/";
        $this->setLocURIPrefix($locURIPrefix);
    }

    public function getEmailAddress()
    {
        return $this->getData("EmailAddress");
    }

    public function setEmailAddress($emailAddress)
    {
        $this->setData("EmailAddress", $emailAddress);
    }

    public function getDomain()
    {
        return $this->getData("Domain");
    }

    public function setDomain($domain)
    {
        $this->setData("Domain", $domain);
    }

    public function getAccountIcon()
    {
        return $this->getData("AccountIcon");
    }

    public function setAccountIcon($accountIcon)
    {
        $this->setData("AccountIcon", $accountIcon);
    }

    public function getAccountType()
    {
        return $this->getData("AccountType");
    }

    public function setAccountType($accountType)
    {
        $this->setData("AccountType", $accountType);
    }

    public function getAccountName()
    {
        return $this->getData("AccountName");
    }

    public function setAccountName($accountName)
    {
        $this->setData("AccountName", $accountName);
    }

    public function getServerName()
    {
        return $this->getData("ServerName");
    }

    public function setServerName($serverName)
    {
        $this->setData("ServerName", $serverName);
    }

    public function getUserName()
    {
        return $this->getData("UserName");
    }

    public function setUserName($userName)
    {
        $this->setData("UserName", $userName);
    }

    public function isUseSSL()
    {
        return $this->getData("Options/UseSSL") ? true : false;
    }

    public function setUseSSL($useSSL)
    {
        $this->setData("Options/UseSSL", $useSSL ? "1" : "0");
    }

    public function getSchedule()
    {
        return $this->getData("Options/Schedule");
    }

    public function setSchedule($schedule)
    {
        $this->setData("Options/Schedule", $schedule, "Replace");
    }

    public function getMailAgeFilter()
    {
        return $this->getData("Options/MailAgeFilter");
    }

    public function setMailAgeFilter($mailAgeFilter)
    {
        $this->setData("Options/MailAgeFilter", $mailAgeFilter, "Replace");
    }

    public function getLogging()
    {
        return $this->getData("Options/Logging");
    }

    public function setLogging($logging)
    {
        $this->setData("Options/Logging", $logging, "Replace");
    }

    public function isEmailEnabled()
    {
        return $this->isData("Options/ContentTypes/%7Bc6d47067-6e92-480e-b0fc-4ba82182fac7%7D/Enabled");
    }

    public function setEmailEnabled($enabled)
    {
        $this->setData("Options/ContentTypes/%7Bc6d47067-6e92-480e-b0fc-4ba82182fac7%7D/Enabled",
            $enabled ? "1" : "0", "Replace");
    }

    public function getEmailName()
    {
        return $this->getData("Options/ContentTypes/%7Bc6d47067-6e92-480e-b0fc-4ba82182fac7%7D/Name");
    }

    public function setEmailName($emailName)
    {
        $this->setData("Options/ContentTypes/%7Bc6d47067-6e92-480e-b0fc-4ba82182fac7%7D/Name", $emailName);
    }

    public function isContactsEnabled()
    {
        return $this->isData("Options/ContentTypes/%7B0dd8685c-e272-4fcb-9ecf-2ead7ea2497b%7D/Enabled");
    }

    public function setContactsEnabled($enabled)
    {
        $this->setData("Options/ContentTypes/%7B0dd8685c-e272-4fcb-9ecf-2ead7ea2497b%7D/Enabled",
            $enabled ? "1" : "0", "Replace");
    }

    public function getContactsName()
    {
        return $this->getData("Options/ContentTypes/%7B0dd8685c-e272-4fcb-9ecf-2ead7ea2497b%7D/Name");
    }

    public function setContactsName($contactsName)
    {
        $this->setData("Options/ContentTypes/%7B0dd8685c-e272-4fcb-9ecf-2ead7ea2497b%7D/Name", $contactsName);
    }

    public function isCalendarEnabled()
    {
        return $this->isData("Options/ContentTypes/%7B4a5d9fe0-f139-4a63-a5a4-4f31ceea02ad%7D/Enabled");
    }

    public function setCalendarEnabled($enabled)
    {
        $this->setData("Options/ContentTypes/%7B4a5d9fe0-f139-4a63-a5a4-4f31ceea02ad%7D/Enabled", $enabled ? "1" : "0");
    }

    public function getCalendarName()
    {
        return $this->getData("Options/ContentTypes/%7B4a5d9fe0-f139-4a63-a5a4-4f31ceea02ad%7D/Name");
    }

    public function setCalendarName($calendarName)
    {
        $this->setData("Options/ContentTypes/%7B4a5d9fe0-f139-4a63-a5a4-4f31ceea02ad%7D/Name", $calendarName);
    }

    public function isTaskEnabled()
    {
        return $this->isData("Options/ContentTypes/%7B783ae4f6-4c12-4423-8270-66361260d4f1%7D/Enabled");
    }

    public function setTaskEnabled($enabled)
    {
        $this->setData("Options/ContentTypes/%7B783ae4f6-4c12-4423-8270-66361260d4f1%7D/Enabled", $enabled ? "1" : "0");
    }

    public function getTaskName()
    {
        return $this->getData("Options/ContentTypes/%7B783ae4f6-4c12-4423-8270-66361260d4f1%7D/Name");
    }

    public function setTaskName($taskName)
    {
        $this->setData("Options/ContentTypes/%7B783ae4f6-4c12-4423-8270-66361260d4f1%7D/Name", $taskName);
    }

    protected function getDefaults()
    {
        $defaults = array(
            "EmailAddress" => "",
            "Domain" => "",
            "AccountIcon" => "res://AccountSettingsSharedRes{ScreenResolution}!%s.office.outlook.png",
            "AccountType" => "Exchange",
            "AccountName" => "auralis Exchange",
            "Password" => "",
            "ServerName" => "",
            "UserName" => "",
            "Options/UseSSL" => "1",
        );

        return $defaults;
    }

    protected function isAllowed($locURI, $data, $command)
    {
        $data; // unused
        if ($locURI == "AccountType" && $command != "Add") {
            return false; // AccountType may not be replaced or deleted
        }

        if ($locURI == "Password" && $command == "Replace" && !$data) {
            return false; // don't set empty password
        }

        return true;
    }
}
