<?php
namespace Topalis\WinPhone\OMA\SyncML;

/**
 * This class contains different elements of a SyncML session.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class SyncMLSession
{

    /**
     *
     * @var boolean
     */
    private $abort = false;

    /**
     *
     * @var \Topalis\Device\Entity\Device
     */
    private $device;

    /**
     *
     * @var string
     */
    private $id;

    /**
     *
     * @var boolean
     */
    private $init = false;

    /**
     *
     * @var string
     */
    private $newNonce;

    /**
     *
     * @var integer
     */
    private $nextCmdID = 1;

    /**
     *
     * @var \Topalis\WinPhone\OMA\SyncML\Entity\SyncML
     */
    private $request;

    /**
     *
     * @var \Topalis\WinPhone\OMA\SyncML\Entity\SyncML
     */
    private $response;

    /**
     * @var boolean
     */
    private $serverAuthFailed = false;

    /**
     * @param \Topalis\WinPhone\OMA\SyncML\Entity\SyncML $request The SyncML request.
     */
    public function __construct(\Topalis\WinPhone\OMA\SyncML\Entity\SyncML $request = null)
    {
        $this->request = $request;
    }

    /**
     *
     * @return boolean
     */
    public function isAbort()
    {
        return $this->abort;
    }

    /**
     *
     * @param boolean $abort
     */
    public function setAbort($abort)
    {
        $this->abort = $abort;
        return $this;
    }

    /**
     *
     * @return \Topalis\Device\Entity\Device
     */
    public function getDevice()
    {
        return $this->device;
    }

    /**
     *
     * @param \Topalis\Device\Entity\Device $device
     */
    public function setDevice(\Topalis\Device\Entity\Device $device)
    {
        $this->device = $device;
        return $this;
    }

    /**
     *
     * @return string
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     *
     * @param string $id
     */
    public function setId($id)
    {
        $this->id = $id;
        return $this;
    }

    /**
     *
     * @return boolean
     */
    public function isInit()
    {
        return $this->init;
    }

    /**
     *
     * @param boolean $init
     */
    public function setInit($init)
    {
        $this->init = $init;
        return $this;
    }

    /**
     *
     * @return string
     */
    public function getNewNonce()
    {
        return $this->newNonce;
    }

    /**
     *
     * @param string $newNonce
     */
    public function setNewNonce($newNonce)
    {
        $this->newNonce = $newNonce;
        return $this;
    }

    /**
     *
     * @return int
     */
    public function getNextCmdID()
    {
        return $this->nextCmdID;
    }

    /**
     *
     * @return int
     */
    public function popNextCmdID()
    {
        return $this->nextCmdID++;
    }

    /**
     *
     * @param int $nextCmdID
     */
    public function setNextCmdID($nextCmdID)
    {
        $this->nextCmdID = $nextCmdID;
        return $this;
    }

    /**
     * @return \Topalis\WinPhone\OMA\SyncML\Entity\SyncML
     */
    public function getRequest()
    {
        return $this->request;
    }

    /**
     *
     * @param \Topalis\WinPhone\OMA\SyncML\Entity\SyncML $request
     */
    public function setRequest(\Topalis\WinPhone\OMA\SyncML\Entity\SyncML $request)
    {
        $this->request = $request;
        return $this;
    }

    /**
     * @return \Topalis\WinPhone\OMA\SyncML\Entity\SyncML
     */
    public function getResponse()
    {
        return $this->response;
    }

    /**
     *
     * @param \Topalis\WinPhone\OMA\SyncML\Entity\SyncML $response
     */
    public function setResponse(\Topalis\WinPhone\OMA\SyncML\Entity\SyncML $response)
    {
        $this->response = $response;
        return $this;
    }

    /**
     *
     * @return boolean
     */
    public function isServerAuthFailed()
    {
        return $this->serverAuthFailed;
    }

    /**
     *
     * @param boolean $serverAuthFailed
     */
    public function setServerAuthFailed($serverAuthFailed)
    {
        $this->serverAuthFailed = $serverAuthFailed;
        return $this;
    }
}
