<?php
namespace Topalis\WinPhone\Soap;

use Symfony\Component\DependencyInjection\ContainerAware;

/**
 * Base class for SOAP services to use in SoapServer.
 *
 * @see SoapServer
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class SoapService extends ContainerAware
{
    /**
     * @var SoapServer
     */
    protected $server;

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    public function getConfig()
    {
        return $this->container->get("config.auralis");
    }

    /**
     * @return \Psr\Log\LoggerInterface
     */
    public function getLogger()
    {
        return $this->container->get("logger");
    }

    /**
     * @return SoapServer
     */
    public function getServer()
    {
        return $this->server;
    }

    /**
     * Sets the SOAP server. This method is called by the SOAP server.
     *
     * @param SoapServer $server
     */
    public function setServer(SoapServer $server)
    {
        $this->server = $server;
    }

    /**
     * This method shall be called in case of an error. If this method is called
     * the SOAP body will be replaced with a corresponding SOAP fault. After
     * calling this method the SOAP service shall return null.
     *
     * @param string $code
     * @param string $subcode
     * @param string $reason
     * @param string $lang The language of the reason (IETF language tag).
     */
    public function createFault($code, $subcode, $reason, $lang = "en-US")
    {
        $this->server->createFault($code, $subcode, $reason, $lang);
    }

    /**
     * Adds the specified SOAP header to the response.
     *
     * @param \SoapHeader $soapHeader
     */
    public function addSoapHeader(\SoapHeader $soapHeader)
    {
        $this->server->addSoapHeader($soapHeader);
    }
}
