<?php
namespace Topalis\WinPhone\Wstep\Entity;

/**
 * This class stands for ContextItem originally named ContextItemType
 * Meta informations extracted from the WSDL
 * - from schema : {@link http://schemas.xmlsoap.org/ws/2006/12/authorization/ws-authorization.xsd}
 * @date 2013-12-03
 */
class ContextItem
{
    /**
     * The Value
     * Meta informations extracted from the WSDL
     * - maxOccurs : 1
     * - minOccurs : 1
     * @var string
     */
    public $Value;
    /**
     * The Name
     * Meta informations extracted from the WSDL
     * - use : required
     * @var anyURI
     */
    public $Name;
    /**
     * The any
     * @var DOMDocument
     */
    public $any;
    /**
     * The Scope
     * Meta informations extracted from the WSDL
     * - use : optional
     * @var anyURI
     */
    public $Scope;
    /**
     * Constructor method for ContextItemType
     * @uses ContextItem::setValue()
     * @uses ContextItem::setName()
     * @uses ContextItem::setAny()
     * @uses ContextItem::setScope()
     * @param  string      $_value
     * @param  anyURI      $_name
     * @param  DOMDocument $_any
     * @param  anyURI      $_scope
     * @return ContextItem
     */
    public function __construct($_value, $_name, $_any = null, $_scope = null)
    {
        $this->setValue($_value);
        $this->setName($_name);
        $this->setAny($_any);
        $this->setScope($_scope);
    }
    /**
     * Get Value value
     * @return string
     */
    public function getValue()
    {
        return $this->Value;
    }
    /**
     * Set Value value
     * @param  string $_value the Value
     * @return string
     */
    public function setValue($_value)
    {
        return ($this->Value = $_value);
    }
    /**
     * Get Name value
     * @return anyURI
     */
    public function getName()
    {
        return $this->Name;
    }
    /**
     * Set Name value
     * @param  anyURI $_name the Name
     * @return anyURI
     */
    public function setName($_name)
    {
        return ($this->Name = $_name);
    }
    /**
     * Get any value
     * @uses DOMDocument::loadXML()
     * @uses DOMDocument::hasChildNodes()
     * @uses DOMDocument::saveXML()
     * @uses DOMNode::item()
     * @uses ContextItem::setAny()
     * @param bool true or false whether to return XML value as string or as DOMDocument
     * @return DOMDocument|null
     */
    public function getAny($_asString = true)
    {
        if (!empty($this->any) && !($this->any instanceof DOMDocument)) {
            $dom = new \DOMDocument('1.0', 'UTF-8');
            $dom->formatOutput = true;
            if ($dom->loadXML($this->any)) {
                $this->setAny($dom);
            }
            unset($dom);
        }

        return ($_asString && ($this->any instanceof DOMDocument)
            && $this->any->hasChildNodes()) ? $this->any->saveXML($this->any->childNodes->item(0)) : $this->any;
    }
    /**
     * Set any value
     * @param  DOMDocument $_any the any
     * @return DOMDocument
     */
    public function setAny($_any)
    {
        return ($this->any = $_any);
    }
    /**
     * Get Scope value
     * @return anyURI|null
     */
    public function getScope()
    {
        return $this->Scope;
    }
    /**
     * Set Scope value
     * @param  anyURI $_scope the Scope
     * @return anyURI
     */
    public function setScope($_scope)
    {
        return ($this->Scope = $_scope);
    }
    /**
     * Method returning the class name
     * @return string __CLASS__
     */
    public function __toString()
    {
        return __CLASS__;
    }
}
