<?php
namespace Topalis\WinPhone\Wstep\Entity;

/**
 * This class stands for KeyInfo originally named KeyInfoType
 * Meta informations extracted from the WSDL
 * - from schema : xmldsig-core-schema.xsd
 * @date 2013-12-03
 */
class KeyInfo
{
    /**
     * The KeyName
     * @var string
     */
    public $KeyName;
    /**
     * The KeyValue
     * @var KeyValue
     */
    public $KeyValue;
    /**
     * The RetrievalMethod
     * @var RetrievalMethod
     */
    public $RetrievalMethod;
    /**
     * The X509Data
     * @var X509Data
     */
    public $X509Data;
    /**
     * The PGPData
     * @var PGPData
     */
    public $PGPData;
    /**
     * The SPKIData
     * @var SPKIData
     */
    public $SPKIData;
    /**
     * The MgmtData
     * @var string
     */
    public $MgmtData;
    /**
     * The any
     * @var DOMDocument
     */
    public $any;
    /**
     * The Id
     * Meta informations extracted from the WSDL
     * - use : optional
     * @var ID
     */
    public $Id;
    /**
     * Constructor method for KeyInfoType
     * @uses KeyInfo::setKeyName()
     * @uses KeyInfo::setKeyValue()
     * @uses KeyInfo::setRetrievalMethod()
     * @uses KeyInfo::setX509Data()
     * @uses KeyInfo::setPGPData()
     * @uses KeyInfo::setSPKIData()
     * @uses KeyInfo::setMgmtData()
     * @uses KeyInfo::setAny()
     * @uses KeyInfo::setId()
     * @param  string          $_keyName
     * @param  KeyValue        $_keyValue
     * @param  RetrievalMethod $_retrievalMethod
     * @param  X509Data        $_x509Data
     * @param  PGPData         $_pGPData
     * @param  SPKIData        $_sPKIData
     * @param  string          $_mgmtData
     * @param  DOMDocument     $_any
     * @param  ID              $_id
     * @return KeyInfo
     */
    public function __construct($_keyName = null, $_keyValue = null, $_retrievalMethod = null,
        $_x509Data = null, $_pGPData = null, $_sPKIData = null, $_mgmtData = null, $_any = null, $_id = null)
    {
        $this->setKeyName($_keyName);
        $this->setKeyValue($_keyValue);
        $this->setRetrievalMethod($_retrievalMethod);
        $this->setX509Data($_x509Data);
        $this->setPGPData($_pGPData);
        $this->setSPKIData($_sPKIData);
        $this->setMgmtData($_mgmtData);
        $this->setAny($_any);
        $this->setId($_id);
    }
    /**
     * Get KeyName value
     * @return string|null
     */
    public function getKeyName()
    {
        return $this->KeyName;
    }
    /**
     * Set KeyName value
     * @param  string $_keyName the KeyName
     * @return string
     */
    public function setKeyName($_keyName)
    {
        return ($this->KeyName = $_keyName);
    }
    /**
     * Get KeyValue value
     * @return KeyValue|null
     */
    public function getKeyValue()
    {
        return $this->KeyValue;
    }
    /**
     * Set KeyValue value
     * @param  KeyValue $_keyValue the KeyValue
     * @return KeyValue
     */
    public function setKeyValue($_keyValue)
    {
        return ($this->KeyValue = $_keyValue);
    }
    /**
     * Get RetrievalMethod value
     * @return RetrievalMethod|null
     */
    public function getRetrievalMethod()
    {
        return $this->RetrievalMethod;
    }
    /**
     * Set RetrievalMethod value
     * @param  RetrievalMethod $_retrievalMethod the RetrievalMethod
     * @return RetrievalMethod
     */
    public function setRetrievalMethod($_retrievalMethod)
    {
        return ($this->RetrievalMethod = $_retrievalMethod);
    }
    /**
     * Get X509Data value
     * @return X509Data|null
     */
    public function getX509Data()
    {
        return $this->X509Data;
    }
    /**
     * Set X509Data value
     * @param  X509Data $_x509Data the X509Data
     * @return X509Data
     */
    public function setX509Data($_x509Data)
    {
        return ($this->X509Data = $_x509Data);
    }
    /**
     * Get PGPData value
     * @return PGPData|null
     */
    public function getPGPData()
    {
        return $this->PGPData;
    }
    /**
     * Set PGPData value
     * @param  PGPData $_pGPData the PGPData
     * @return PGPData
     */
    public function setPGPData($_pGPData)
    {
        return ($this->PGPData = $_pGPData);
    }
    /**
     * Get SPKIData value
     * @return SPKIData|null
     */
    public function getSPKIData()
    {
        return $this->SPKIData;
    }
    /**
     * Set SPKIData value
     * @param  SPKIData $_sPKIData the SPKIData
     * @return SPKIData
     */
    public function setSPKIData($_sPKIData)
    {
        return ($this->SPKIData = $_sPKIData);
    }
    /**
     * Get MgmtData value
     * @return string|null
     */
    public function getMgmtData()
    {
        return $this->MgmtData;
    }
    /**
     * Set MgmtData value
     * @param  string $_mgmtData the MgmtData
     * @return string
     */
    public function setMgmtData($_mgmtData)
    {
        return ($this->MgmtData = $_mgmtData);
    }
    /**
     * Get any value
     * @uses DOMDocument::loadXML()
     * @uses DOMDocument::hasChildNodes()
     * @uses DOMDocument::saveXML()
     * @uses DOMNode::item()
     * @uses KeyInfo::setAny()
     * @param bool true or false whether to return XML value as string or as DOMDocument
     * @return DOMDocument|null
     */
    public function getAny($_asString = true)
    {
        if (!empty($this->any) && !($this->any instanceof DOMDocument)) {
            $dom = new \DOMDocument('1.0', 'UTF-8');
            $dom->formatOutput = true;
            if ($dom->loadXML($this->any)) {
                $this->setAny($dom);
            }
            unset($dom);
        }

        return ($_asString && ($this->any instanceof DOMDocument)
            && $this->any->hasChildNodes()) ? $this->any->saveXML($this->any->childNodes->item(0)) : $this->any;
    }
    /**
     * Set any value
     * @param  DOMDocument $_any the any
     * @return DOMDocument
     */
    public function setAny($_any)
    {
        return ($this->any = $_any);
    }
    /**
     * Get Id value
     * @return ID|null
     */
    public function getId()
    {
        return $this->Id;
    }
    /**
     * Set Id value
     * @param  ID $_id the Id
     * @return ID
     */
    public function setId($_id)
    {
        return ($this->Id = $_id);
    }
    /**
     * Method returning the class name
     * @return string __CLASS__
     */
    public function __toString()
    {
        return __CLASS__;
    }
}
