<?php
namespace Topalis\WinPhone\Wstep\Entity;

/**
 * This class stands for SignatureMethod originally named SignatureMethodType
 * Meta informations extracted from the WSDL
 * - from schema : xmldsig-core-schema.xsd
 * @date 2013-12-03
 */
class SignatureMethod
{
    /**
     * The Algorithm
     * Meta informations extracted from the WSDL
     * - use : required
     * @var anyURI
     */
    public $Algorithm;
    /**
     * The HMACOutputLength
     * Meta informations extracted from the WSDL
     * - minOccurs : 0
     * @var integer
     */
    public $HMACOutputLength;
    /**
     * The any
     * @var DOMDocument
     */
    public $any;
    /**
     * Constructor method for SignatureMethodType
     * @uses SignatureMethod::setAlgorithm()
     * @uses SignatureMethod::setHMACOutputLength()
     * @uses SignatureMethod::setAny()
     * @param  anyURI          $_algorithm
     * @param  integer         $_hMACOutputLength
     * @param  DOMDocument     $_any
     * @return SignatureMethod
     */
    public function __construct($_algorithm, $_hMACOutputLength = null, $_any = null)
    {
        $this->setAlgorithm($_algorithm);
        $this->setHMACOutputLength($_hMACOutputLength);
        $this->setAny($_any);
    }
    /**
     * Get Algorithm value
     * @return anyURI
     */
    public function getAlgorithm()
    {
        return $this->Algorithm;
    }
    /**
     * Set Algorithm value
     * @param  anyURI $_algorithm the Algorithm
     * @return anyURI
     */
    public function setAlgorithm($_algorithm)
    {
        return ($this->Algorithm = $_algorithm);
    }
    /**
     * Get HMACOutputLength value
     * @return integer|null
     */
    public function getHMACOutputLength()
    {
        return $this->HMACOutputLength;
    }
    /**
     * Set HMACOutputLength value
     * @param  integer $_hMACOutputLength the HMACOutputLength
     * @return integer
     */
    public function setHMACOutputLength($_hMACOutputLength)
    {
        return ($this->HMACOutputLength = $_hMACOutputLength);
    }
    /**
     * Get any value
     * @uses DOMDocument::loadXML()
     * @uses DOMDocument::hasChildNodes()
     * @uses DOMDocument::saveXML()
     * @uses DOMNode::item()
     * @uses SignatureMethod::setAny()
     * @param bool true or false whether to return XML value as string or as DOMDocument
     * @return DOMDocument|null
     */
    public function getAny($_asString = true)
    {
        if (!empty($this->any) && !($this->any instanceof DOMDocument)) {
            $dom = new \DOMDocument('1.0', 'UTF-8');
            $dom->formatOutput = true;
            if ($dom->loadXML($this->any)) {
                $this->setAny($dom);
            }
            unset($dom);
        }

        return ($_asString && ($this->any instanceof DOMDocument)
            && $this->any->hasChildNodes()) ? $this->any->saveXML($this->any->childNodes->item(0)) : $this->any;
    }
    /**
     * Set any value
     * @param  DOMDocument $_any the any
     * @return DOMDocument
     */
    public function setAny($_any)
    {
        return ($this->any = $_any);
    }
    /**
     * Method returning the class name
     * @return string __CLASS__
     */
    public function __toString()
    {
        return __CLASS__;
    }
}
