<?php
namespace Topalis\WinPhone\Wstep\Entity;

/**
 * This class stands for UsernameToken originally named UsernameTokenType
 * Documentation : This type represents a username token per Section 4.1
 * Meta informations extracted from the WSDL
 * - from schema : oasis-200401-wss-wssecurity-secext-1.0.xsd
 * @date 2013-12-03
 */
class UsernameToken
{
    /**
     * The Username
     * @var AttributedString
     */
    public $Username;
    /**
     * The any
     * @var DOMDocument
     */
    public $any;
    /**
     * The Id
     * @var ID
     */
    public $Id;
    /**
     * Constructor method for UsernameTokenType
     * @uses UsernameToken::setUsername()
     * @uses UsernameToken::setAny()
     * @uses UsernameToken::setId()
     * @param  AttributedString $_username
     * @param  DOMDocument      $_any
     * @param  ID               $_id
     * @return UsernameToken
     */
    public function __construct($_username = null, $_any = null, $_id = null)
    {
        $this->setUsername($_username);
        $this->setAny($_any);
        $this->setId($_id);
    }
    /**
     * Get Username value
     * @return AttributedString|null
     */
    public function getUsername()
    {
        return $this->Username;
    }
    /**
     * Set Username value
     * @param  AttributedString $_username the Username
     * @return AttributedString
     */
    public function setUsername($_username)
    {
        return ($this->Username = $_username);
    }
    /**
     * Get any value
     * @uses DOMDocument::loadXML()
     * @uses DOMDocument::hasChildNodes()
     * @uses DOMDocument::saveXML()
     * @uses DOMNode::item()
     * @uses UsernameToken::setAny()
     * @param bool true or false whether to return XML value as string or as DOMDocument
     * @return DOMDocument|null
     */
    public function getAny($_asString = true)
    {
        if (!empty($this->any) && !($this->any instanceof DOMDocument)) {
            $dom = new \DOMDocument('1.0', 'UTF-8');
            $dom->formatOutput = true;
            if ($dom->loadXML($this->any)) {
                $this->setAny($dom);
            }
            unset($dom);
        }

        return ($_asString && ($this->any instanceof DOMDocument)
            && $this->any->hasChildNodes()) ? $this->any->saveXML($this->any->childNodes->item(0)) : $this->any;
    }
    /**
     * Set any value
     * @param  DOMDocument $_any the any
     * @return DOMDocument
     */
    public function setAny($_any)
    {
        return ($this->any = $_any);
    }
    /**
     * Get Id value
     * @return ID|null
     */
    public function getId()
    {
        return $this->Id;
    }
    /**
     * Set Id value
     * @param  ID $_id the Id
     * @return ID
     */
    public function setId($_id)
    {
        return ($this->Id = $_id);
    }
    /**
     * Method returning the class name
     * @return string __CLASS__
     */
    public function __toString()
    {
        return __CLASS__;
    }
}
