<?php
namespace Topalis\WinPhone\Xcep\Entity;

/**
 * This class stands for CA originally named CA
 * Meta informations extracted from the WSDL
 * - from schema : xcep.xsd
 * @date 2013-12-10
 */
class CA
{
    /**
     * The uris
     * @var CAURICollection
     */
    public $uris;

    /**
     * The certificate
     * @var base64Binary
     */
    public $certificate;

    /**
     * The enrollPermission
     * @var boolean
     */
    public $enrollPermission;

    /**
     * The cAReferenceID
     * @var int
     */
    public $cAReferenceID;

    /**
     * The any
     * @var DOMDocument
     */
    public $any;

    /**
     * Constructor method for CA
     * @uses CA::setUris()
     * @uses CA::setCertificate()
     * @uses CA::setEnrollPermission()
     * @uses CA::setCAReferenceID()
     * @uses CA::setAny()
     * @param CAURICollection $_uris
     * @param base64Binary $_certificate
     * @param boolean $_enrollPermission
     * @param int $_cAReferenceID
     * @param DOMDocument $_any
     * @return CA
     */
    public function __construct($_uris = null, $_certificate = null, $_enrollPermission = null,
        $_cAReferenceID = null, $_any = null)
    {
        $this->setUris($_uris);
        $this->setCertificate($_certificate);
        $this->setEnrollPermission($_enrollPermission);
        $this->setCAReferenceID($_cAReferenceID);
        $this->setAny($_any);
    }

    /**
     * Get uris value
     * @return CAURICollection|null
     */
    public function getUris()
    {
        return $this->uris;
    }

    /**
     * Set uris value
     * @param CAURICollection $_uris the uris
     * @return CAURICollection
     */
    public function setUris($_uris)
    {
        return ($this->uris = $_uris);
    }

    /**
     * Get certificate value
     * @return base64Binary|null
     */
    public function getCertificate()
    {
        return $this->certificate;
    }

    /**
     * Set certificate value
     * @param base64Binary $_certificate the certificate
     * @return base64Binary
     */
    public function setCertificate($_certificate)
    {
        return ($this->certificate = $_certificate);
    }

    /**
     * Get enrollPermission value
     * @return boolean|null
     */
    public function getEnrollPermission()
    {
        return $this->enrollPermission;
    }

    /**
     * Set enrollPermission value
     * @param boolean $_enrollPermission the enrollPermission
     * @return boolean
     */
    public function setEnrollPermission($_enrollPermission)
    {
        return ($this->enrollPermission = $_enrollPermission);
    }

    /**
     * Get cAReferenceID value
     * @return int|null
     */
    public function getCAReferenceID()
    {
        return $this->cAReferenceID;
    }

    /**
     * Set cAReferenceID value
     * @param int $_cAReferenceID the cAReferenceID
     * @return int
     */
    public function setCAReferenceID($_cAReferenceID)
    {
        return ($this->cAReferenceID = $_cAReferenceID);
    }

    /**
     * Get any value
     * @uses DOMDocument::loadXML()
     * @uses DOMDocument::hasChildNodes()
     * @uses DOMDocument::saveXML()
     * @uses DOMNode::item()
     * @uses CA::setAny()
     * @param bool true or false whether to return XML value as string or as DOMDocument
     * @return DOMDocument|null
     */
    public function getAny($_asString = true)
    {
        if (!empty($this->any) && !($this->any instanceof DOMDocument)) {
            $dom = new \DOMDocument('1.0', 'UTF-8');
            $dom->formatOutput = true;
            if ($dom->loadXML($this->any)) {
                $this->setAny($dom);
            }

            unset($dom);
        }

        return ($_asString && ($this->any instanceof DOMDocument)
            && $this->any->hasChildNodes()) ? $this->any->saveXML($this->any->childNodes->item(0)) : $this->any;
    }

    /**
     * Set any value
     * @param DOMDocument $_any the any
     * @return DOMDocument
     */
    public function setAny($_any)
    {
        return ($this->any = $_any);
    }

    /**
     * Method returning the class name
     * @return string __CLASS__
     */
    public function __toString()
    {
        return __CLASS__;
    }

}
