<?php
namespace Topalis\WinPhone\Xcep\Entity;

/**
 * This class stands for OID originally named OID
 * Meta informations extracted from the WSDL
 * - from schema : xcep.xsd
 * @date 2013-12-10
 */
class OID
{
    /**
     * The value
     * @var string
     */
    public $value;

    /**
     * The group
     * @var unsignedInt
     */
    public $group;

    /**
     * The oIDReferenceID
     * @var int
     */
    public $oIDReferenceID;

    /**
     * The defaultName
     * Meta informations extracted from the WSDL
     * - nillable : true
     * @var string
     */
    public $defaultName;

    /**
     * The any
     * @var DOMDocument
     */
    public $any;

    /**
     * Constructor method for OID
     * @uses OID::setValue()
     * @uses OID::setGroup()
     * @uses OID::setOIDReferenceID()
     * @uses OID::setDefaultName()
     * @uses OID::setAny()
     * @param string $_value
     * @param unsignedInt $_group
     * @param int $_oIDReferenceID
     * @param string $_defaultName
     * @param DOMDocument $_any
     * @return OID
     */
    public function __construct($_value = null, $_group = null, $_oIDReferenceID = null,
        $_defaultName = null, $_any = null)
    {
        $this->setValue($_value);
        $this->setGroup($_group);
        $this->setOIDReferenceID($_oIDReferenceID);
        $this->setDefaultName($_defaultName);
        $this->setAny($_any);
    }

    /**
     * Get value value
     * @return string|null
     */
    public function getValue()
    {
        return $this->value;
    }

    /**
     * Set value value
     * @param string $_value the value
     * @return string
     */
    public function setValue($_value)
    {
        return ($this->value = $_value);
    }

    /**
     * Get group value
     * @return unsignedInt|null
     */
    public function getGroup()
    {
        return $this->group;
    }

    /**
     * Set group value
     * @param unsignedInt $_group the group
     * @return unsignedInt
     */
    public function setGroup($_group)
    {
        return ($this->group = $_group);
    }

    /**
     * Get oIDReferenceID value
     * @return int|null
     */
    public function getOIDReferenceID()
    {
        return $this->oIDReferenceID;
    }

    /**
     * Set oIDReferenceID value
     * @param int $_oIDReferenceID the oIDReferenceID
     * @return int
     */
    public function setOIDReferenceID($_oIDReferenceID)
    {
        return ($this->oIDReferenceID = $_oIDReferenceID);
    }

    /**
     * Get defaultName value
     * @return string|null
     */
    public function getDefaultName()
    {
        return $this->defaultName;
    }

    /**
     * Set defaultName value
     * @param string $_defaultName the defaultName
     * @return string
     */
    public function setDefaultName($_defaultName)
    {
        return ($this->defaultName = $_defaultName);
    }

    /**
     * Get any value
     * @uses DOMDocument::loadXML()
     * @uses DOMDocument::hasChildNodes()
     * @uses DOMDocument::saveXML()
     * @uses DOMNode::item()
     * @uses OID::setAny()
     * @param bool true or false whether to return XML value as string or as DOMDocument
     * @return DOMDocument|null
     */
    public function getAny($_asString = true)
    {
        if (!empty($this->any) && !($this->any instanceof DOMDocument)) {
            $dom = new \DOMDocument('1.0', 'UTF-8');
            $dom->formatOutput = true;
            if ($dom->loadXML($this->any)) {
                $this->setAny($dom);
            }

            unset($dom);
        }

        return ($_asString && ($this->any instanceof DOMDocument)
            && $this->any->hasChildNodes()) ? $this->any->saveXML($this->any->childNodes->item(0)) : $this->any;
    }

    /**
     * Set any value
     * @param DOMDocument $_any the any
     * @return DOMDocument
     */
    public function setAny($_any)
    {
        return ($this->any = $_any);
    }

    /**
     * Method returning the class name
     * @return string __CLASS__
     */
    public function __toString()
    {
        return __CLASS__;
    }

}
