<?php
namespace Topalis\WinPhone\Xcep;

use Topalis\WinPhone\Xcep\Entity\GetPolicies;
use Topalis\WinPhone\Xcep\Entity\GetPoliciesResponse;
use Topalis\WinPhone\Xcep\Entity\Response;
use Topalis\WinPhone\Xcep\Entity\PolicyCollection;
use Topalis\WinPhone\Xcep\Entity\CertificateEnrollmentPolicy;
use Topalis\WinPhone\Xcep\Entity\Attributes;
use Topalis\WinPhone\Xcep\Entity\PrivateKeyAttributes;
use Topalis\WinPhone\Xcep\Entity\OIDCollection;
use Topalis\WinPhone\Xcep\Entity\OID;
use Topalis\WinPhone\Xcep\Entity\CA;
use Topalis\WinPhone\Xcep\Entity\CACollection;
use Topalis\WinPhone\Xcep\Entity\CAReferenceCollection;
use Topalis\WinPhone\Xcep\Entity\CAURICollection;
use Topalis\WinPhone\Xcep\Entity\CAURI;
use Topalis\WinPhone\Soap\SoapServiceAuth;

/**
 * Implements the MS-XCEP protocol for the rollout of Windows Phone devices.
 *
 * @see http://msdn.microsoft.com/en-us/library/dd302869.aspx
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class XcepService extends SoapServiceAuth
{
    private $messageId;

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    private function addResponseHeaders()
    {
        $xmlnsA = "http://www.w3.org/2006/05/addressing";
        $action = "http://schemas.microsoft.com/windows/pki/2009/01/enrollmentpolicy/IPolicy/GetPoliciesResponse";
        $this->addSoapHeader(new \SoapHeader($xmlnsA, "Action", $action, true));
        $this->addSoapHeader(new \SoapHeader($xmlnsA, "RelatesTo", $this->messageId));
    }

    /*
     * Header begin
     */

    // @codingStandardsIgnoreStart
    public function Action($action)
    {
        // nothing to do
    }
    // @codingStandardsIgnoreEnd

    // @codingStandardsIgnoreStart
    public function Address($address)
    {
        // nothing to do
    }
    // @codingStandardsIgnoreEnd

    // @codingStandardsIgnoreStart
    public function MessageID($messageId)
    {
        $this->messageId = $messageId;
    }
    // @codingStandardsIgnoreEnd

    // @codingStandardsIgnoreStart
    public function ReplyTo($replyTo)
    {
        // noting to do
    }
    // @codingStandardsIgnoreEnd

    // @codingStandardsIgnoreStart
    public function To($to)
    {
        // nothing to do
    }
    // @codingStandardsIgnoreEnd

    /*
     * Header end
     */

    // @codingStandardsIgnoreStart
    public function GetPolicies($request)
    {
        $request; // unused

        $this->addResponseHeaders();

        if (!$this->checkAuth()) {
            return null;
        }

        $policiesResponse = new GetPoliciesResponse();
        $response = new Response();

        $policy = new CertificateEnrollmentPolicy();
        $policy->setPolicyOIDReference(0);
        $policy->setCAs(new CAReferenceCollection(0));
        $attributes = new Attributes();
        $attributes->setPolicySchema(3);
        $attributes->setCommonName("auralis Enrollment Policy");
        $privateKeyAttributes = new PrivateKeyAttributes();
        $privateKeyAttributes->setMinimalKeyLength(2048);
        $privateKeyAttributes->setKeyUsageProperty(7); // digitalSignature, nonRepudiation, dataEncipherment
        $attributes->setPrivateKeyAttributes($privateKeyAttributes);
        $attributes->setHashAlgorithmOIDReference(0);
        $policy->setAttributes($attributes);

        $response->setPolicies(new PolicyCollection($policy));
        $policiesResponse->setResponse($response);

        $oid = new OID();
        $oid->setValue("1.3.14.3.2.29");
        $oid->setGroup(1);
        $oid->setOIDReferenceID(0);
        $oid->setDefaultName("szOID_OIWSEC_sha1RSASign");
        $policiesResponse->setOIDs(new OIDCollection($oid));

        $ca = new CA();
        $ca->setCAReferenceID(0);
        $caUri = new CAURI();
        $caUri->setClientAuthentication(1);
        $caUri->setUri("https://" . $this->getConfig()->getHostname() . ":"
            . $this->getConfig()->getSecondaryPort() . "/cacert.cer");
        $caUri->setRenewalOnly(false);
        $ca->setUris(new CAURICollection($caUri));
        $ca->setCertificate(base64_encode($this->getCertificateManager()->getCACertificate()->getDer()));
        $ca->setEnrollPermission(true);
        $policiesResponse->setCAs(new CACollection($ca));

        return $policiesResponse;
    }
    // @codingStandardsIgnoreEnd
}
