<?php
namespace Topalis\Certificate\Controller;

use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Component\HttpFoundation\Response;
use Topalis\Certificate\Certificate;
use Topalis\Certificate\Exception\CertificateException;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;

/**
 *
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 * @Route("/admin/config/cert/ca", service="cacert.controller")
 */
class CACertController extends AbstractCertController
{
    /**
     * @Route("", name="admin_cacert")
     * @Template("TopalisCertificateBundle:Cert:ca_cert.html.twig")
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|\Symfony\Component\HttpFoundation\Response
     */
    // avoid useless method override warning
    // @codingStandardsIgnoreStart
    public function indexAction()
    {
        return parent::indexAction();
    }
    // @codingStandardsIgnoreEnd

    /**
     * @Route("/edit", name="admin_cacert_edit")
     * @Template("TopalisCertificateBundle:Cert:ca_cert.html.twig")
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|\Symfony\Component\HttpFoundation\Response
     */
    // avoid useless method override warning
    // @codingStandardsIgnoreStart
    public function editAction()
    {
        return parent::editAction();
    }
    // @codingStandardsIgnoreEnd

    /**
     * @return \Topalis\Certificate\Certificate
     */
    protected function getCertificate()
    {
        $certMgr = $this->getCertificateManager();
        $cert = $certMgr->getCACertificate();
        return ($cert !== null) ? $cert : new Certificate(null, null,
            array("config" => $certMgr->getSslConfigFilename()));
    }

    protected function getPathBaseName()
    {
        return "admin_cacert";
    }

    protected function getTemplate()
    {
        return "admin/caCert.html.twig";
    }

    protected function getUploadFormLegend()
    {
        return $this->tr("admin.cert.ca.upload");
    }

    protected function getFormLegend()
    {
        return $this->tr("admin.cert.ca.create");
    }

    protected function handleEdit(Certificate $certificate)
    {
        $data = $certificate->export(null);
        $privateKey = $certificate->exportPrivateKey();

        $config = $this->getConfig();
        $config->setCACert($data);
        $config->setCAPrivateKey($privateKey);
        $config->setCAChain(serialize(array()));

        $this->getCertificateManager()->exportCACert();

        $this->addFlash("success", $this->tr("admin.cert.saveSuccess"));
        return $this->redirect($this->generateUrl("admin_cacert"));
    }

    protected function handleUpload(array $data)
    {
        $content = file_get_contents($data["file"]->getPathname());
        $certs = array();
        openssl_pkcs12_read($content, $certs, $data['password']);

        $caCertificate = null;
        $privateKey = null;
        try {
            openssl_x509_export($certs['cert'], $caCertificate);
            openssl_pkey_export($certs['pkey'], $privateKey);
        } catch (\Exception $e) {
            $msg = "[openssl]: " . $e->getMessage();
            throw new CertificateException($msg, $e->getCode(), $e);
        }

        $chain = array();
        if (isset($certs['extracerts']) && is_array($certs['extracerts'])) {
            $chain = $certs['extracerts'];
        }

        $this->getDB()->beginTransaction();
        try {
            $config = $this->getConfig();
            $config->setCAChain(serialize($chain));
            $config->setCACert($caCertificate);
            $config->setCAPrivateKey($privateKey);

            $this->getCertificateManager()->exportCACert();
            $this->getDB()->commitTransaction();
            return $this->redirect($this->generateUrl("admin_cacert"));
        } catch (\Exception $e) {
            $this->getDB()->rollbackTransaction();
            throw $e;
        }
    }

    /**
     * @return string name of the active subsection
     */
    protected function getSubSection()
    {
        return "ca";
    }
}
