<?php
namespace Topalis\Certificate\Form;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Topalis\Certificate\Validator\CertificateField;
use Topalis\Locale\Form\Choice\CountryChoiceList;

/**
 * Certificate form used for server and ca cert
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class CertForm extends AbstractType
{
    protected $fqdn = false;

    public function __construct($fqdn = false)
    {
        $this->fqdn = $fqdn;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $options; // unused
        $builder
            ->add("country", "choice", array(
                "label" => "admin.cert.country",
                "choice_list" => new CountryChoiceList()
            ))
            ->add("state", "text", array(
                "label" => "admin.cert.state",
                "constraints" => new CertificateField()
            ))
            ->add("location", "text", array(
                "label" => "admin.cert.locality",
                "constraints" => new CertificateField()
            ))
            ->add("organization", "text", array(
                "label" => "admin.cert.organization",
                "constraints" => new CertificateField()
            ))
            ->add("organizationalUnit", "text", array(
                "label" => "admin.cert.ou",
                "constraints" => new CertificateField(),
                "required" => false
            ))
            ->add("commonName", "text", array(
                "label" => ($this->fqdn ? "admin.cert.fqdn" : "admin.cert.commonName"),
                "constraints" => new CertificateField()
            ))
            ->add("email", "email")
        ;
    }

    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(array(
            "data_class" => "Topalis\Certificate\Certificate",
        ));
    }

    /**
     * Returns the name of this type.
     * @return string The name of this type
     */
    public function getName()
    {
        return "cert";
    }
}
