<?php
namespace Topalis\Certificate\Validator;

use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Exception\UnexpectedTypeException;

/**
 * check whether hostname equals config hostname, but covers wildcards.
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class HostnameValidator extends ConstraintValidator
{
    /**
     * @var \Topalis\Configuration\AuralisConfig
     */
    private $config;

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    public function getConfig()
    {
        return $this->config;
    }

    /**
     * @param \Topalis\Configuration\AuralisConfig $config
     */
    public function setConfig(\Topalis\Configuration\AuralisConfig $config)
    {
        $this->config = $config;
    }

    public function validate($value, Constraint $constraint)
    {
        /** @var $constraint Hostname */
        if (null === $value || '' === $value) {
            return;
        }

        if (!is_scalar($value) && !(is_object($value) && method_exists($value, '__toString'))) {
            throw new UnexpectedTypeException($value, 'string');
        }

        $hostname = $this->getConfig()->getHostname();
        if (!fnmatch($value, $hostname, FNM_CASEFOLD)) {
            $this->context->addViolation($constraint->message);
        }

    }

}
