<?php
namespace Topalis\Certificate\Validator;

use Symfony\Component\HttpFoundation\File\UploadedFile;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Exception\UnexpectedTypeException;

/**
 * validates a push certificate
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class PushCertificateValidator extends ConstraintValidator
{
    /**
     * @var \Topalis\Configuration\AuralisConfig
     */
    private $config;

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    public function getConfig()
    {
        return $this->config;
    }

    /**
     * @param \Topalis\Configuration\AuralisConfig $config
     */
    public function setConfig(\Topalis\Configuration\AuralisConfig $config)
    {
        $this->config = $config;
    }

    public function validate($value, Constraint $constraint)
    {
        /** @var $constraint PushCertificate */
        if (null === $value || '' === $value) {
            return;
        }

        if (!$value instanceof UploadedFile) {
            throw new UnexpectedTypeException($value, "Symfony\Component\HttpFoundation\File\UploadedFile");
        }
        $content = file_get_contents($value->getRealPath());
        if (!openssl_x509_check_private_key($content, $this->getConfig()->getApnsPrivKey())) {
            $this->context->addViolation($constraint->privateKeyMissmatch);
        }
    }
}
