<?php
namespace Topalis\Certificate\Validator;

use Symfony\Component\HttpFoundation\File\File;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Exception\UnexpectedTypeException;

/**
 *
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class ServerCertificateValidator extends ConstraintValidator
{
    public function validate($value, Constraint $constraint)
    {
        /** @var $constraint ServerCertificate */
        if (null === $value || '' === $value) {
            return;
        }

        if (!is_array($value)) {
            throw new UnexpectedTypeException($value, "array");
        }

        $value += array("file" => null, "password" => null);

        if (!$value['file'] instanceof File) {
            throw new UnexpectedTypeException($value, "Symfony\Component\HttpFoundation\File\File");
        }

        $content = file_get_contents($value['file']->getPathname());
        $certs = array();
        if (false === openssl_pkcs12_read($content, $certs, $value['password'])) {
            $openSslMsg = openssl_error_string();
            $this->context->addViolation($constraint->invalidPKCS12Message, array("%errmsg%" => $openSslMsg));

            return false;
        }

        $x509 = openssl_x509_parse($certs['cert']);
        if ($x509['subject'] == $x509['issuer']) {
            $this->context->addViolation($constraint->selfSignedMessage);

            return false;
        }

        if (isset($x509['extensions']['basicConstraints']) && $x509['extensions']['basicConstraints'] == 'CA:TRUE') {
            $this->context->addViolation($constraint->isCAMessage);
        }

        // TODO: Completely(!!) verify CA cert against chain.
    }

}
