<?php
namespace Topalis\Profile;

/**
 * Generates uuids to be used in profiles
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class UUIDGenerator
{
    protected $uuids = array();

    /**
     * add a already used uuid to prevent duplicates
     * @param string $uuid
     */
    public function addUUID($uuid)
    {
        if (!$this->isUsed($uuid)) {
            $this->uuids[] = $uuid;
        }
    }

    /**
     * Clears all UUIDs.
     */
    public function clear()
    {
        $this->uuids = array();
    }

    /**
     * Generates a new UUID.
     *
     * @return string The newly generated UUID.
     */
    public function generateUUID()
    {
        do {
            $ret = sprintf(
                '%04X%04X-%04X-%04X-%04X-%04X%04X%04X',
                mt_rand(0, 0xffff),
                mt_rand(0, 0xffff),
                mt_rand(0, 0xffff),
                mt_rand(0, 0x0fff) | 0x4000,
                mt_rand(0, 0x3fff) | 0x8000,
                mt_rand(0, 0xffff),
                mt_rand(0, 0xffff),
                mt_rand(0, 0xffff)
            );
        } while ($this->isUsed($ret));
        $this->uuids[] = $ret;
        return $ret;
    }

    /**
     * Checks if the specified UUID is already in use.
     *
     * @param string $uuid The UUID to check.
     * @return boolean true if the specified UUID is already used, false
     *     oterwise.
     */
    public function isUsed($uuid)
    {
        return in_array($uuid, $this->uuids);
    }

}
