<?php
namespace Topalis\Scep;

/**
 * This class contains constants used in SCEP.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class ScepConstants
{
    /**
     * PKCS#10 [RFC2986] certificate request
     * @var int
     */
    const MESSAGE_TYPE_PKCSREQ = 19;

    /**
     * Response to certificate or CRL request
     * @var int
     */
    const MESSAGE_TYPE_CERTREP = 3;

    /**
     * request granted
     * @var int
     */
    const PKISTATUS_SUCCESS = 0;

    /**
     * request rejected. When pkiStatus is FAILURE, the failInfo attribute, as
     * defined in Section 3.1.1.4, MUST also be present.
     * @var int
     */
    const PKISTATUS_FAILURE = 2;

    /**
     * Unrecognized or unsupported algorithm identifier
     * @var int
     */
    const FAILINFO_BAD_ALG = 0;

    /**
     * integrity check failed
     * @var int
     */
    const FAILINFO_BAD_MESSAGE_CHECK = 1;

    /**
     * transaction not permitted or supported
     * @var int
     */
    const FAILINFO_BAD_REQUEST = 2;

    /**
     * The signingTime attribute from the PKCS#7 authenticatedAttributes was not
     * sufficiently close to the system time (see Section 3.1.1.6).
     * @var int
     */
    const FAILINFO_BAD_TIME = 3;

    /**
     * No certificate could be identified matching the provided criteria
     * @var int
     */
    const FAILINFO_BAD_CERT_ID = 4;

    /**
     * OIDs used in SCEP
     *
     * @var array
     */
    public static $oids = array(
        "messageType" => "2.16.840.1.113733.1.9.2",
        "pkiStatus" => "2.16.840.1.113733.1.9.3",
        "failInfo" => "2.16.840.1.113733.1.9.4",
        "senderNonce" => "2.16.840.1.113733.1.9.5",
        "recipientNonce" => "2.16.840.1.113733.1.9.6",
        "transactionID" => "2.16.840.1.113733.1.9.7",
        "extensionReq" => "2.16.840.1.113733.1.9.8"
    );

    /**
     * Mapping from OIDs to their PKCS#7 data type.
     *
     * @var array
     */
    public static $oidTypes = array(
        "2.16.840.1.113733.1.9.2" => "PRINTABLESTRING",
        "2.16.840.1.113733.1.9.3" => "PRINTABLESTRING",
        "2.16.840.1.113733.1.9.4" => "PRINTABLESTRING",
        "2.16.840.1.113733.1.9.5" => "OCTET_STRING",
        "2.16.840.1.113733.1.9.6" => "OCTET_STRING",
        "2.16.840.1.113733.1.9.7" => "PRINTABLESTRING",
        "2.16.840.1.113733.1.9.8" => "PRINTABLESTRING"
    );
}
