<?php
namespace Topalis\System\Validator;

use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Exception\UnexpectedTypeException;

/**
 *
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class DynamicIPValidator extends ConstraintValidator
{
    public function validate($value, Constraint $constraint)
    {
        /** @var $constraint DynamicIP */
        if (null === $value || '' === $value) {
            return;
        }

        if (!is_array($value)) {
            throw new UnexpectedTypeException($value, "array");
        }

        $protocol = $value['bootProto'];

        $requiredFields = array("ipAddress", "netmask", "gateway", "dns1");

        if ("dhcp" != $protocol) {
            foreach ($requiredFields as $field) {
                if (null != $value[$field]) {
                    continue;
                }
                $this->context->addViolationAtSubPath("[" . $field. "]", $constraint->message, array(), $value[$field]);
            }
        }
    }

}
