<?php
namespace Topalis\Util;

/**
 * Class with file system specific helper functions.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class FileSystemUtil
{
    /**
     * Copies directory recursive.
     *
     * @param string $source Path to the source directory.
     * @param string $dest Path to the destination directory.
     * @param int $mode The mode for newly generated directories. If not
     *     specified the default of mkdir() is used.
     */
    public static function copyRecursive($source, $dest, $mode = null)
    {
        foreach ($iterator = new \RecursiveIteratorIterator(
            new \RecursiveDirectoryIterator($source, \RecursiveDirectoryIterator::SKIP_DOTS),
                \RecursiveIteratorIterator::SELF_FIRST) as $item) {
            if ($item->isDir()) {
                $path = $dest . DIRECTORY_SEPARATOR . $iterator->getSubPathName();
                if (!is_dir($path)) {
                    mkdir($path, $mode);
                }
            } else {
                copy($item, $dest . DIRECTORY_SEPARATOR . $iterator->getSubPathName());
            }
        }
    }

    /**
     * Tries to delete the specified file.
     *
     * @param string $filename The file to delete.
     * @return boolean|Exception true if the file does not exist or was
     *     successfully deleted, false if the file exists, but is no regular
     *     file, or the excpetion that was thrown otherwise.
     */
    public static function tryUnlinkFile($filename)
    {
        try {
            if (is_file($filename)) {
                unlink($filename);
                return true;
            }
            return !file_exists($filename);
        } catch (\Exception $e) {
            return $e;
        }
    }

    /**
     * Recursively removes the specified directory.
     *
     * @param string $dirname The directory to remove.
     */
    public static function rmdirRecursive($dirname)
    {
        $it = new \RecursiveIteratorIterator(
            new \RecursiveDirectoryIterator($dirname, \FilesystemIterator::SKIP_DOTS),
            \RecursiveIteratorIterator::CHILD_FIRST);
        foreach ($it as $path) {
            $path->isFile() ? unlink($path->getPathname()) : rmdir($path->getPathname());
        }
        return rmdir($dirname);
    }
}
