<?php
namespace Topalis\Util;

/**
 * This class contains methods for interpolation.
 */
class Interpolator
{
    /**
     * Interpolates the start and end values of the specified array. The
     * specified array must be sorted ascending by key. For the interpolation
     * a value with key less than $start and a value with key greater than $end
     * is required.
     *
     * @param array $array The array to interpolate sorted ascending by key.
     * @param integer $start The minimal key.
     * @param integer $end The maximal key.
     * @return array Interpolated array with $start as minimal key and $end as
     *     maximal key.
     */
    public static function interpolateArray(array $array, $start, $end)
    {
        $rv = array();

        foreach ($array as $key => $value) {
            if ($key < $start) {
                $beforeKey = $key;
                $before = $value;
            } else if ($key > $end) {
                $afterKey = $key;
                $after = $value;
                break;
            } else {
                $rv[$key] = $value;
            }
        }

        if (!isset($rv[$start]) && isset($before) && ($rv || isset($after))) {
            $y = reset($rv);
            if ($y === false) {
                $y = $after;
                $x = $afterKey;
            } else {
                $x = key($rv);
            }
            $rv = array($start => self::interpolate($start, $beforeKey, $before, $x, $y)) + $rv;
        }
        if (!isset($rv[$end]) && isset($after) && ($rv || isset($before))) {
            $y = end($rv);
            if ($y === false) {
                $y = $before;
                $x = $beforeKey;
            } else {
                $x = key($rv);
            }
            $rv[$end] = self::interpolate($end, $x, $y, $afterKey, $after);
        }

        return $rv;
    }

    /**
     * Performs a linear interpolation.
     *
     * @param integer $x The coordinate to get the interpolated value for.
     * @param integer $x1
     * @param integer $y1
     * @param integer $x2
     * @param integer $y2
     * @return integer|NULL The interpolated value, or null if $x2 == $x1.
     */
    public static function interpolate($x, $x1, $y1, $x2, $y2)
    {
        if ($x2 == $x1) {
            return null;
        }

        return $y1 + ($y2 - $y1) * (($x - $x1) / ($x2 - $x1));
    }
}
