<?php
namespace Topalis\Util;

/**
 * This class contains some helper functions for string handling.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class StringUtil
{
    /**
     * Checks if haystack starts with needle.
     * @param string $haystack
     * @param string $needle
     * @return boolean true if haystack starts with needle, false otherwise.
     */
    public static function startsWith($haystack, $needle)
    {
        $length = strlen($needle);
        return (substr($haystack, 0, $length) === $needle);
    }

    /**
     * Checks if haystack ends with needle.
     * @param string $haystack
     * @param string $needle
     * @return boolean true if haystack ends with needle, false otherwise.
     */
    public static function endsWith($haystack, $needle)
    {
        $length = strlen($needle);
        if ($length == 0) {
            return true;
        }

        return (substr($haystack, -$length) === $needle);
    }

    /**
     * Does the same string to boolean conversion as PHP, but also converts "false"
     * (case insensitive) to false. If $strict is true then only "true" (case
     * insensitive) and "1" are converted to true and "false" (case insensitive) and
     * "0" are converted to false. In all other cases null is returned.
     *
     * @param string $string The string to convert.
     * @param boolean $strict Enables strict conversion.
     * @return boolean The boolean.
     */
    public static function strtobool($string, $strict = false)
    {
        if (!$strict) {
            return $string && strtolower($string) != "false";
        }
        $lowerString = strtolower($string);
        if ($string == "1" || $lowerString == "true") {
            return true;
        }
        if ($string == "0" || $lowerString == "false") {
            return false;
        }
        return null;
    }

    /**
     * Converts a boolean to a string.
     *
     * @param boolean $boolean The boolean to convert.
     * @return string "true" if $boolean is true, "false" otherwise.
     */
    public static function booltostr($boolean)
    {
        if ($boolean) {
            return "true";
        }

        return "false";
    }

    /**
     * Converts any given variable to a string using the following rules:
     * <ul>
     * <li>boolean: "true" or "false"</li>
     * <li>scalar: default string casting</li>
     * <li>object with __toString method: __toString</li>
     * <li>other: print_r<li>
     * </ul>
     *
     * @param mixed $data The variable to convert.
     * @return string The converted string.
     */
    public static function toString($data)
    {
        if (is_scalar($data)) {
            if (is_bool($data)) {
                return self::booltostr($data);
            }
            return (string) $data;
        }
        if (is_object($data)) {
            if (method_exists($data, "__toString")) {
                return (string) $data;
            }
        }
        // @codingStandardsIgnoreStart
        return print_r($data, true);
        // @codingStandardsIgnoreEnd
    }
}
