<?php
namespace Topalis\Util;

/**
 * This class contains methods for type checks PHP does not provide itself.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class TypeCheck
{
    /**
     * Checks if the specified variable is an associative array.
     * @param mixed $var The variable to check.
     * @return boolean true if the specified variable is an assiciative array, false
     *     otherwise.
     */
    public static function isAssocArray($var)
    {
        if (!is_array($var)) {
            return false;
        }
        return (bool) count(array_filter(array_keys($var), 'is_string'));
    }

    /**
     * Checks if the specified variable contains a value that can be safely
     * casted to an integer or is already an integer. A cast is safe if there
     * is no information loss. Examples:
     * Returns true for the following values: 3, "3", "3.0", "+3", "-3"
     * Returns false for the following values: "3a", "3.1", any variable that
     * is not of type float, integer, or string.
     *
     * @param mixed $var The varibale to check.
     * @return boolean true if the specified variable can be safely casted to an
     *     integer or is already an integer, false otherwise.
     */
    public static function isInt($var)
    {
        if (is_int($var)) {
            return true;
        }

        $varInt = (int) $var;

        if (is_float($var)) {
            return $varInt == $var;
        }

        if (is_string($var)) {
            if (empty($var)) {
                return false;
            }

            if ($var[0] === "+") {
                // remove leading plus sign
                $var = substr($var, 1);
            }
            return (string) $varInt === $var;
        }

        return false;
    }
}
