<?php
namespace Topalis\Util;

/**
 * This class contains methods for type conversions PHP does not support itself.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class TypeConverter
{
    /**
     * Converts the specified object to an one dimensional associative array with
     * variable names as keys.
     *
     * @param object $object The object to convert.
     * @param boolean $includeNulls false, to omit null values; true by default.
     * @return array The converted object.
     */
    public static function objectToArray($object, $includeNulls = true)
    {
        $values = array();
        foreach (get_object_vars($object) as $k => $v) {
            if (is_array($v) || is_object($v) || $k[0] == '_') {
                // internal or NA field
                continue;
            }
            if ($v === null && !$includeNulls) {
                continue;
            }
            $values[$k] = $v;
        }
        return $values;
    }
}
