<?php
namespace Topalis\Util;

/**
 * This class contains functions to convert units.
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class UnitConverter
{
    /**
     * Format a number (bytes) into a human readable format.
     * @param int $bytes The number of bytes to format.
     * @param int $dp The number of decimal places to show (default: 2).
     * @return string The human readable formatted number.
     */
    public static function humanReadableBytes($bytes, $dp = 2)
    {
        if ($bytes === null) {
            return null;
        }
        if ($bytes == 0) {
            return "0 B";
        }
        $magnitude = array('B', 'KB', 'MB', 'GB', 'TB', 'PB', 'HB');
        $byteSize = $bytes/pow(1024, ($i = floor(log(abs($bytes), 1024))));

        if ($i == 0) {
            // decimal places make no sense for unit byte
            $dp = 0;
        }
        if (is_int($byteSize)) {
            return $byteSize . ' ' . $magnitude[$i];
        } else {
            $pattern = "###";
            if ($dp > 0) {
                $pattern .= "." . str_repeat("#", $dp);
            }
            $nf = new \NumberFormatter(\Locale::getDefault(), \NumberFormatter::PATTERN_DECIMAL, $pattern);
            return $nf->format($byteSize) . ' ' . $magnitude[$i];
        }
    }

}
