if (typeof String.prototype.startsWith != "function") {
  String.prototype.startsWith = function (str){
      return this.lastIndexOf(str, 0) === 0
  };
}

var App = Ember.Application.create({
    Router: null
});
App.Router.reopen({
    location: 'none'
});

Ember.TextField.reopen({
    attributeBindings: ['name']
});

Ember.Checkbox.reopen({
    attributeBindings: ['name']
});

window.tableConfig = {};

jQuery.fn.dataTableExt.afnSortData['dom-data'] = function ( oSettings, iColumn )
{
    var aData = [];
    $( 'td:eq('+iColumn+')', oSettings.oApi._fnGetTrNodes(oSettings) ).each( function () {
        var value = $(this).data('sort-value') || 0;
        aData.push(parseInt(value));
    });
    return aData;
};

function loading() {
    $('#loading').show();

    return function() {
        $('#loading').hide();
    }
}

function openModalForm(url, callback, options) {
    var $el = $('<div></div>').addClass('modal2').modal2(_.extend({url: url, callback: callback}, options));
    $("body").append($el);

    return $el;
}

function openModal(url, callback) {
    var finished = loading();
    $.ajax(url, {
        success: function(data) {
            finished();
            setModalContent(data);
        }
    });

    function setModalContent(data) {
        var $container = $('<div></div>').html(data);
        $modalData = $container.find('>.modal');

        $('body').append($modalData);
        setup($modalData);
        if (callback) {
            callback($modalData);
        }
        $modalData.modal().on('hidden', function() {
            $modalData.remove();
        });
    }
}

function setup($element) {

    // setup tooltips on .tip elements
    $element.find('.tip').tooltip();

    // setup tooltips that don't hide while the tip is hovered
    $element.find(".hover-tip").tooltip({
        trigger: "manual"
    }).on("click", function (e) {
        e.preventDefault();
    }).on("mouseenter", function () {
        var _this = this;
        $(this).tooltip("show");
        $(this).siblings(".tooltip").on("mouseleave", function () {
            $(_this).tooltip('hide');
        });
    }).on("mouseleave", function () {
        var _this = this;
        setTimeout(function () {
            if (!$(".tooltip:hover").length) {
                $(_this).tooltip("hide")
            }
        }, 100);
    });

    // colobox modal on elements with .colorbox
    $element.find('.colorbox').colorbox({
        maxHeight: "80%",
        maxWidth:  "90%"
    });

    // default open modals
    $element.find('.modal.show').modal();

    // configure tables
    $.extend( $.fn.dataTableExt.oStdClasses, {
        "sWrapper": "dataTables_wrapper form-inline",
        "sFilter": "dataTables_filter form-search"
    });
    $element.find('.table:not(.static)').each(function() {
        var table = $(this);
        var id = table.attr('id');
        var config = null;
        if (typeof id !== "undefined") {
            config = tableConfig[id];
        }
        initTable(table, config);
    });
};

function initTable(table, tableConfig) {
    var notSortable = [];
    var index = 0;
    var sortColumn = 0;
    table.find('thead th').each(function(key, column) {
        if ($(column).attr("colspan") > 1) {
            // columns that span multiple columns cannot contain data
            return;
        }
        if (!$(column).hasClass("sortable")) {
            notSortable.push(index);
            // if default sort column is not sortable use next column
            if (sortColumn === index) {
                sortColumn++;
            }
        }
        index++;
    });

    var config = {
        "bPaginate": false,
        "bLengthChange": false,
        "bInfo": false,
        "columnDefs": [
            { "sortable": false, "targets": notSortable }
        ],
        "order": [[ sortColumn, "asc" ]],
        "language": {
            "lengthMenu": i18n['lengthMenu'],
            "search": i18n['search']+":",
            "zeroRecords": i18n['No data available in table'],
            "paginate": {
                "first": i18n['first'],
                "last": i18n['last'],
                "next": i18n['next'],
                "previous": i18n['previous']
            }
        }
    };
    if (table.hasClass('table-paginated')) {
        config.bPaginate = true;
        config.bLengthChange = true;
        config.lengthMenu = [ 25, 50, 75, 100 ];
    }

    if (typeof tableConfig != "undefined") {
        _.extend(config, tableConfig);
    }
    table.dataTable(config);
    var tableContainer = table.parent();
    var classList = table.attr("class").split(/\s+/);
    $.each(classList, function(index, item) {
        if (item.startsWith("table-")) {
            tableContainer.addClass("dataT" + item.substr(1));
        }
    });
    table.data('table', table);
}

jQuery(document).ready(function() {

    setup($(document));

    /**
     *  confirm dialog for .confirm elements
     *  text from data-confirm attribute
     */
    $(document).on('click', 'a.confirm', function(event) {
        event.preventDefault();
        $('.modal').each(function() {
            if (!$(this).hasClass('confirmation')) {
                $(this).modal('hide');
            }
        });
        var element = $(this);
        var text = element.data('confirm');
        $('.confirmation .modal-body').empty();
        if (element.hasClass("confirm-danger")) {
            $('.confirmation .modal-body').append($("<p class=\"alert alert-danger\"/>").html(text));
        } else {
            $('.confirmation .modal-body').append($("<p />").html(text));
        }
        $('.confirmation').modal();
        $('.confirmation #confirm-no').unbind('click');
        $('.confirmation #confirm-no').click(function(e) {
            e.preventDefault();
            $('.confirmation').modal('hide');
        });

        $('.confirmation #confirm-yes').unbind('click');
        $('.confirmation #confirm-yes').click(function(e) {
            event.preventDefault();
            var ajax = element.data('ajax');
            if (true != ajax) {
                window.location.href = element.attr('href');
            } else {
                var token = $('.token').attr('name');
                $.ajax(element.attr('href'), {
                    type: 'POST',
                    data: element.data('confirm-data') + "&" + token + "=42",
                    error: function(xhr, error, errorMsg) {
                        alert("Error: " + errorMsg);
                    },
                    success: function (resp) {
                        window.location.reload();
                    }
                });
            }

        });
    });


    // popovers

    $('[rel=popover]').each(function(){
        var hidden = true;
        $(this).mouseover(function(event) {
            if (!hidden) {
                return;
            }
            hidden = false;
            var element = $(this);
            $(this).popover({
                trigger: 'manual'
            }).popover('show');
            var timeout = null;
            var close = function(event) {
                if (event.target == element.get(0)) {
                    return;
                }
                var $target = $(event.target);
                if ($target.hasClass('popover') || $target.parents('.popover').length > 0) {
                    return;
                }
                if ($target.parents("[rel=popover]").length > 0) {
                    return;
                }
                if (timeout) {
                    clearTimeout(timeout);
                }
                timeout = setTimeout(function() {
                    hidden = true;
                    element.popover('hide');
                    $(document).unbind('mousemove', close);
                }, 50);

            }

            $(document).mousemove(close);

        })

    });

    $('.nav-tabs').each(function() {
        if (!localStorage) {
            return;
        }
        if ($(this).hasClass('no-tab-restore')) {
            return;
        }
        var $nav = $(this);

        //go to the latest tab, if it exists:
        var lastTab = localStorage.getItem('lastTab');
        if (lastTab) {
            $nav.find('a[href="#'+lastTab+'"]').tab('show');
        }

        $nav.find('> li > a[data-toggle="tab"]').on('shown', function(e) {
            localStorage.setItem("lastTab", $(e.target).attr('href').substr(1));
        });

    });

    // siwtch to tab with error
    jQuery(document).ready(function() {
        var tabs = $('form .tabbable');
        tabs.each(function() {
            var tabbable = $(this);
            var found = false;
            tabbable.find('.nav-tabs > li > a').each(function() {
                var tab = $(this);
                var content = $(tab.attr('href'));
                if (content.find('.error').length > 0) {
                    if (!found) {
                        found = true;
                        tab.tab('show');
                    }
                    tab.addClass('error');
                    tab.append($('<span class="badge badge-important badge-danger pull-right"></span>').html(content.find('.error').length));
                }
            });
        });
    });
});

/*
 * formatting methods 
 */
function formatBytes(y, round) {
    if (null === round) {
        round = 2;
    }
    if (!jQuery.isNumeric(y)) {
        return y;
    }
    var abs_y = Math.abs(y);
    if (abs_y >= 1125899906842624)   { return $.localeNumber((y / 1125899906842624), round) + " PB" }
    else if (abs_y >= 1099511627776) { return $.localeNumber((y / 1099511627776), round) + " TB" }
    else if (abs_y >= 1073741824)    { return $.localeNumber((y / 1073741824), round) + " GB" }
    else if (abs_y >= 1048576)       { return $.localeNumber((y / 1048576), round) + " MB" }
    else if (abs_y >= 1024)          { return $.localeNumber((y / 1024), round) + " KB" }
    else                             { return $.localeNumber(y, 0) + " B" }
};
/*
 * formatting methods end
 */

/*
 * methods for DataTables
 */
DataTables = {
    intData: function(row, type, set, meta, name) {
        if (type === "set") {
            // ensure value is integer
            row[name] = set - 0;
        }
        return row[name];
    }
}
/*
 * methods for DataTables end
 */