<?php
namespace Topalis\Profile\Payload\Android;

use Topalis\Profile\Payload\AbstractPayload;
use Topalis\Profile\Annotation\Payload\Key;

/**
 * @Key(name="AllowCamera", type="boolean", default="true")
 * @Key(name="ForceEncryption", type="boolean", default="false")
 * @Key(name="ForceExternalEncryption", type="boolean", default="false")
 * @Key(name="AllowAndroidBeam", type="boolean", default="true")
 * @Key(name="AllowAudioRecord", type="boolean", default="true")
 * @Key(name="AllowBackgroundProcessLimit", type="boolean", default="true")
 * @Key(name="AllowBluetooth", type="boolean", default="true")
 * @Key(name="AllowClipboardShare", type="boolean", default="true")
 * @Key(name="AllowFactoryReset", type="boolean", default="true")
 * @Key(name="AllowGoogleCrashReport", type="boolean", default="true")
 * @Key(name="AllowKillingActivitiesOnLeave", type="boolean", default="true")
 * @Key(name="AllowLockScreenView", type="boolean", default="true")
 * @Key(name="AllowOTAUpgrade", type="boolean", default="true")
 * @Key(name="AllowPowerOff", type="boolean", default="true")
 * @Key(name="AllowSBeam", type="boolean", default="true")
 * @Key(name="AllowSDCardWrite", type="boolean", default="true")
 * @Key(name="AllowSVoice", type="boolean", default="true")
 * @Key(name="AllowSafeMode", type="boolean", default="true")
 * @Key(name="AllowSettingsChanges", type="boolean", default="true")
 * @Key(name="AllowShareList", type="boolean", default="true")
 * @Key(name="AllowStatusBarExpansion", type="boolean", default="true")
 * @Key(name="AllowStopSystemApp", type="boolean", default="true")
 * @Key(name="AllowUsbHostStorage", type="boolean", default="true")
 * @Key(name="AllowUserMobileDataLimit", type="boolean", default="true")
 * @Key(name="AllowVideoRecord", type="boolean", default="true")
 * @Key(name="AllowVpn", type="boolean", default="true")
 * @Key(name="AllowWallpaperChange", type="boolean", default="true")
 * @Key(name="AllowWiFi", type="boolean", default="true")
 * @Key(name="AllowWifiDirect", type="boolean", default="true")
 * @Key(name="AllowNonMarketApps", type="boolean", default="false")
 * @Key(name="AllowBackgroundData", type="boolean", default="true")
 * @Key(name="AllowBackup", type="boolean", default="true")
 * @Key(name="AllowTethering", type="boolean", default="true")
 * @Key(name="AllowBluetoothTethering", type="boolean", default="true")
 * @Key(name="AllowUsbTethering", type="boolean", default="true")
 * @Key(name="AllowWifiTethering", type="boolean", default="true")
 * @Key(name="AllowCellularData", type="boolean", default="true")
 * @Key(name="AllowClipboard", type="boolean", default="true")
 * @Key(name="AllowNFC", type="boolean", default="true")
 * @Key(name="AllowMicrophone", type="boolean", default="true")
 * @Key(name="AllowNonTrustedAppInstall", type="boolean", default="true")
 * @Key(name="AllowScreenCapture", type="boolean", default="true")
 * @Key(name="AllowSdCard", type="boolean", default="true")
 * @Key(name="AllowUsbDebugging", type="boolean", default="true")
 * @Key(name="AllowUsbMediaPlayer", type="boolean", default="true")
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class RestrictionsPayload extends AbstractPayload
{
    const TYPE = "de.auralis.android.restrictions";

    public function setAllowCamera($allowCamera)
    {
        $this->setValue("AllowCamera", $allowCamera, self::TYPE_BOOLEAN);
    }

    public function isAllowCamera()
    {
        return $this->getValue("AllowCamera", true);
    }

    public function setForceEncryption($forceEncryption)
    {
        $this->setValue("ForceEncryption", $forceEncryption, self::TYPE_BOOLEAN);
    }

    public function isForceEncryption()
    {
        return $this->getValue("ForceEncryption", false);
    }

    public function setForceExternalEncryption($forceExternalEncryption)
    {
        $this->setValue("ForceExternalEncryption", $forceExternalEncryption, self::TYPE_BOOLEAN);
    }

    public function isForceExternalEncryption()
    {
        return $this->getValue("ForceExternalEncryption", false);
    }

    public function setAllowAndroidBeam($allowAndroidBeam)
    {
        $this->setValue("AllowAndroidBeam", $allowAndroidBeam, self::TYPE_BOOLEAN);
    }

    public function isAllowAndroidBeam()
    {
        return $this->getValue("AllowAndroidBeam", true);
    }

    public function setAllowAudioRecord($allowAudioRecord)
    {
        $this->setValue("AllowAudioRecord", $allowAudioRecord, self::TYPE_BOOLEAN);
    }

    public function isAllowAudioRecord()
    {
        return $this->getValue("AllowAudioRecord", true);
    }

    public function setAllowBackgroundProcessLimit($allowBackgroundProcessLimit)
    {
        $this->setValue("AllowBackgroundProcessLimit", $allowBackgroundProcessLimit, self::TYPE_BOOLEAN);
    }

    public function isAllowBackgroundProcessLimit()
    {
        return $this->getValue("AllowBackgroundProcessLimit", true);
    }

    public function setAllowBluetooth($allowBluetooth)
    {
        $this->setValue("AllowBluetooth", $allowBluetooth, self::TYPE_BOOLEAN);
    }

    public function isAllowBluetooth()
    {
        return $this->getValue("AllowBluetooth", true);
    }

    public function setAllowClipboardShare($allowClipboardShare)
    {
        $this->setValue("AllowClipboardShare", $allowClipboardShare, self::TYPE_BOOLEAN);
    }

    public function isAllowClipboardShare()
    {
        return $this->getValue("AllowClipboardShare", true);
    }

    public function setAllowFactoryReset($allowFactoryReset)
    {
        $this->setValue("AllowFactoryReset", $allowFactoryReset, self::TYPE_BOOLEAN);
    }

    public function isAllowFactoryReset()
    {
        return $this->getValue("AllowFactoryReset", true);
    }

    public function setAllowGoogleCrashReport($allowGoogleCrashReport)
    {
        $this->setValue("AllowGoogleCrashReport", $allowGoogleCrashReport, self::TYPE_BOOLEAN);
    }

    public function isAllowGoogleCrashReport()
    {
        return $this->getValue("AllowGoogleCrashReport", true);
    }

    public function setAllowKillingActivitiesOnLeave($allowKillingActivitiesOnLeave)
    {
        $this->setValue("AllowKillingActivitiesOnLeave", $allowKillingActivitiesOnLeave, self::TYPE_BOOLEAN);
    }

    public function isAllowKillingActivitiesOnLeave()
    {
        return $this->getValue("AllowKillingActivitiesOnLeave", true);
    }

    public function setAllowLockScreenView($allowLockScreenView)
    {
        $this->setValue("AllowLockScreenView", $allowLockScreenView, self::TYPE_BOOLEAN);
    }

    public function isAllowLockScreenView()
    {
        return $this->getValue("AllowLockScreenView", true);
    }

    public function setAllowOTAUpgrade($allowOTAUpgrade)
    {
        $this->setValue("AllowOTAUpgrade", $allowOTAUpgrade, self::TYPE_BOOLEAN);
    }

    public function isAllowOTAUpgrade()
    {
        return $this->getValue("AllowOTAUpgrade", true);
    }

    public function setAllowPowerOff($allowPowerOff)
    {
        $this->setValue("AllowPowerOff", $allowPowerOff, self::TYPE_BOOLEAN);
    }

    public function isAllowPowerOff()
    {
        return $this->getValue("AllowPowerOff", true);
    }

    public function setAllowSBeam($allowSBeam)
    {
        $this->setValue("AllowSBeam", $allowSBeam, self::TYPE_BOOLEAN);
    }

    public function isAllowSBeam()
    {
        return $this->getValue("AllowSBeam", true);
    }

    public function setAllowSDCardWrite($allowSDCardWrite)
    {
        $this->setValue("AllowSDCardWrite", $allowSDCardWrite, self::TYPE_BOOLEAN);
    }

    public function isAllowSDCardWrite()
    {
        return $this->getValue("AllowSDCardWrite", true);
    }

    public function setAllowSVoice($allowSVoice)
    {
        $this->setValue("AllowSVoice", $allowSVoice, self::TYPE_BOOLEAN);
    }

    public function isAllowSVoice()
    {
        return $this->getValue("AllowSVoice", true);
    }

    public function setAllowSafeMode($allowSafeMode)
    {
        $this->setValue("AllowSafeMode", $allowSafeMode, self::TYPE_BOOLEAN);
    }

    public function isAllowSafeMode()
    {
        return $this->getValue("AllowSafeMode", true);
    }

    public function setAllowSettingsChanges($allowSettingsChanges)
    {
        $this->setValue("AllowSettingsChanges", $allowSettingsChanges, self::TYPE_BOOLEAN);
    }

    public function isAllowSettingsChanges()
    {
        return $this->getValue("AllowSettingsChanges", true);
    }

    public function setAllowShareList($allowShareList)
    {
        $this->setValue("AllowShareList", $allowShareList, self::TYPE_BOOLEAN);
    }

    public function isAllowShareList()
    {
        return $this->getValue("AllowShareList", true);
    }

    public function setAllowStatusBarExpansion($allowStatusBarExpansion)
    {
        $this->setValue("AllowStatusBarExpansion", $allowStatusBarExpansion, self::TYPE_BOOLEAN);
    }

    public function isAllowStatusBarExpansion()
    {
        return $this->getValue("AllowStatusBarExpansion", true);
    }

    public function setAllowStopSystemApp($allowStopSystemApp)
    {
        $this->setValue("AllowStopSystemApp", $allowStopSystemApp, self::TYPE_BOOLEAN);
    }

    public function isAllowStopSystemApp()
    {
        return $this->getValue("AllowStopSystemApp", true);
    }

    public function setAllowUsbHostStorage($allowUsbHostStorage)
    {
        $this->setValue("AllowUsbHostStorage", $allowUsbHostStorage, self::TYPE_BOOLEAN);
    }

    public function isAllowUsbHostStorage()
    {
        return $this->getValue("AllowUsbHostStorage", true);
    }

    public function setAllowUserMobileDataLimit($allowUserMobileDataLimit)
    {
        $this->setValue("AllowUserMobileDataLimit", $allowUserMobileDataLimit, self::TYPE_BOOLEAN);
    }

    public function isAllowUserMobileDataLimit()
    {
        return $this->getValue("AllowUserMobileDataLimit", true);
    }

    public function setAllowVideoRecord($allowVideoRecord)
    {
        $this->setValue("AllowVideoRecord", $allowVideoRecord, self::TYPE_BOOLEAN);
    }

    public function isAllowVideoRecord()
    {
        return $this->getValue("AllowVideoRecord", true);
    }

    public function setAllowVpn($allowVpn)
    {
        $this->setValue("AllowVpn", $allowVpn, self::TYPE_BOOLEAN);
    }

    public function isAllowVpn()
    {
        return $this->getValue("AllowVpn", true);
    }

    public function setAllowWallpaperChange($allowWallpaperChange)
    {
        $this->setValue("AllowWallpaperChange", $allowWallpaperChange, self::TYPE_BOOLEAN);
    }

    public function isAllowWallpaperChange()
    {
        return $this->getValue("AllowWallpaperChange", true);
    }

    public function setAllowWiFi($allowWiFi)
    {
        $this->setValue("AllowWiFi", $allowWiFi, self::TYPE_BOOLEAN);
    }

    public function isAllowWiFi()
    {
        return $this->getValue("AllowWiFi", true);
    }

    public function setAllowWifiDirect($allowWifiDirect)
    {
        $this->setValue("AllowWifiDirect", $allowWifiDirect, self::TYPE_BOOLEAN);
    }

    public function isAllowWifiDirect()
    {
        return $this->getValue("AllowWifiDirect", true);
    }

    public function setAllowNonMarketApps($allowNonMarketApps)
    {
        $this->setValue("AllowNonMarketApps", $allowNonMarketApps, self::TYPE_BOOLEAN);
    }

    public function isAllowNonMarketApps()
    {
        return $this->getValue("AllowNonMarketApps", false);
    }

    public function setAllowBackgroundData($allowBackgroundData)
    {
        $this->setValue("AllowBackgroundData", $allowBackgroundData, self::TYPE_BOOLEAN);
    }

    public function isAllowBackgroundData()
    {
        return $this->getValue("AllowBackgroundData", true);
    }

    public function setAllowBackup($allowBackup)
    {
        $this->setValue("AllowBackup", $allowBackup, self::TYPE_BOOLEAN);
    }

    public function isAllowBackup()
    {
        return $this->getValue("AllowBackup", true);
    }

    public function setAllowTethering($allowTethering)
    {
        $this->setValue("AllowTethering", $allowTethering, self::TYPE_BOOLEAN);
    }

    public function isAllowTethering()
    {
        return $this->getValue("AllowTethering", true);
    }

    public function setAllowBluetoothTethering($allowBluetoothTethering)
    {
        $this->setValue("AllowBluetoothTethering", $allowBluetoothTethering, self::TYPE_BOOLEAN);
    }

    public function isAllowBluetoothTethering()
    {
        return $this->getValue("AllowBluetoothTethering", true);
    }

    public function setAllowUsbTethering($allowUsbTethering)
    {
        $this->setValue("AllowUsbTethering", $allowUsbTethering, self::TYPE_BOOLEAN);
    }

    public function isAllowUsbTethering()
    {
        return $this->getValue("AllowUsbTethering", true);
    }

    public function setAllowWifiTethering($allowWifiTethering)
    {
        $this->setValue("AllowWifiTethering", $allowWifiTethering, self::TYPE_BOOLEAN);
    }

    public function isAllowWifiTethering()
    {
        return $this->getValue("AllowWifiTethering", true);
    }

    public function setAllowCellularData($allowCellularData)
    {
        $this->setValue("AllowCellularData", $allowCellularData, self::TYPE_BOOLEAN);
    }

    public function isAllowCellularData()
    {
        return $this->getValue("AllowCellularData", true);
    }

    public function setAllowClipboard($allowClipboard)
    {
        $this->setValue("AllowClipboard", $allowClipboard, self::TYPE_BOOLEAN);
    }

    public function isAllowClipboard()
    {
        return $this->getValue("AllowClipboard", true);
    }

    public function setAllowNFC($allowNFC)
    {
        $this->setValue("AllowNFC", $allowNFC, self::TYPE_BOOLEAN);
    }

    public function isAllowNFC()
    {
        return $this->getValue("AllowNFC", true);
    }

    public function setAllowMicrophone($allowMicrophone)
    {
        $this->setValue("AllowMicrophone", $allowMicrophone, self::TYPE_BOOLEAN);
    }

    public function isAllowMicrophone()
    {
        return $this->getValue("AllowMicrophone", true);
    }

    public function setAllowNonTrustedAppInstall($allowNonTrustedAppInstall)
    {
        $this->setValue("AllowNonTrustedAppInstall", $allowNonTrustedAppInstall, self::TYPE_BOOLEAN);
    }

    public function isAllowNonTrustedAppInstall()
    {
        return $this->getValue("AllowNonTrustedAppInstall", true);
    }

    public function setAllowScreenCapture($allowScreenCapture)
    {
        $this->setValue("AllowScreenCapture", $allowScreenCapture, self::TYPE_BOOLEAN);
    }

    public function isAllowScreenCapture()
    {
        return $this->getValue("AllowScreenCapture", true);
    }

    public function setAllowSdCard($allowSdCard)
    {
        $this->setValue("AllowSdCard", $allowSdCard, self::TYPE_BOOLEAN);
    }

    public function isAllowSdCard()
    {
        return $this->getValue("AllowSdCard", true);
    }

    public function setAllowUsbDebugging($allowUsbDebugging)
    {
        $this->setValue("AllowUsbDebugging", $allowUsbDebugging, self::TYPE_BOOLEAN);
    }

    public function isAllowUsbDebugging()
    {
        return $this->getValue("AllowUsbDebugging", true);
    }

    public function setAllowUsbMediaPlayer($allowUsbMediaPlayer)
    {
        $this->setValue("AllowUsbMediaPlayer", $allowUsbMediaPlayer, self::TYPE_BOOLEAN);
    }

    public function isAllowUsbMediaPlayer()
    {
        return $this->getValue("AllowUsbMediaPlayer", true);
    }
}
