<?php
namespace Topalis\Profile\Payload;

// @codingStandardsIgnoreStart
/**
 * Represents a Cellular payload.
 * This payload configures cellular network settings on the device.
 * @see https://developer.apple.com/library/ios/featuredarticles/iPhoneConfigurationProfileRef/iPhoneConfigurationProfileRef.pdf#//apple_ref/doc/uid/TP40010206-CH1-SW48
 * @since iOS 7.0
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
// @codingStandardsIgnoreEnd
class CellularPayload extends AbstractPayload
{
    const TYPE = "com.apple.cellular";

    public function getAttachApn()
    {
        return $this->getValue("AttachAPN");
    }

    public function setAttachApn(array $attachApn = null)
    {
        $this->setValue("AttachAPN", $attachApn, self::TYPE_DICTIONARY);
    }

    public function getApns()
    {
        return $this->getValue("APNS");
    }

    public function setApns(array $apns = null)
    {
        $this->setValue("APNS", $apns, self::TYPE_ARRAY);
    }

    public function getName()
    {
        return $this->getValue("Name");
    }
}
