<?php
namespace Topalis\Profile\Payload;

use CFPropertyList\CFNumber;
use CFPropertyList\CFBoolean;
use CFPropertyList\CFString;

/**
 * This class represents a MDM payload.
 * @see Mobile Device Management Protocol Reference
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class MdmPayload extends AbstractPayload
{
    const TYPE = "com.apple.mdm";

    public function getIdentityCertificateUUID()
    {
        return $this->getValue("IdentityCertificateUUID");
    }

    public function setIdentityCertificateUUID($identityCertificateUUID)
    {
        $this->set("IdentityCertificateUUID", new CFString($identityCertificateUUID));
    }

    public function getServerUrl()
    {
        return $this->getValue("ServerURL");
    }

    public function setServerUrl($serverUrl)
    {
        $this->set("ServerURL", new CFString($serverUrl));
    }

    public function getTopic()
    {
        return $this->getValue("Topic");
    }

    public function setTopic($topic)
    {
        $this->set("Topic", new CFString($topic));
    }

    public function isUseDevelopmentAPNS()
    {
        return $this->getValue("UseDevelopmentAPNS", false);
    }

    public function setUseDevelopmentAPNS($useDevelopmentAPNS)
    {
        if ($useDevelopmentAPNS === null) {
            $this->del("UseDevelopmentAPNS");
        } else {
            $this->set("UseDevelopmentAPNS", new CFBoolean($useDevelopmentAPNS));
        }
    }

    public function getAccessRights()
    {
        return $this->getValue("AccessRights");
    }

    public function setAccessRights($accessRights)
    {
        $this->set("AccessRights", new CFNumber($accessRights));
    }

    public function isCheckOutWhenRemoved()
    {
        return $this->getValue("CheckOutWhenRemoved", false);
    }

    public function setCheckOutWhenRemoved($checkOutWhenRemoved)
    {
        if ($checkOutWhenRemoved === null) {
            $this->del("CheckOutWhenRemoved");
        } else {
            $this->set("CheckOutWhenRemoved", new CFBoolean($checkOutWhenRemoved));
        }
    }
}
