<?php
namespace Topalis\Profile\Payload;

// @codingStandardsIgnoreStart
/**
 * This class represents a Passcode Policy payload.
 * A Passcode Policy payload is used to define a passcode policy for the device.
 * @see https://developer.apple.com/library/ios/featuredarticles/iPhoneConfigurationProfileRef/iPhoneConfigurationProfileRef.pdf#//apple_ref/doc/uid/TP40010206-CH1-SW9
 *
 * @see    https://developer.apple.com/library/ios/featuredarticles/iPhoneConfigurationProfileRef/iPhoneConfigurationProfileRef.pdf#//apple_ref/doc/uid/TP40010206-CH1-SW9
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
// @codingStandardsIgnoreEnd
class PasscodePayload extends AbstractPayload
{
    const TYPE = "com.apple.mobiledevice.passwordpolicy";

    public function setAllowSimple($allowSimple)
    {
        $this->setValue("allowSimple", $allowSimple, self::TYPE_BOOLEAN);
    }

    public function isAllowSimple()
    {
        return $this->getValue("allowSimple");
    }

    public function setRequireAlphanumeric($requireAlphanumeric)
    {
        $this->setValue("requireAlphanumeric", $requireAlphanumeric, self::TYPE_BOOLEAN);
    }

    public function isRequireAlphanumeric()
    {
        return $this->getValue("requireAlphanumeric");
    }

    public function setMinLength($minLength)
    {
        $this->setValue("minLength", $minLength, self::TYPE_NUMBER);
    }

    public function getMinLength()
    {
        return $this->getValue("minLength");
    }

    public function setMinComplexChars($minComplexChars)
    {
        $this->setValue("minComplexChars", $minComplexChars, self::TYPE_NUMBER);
    }

    public function getMinComplexChars()
    {
        return $this->getValue("minComplexChars");
    }

    public function setMaxPINAgeInDays($maxPINAgeInDays)
    {
        $this->setValue("maxPINAgeInDays", $maxPINAgeInDays, self::TYPE_NUMBER);
    }

    public function getMaxPINAgeInDays()
    {
        return $this->getValue("maxPINAgeInDays");
    }

    public function setMaxInactivity($maxInactivity)
    {
        $this->setValue("maxInactivity", $maxInactivity, self::TYPE_NUMBER);
    }

    public function getMaxInactivity()
    {
        return $this->getValue("maxInactivity");
    }

    public function setPinHistory($pinHistory)
    {
        $this->setValue("pinHistory", $pinHistory, self::TYPE_NUMBER);
    }

    public function getPinHistory()
    {
        return $this->getValue("pinHistory");
    }

    public function setMaxGracePeriod($maxGracePeriod)
    {
        $this->setValue("maxGracePeriod", $maxGracePeriod, self::TYPE_NUMBER);
    }

    public function getMaxGracePeriod()
    {
        return $this->getValue("maxGracePeriod");
    }

    public function setMaxFailedAttempts($maxFailedAttempts)
    {
        $this->setValue("maxFailedAttempts", $maxFailedAttempts, self::TYPE_NUMBER);
    }

    public function getMaxFailedAttempts()
    {
        return $this->getValue("maxFailedAttempts");
    }
}
