<?php
namespace Topalis\Profile\Payload;

// @codingStandardsIgnoreStart
/**
 * Represents a Per-App VPN payload.
 * This payload configures add-on VPN software on the device.
 * @see https://developer.apple.com/library/ios/featuredarticles/iPhoneConfigurationProfileRef/iPhoneConfigurationProfileRef.pdf#//apple_ref/doc/uid/TP40010206-CH1-SW37
 * @since iOS 7.0
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
// @codingStandardsIgnoreEnd
class PerAppVpnPayload extends VpnPayload
{
    const TYPE = "com.apple.vpn.managed.applayer";

    public function getVpnUuid()
    {
        return $this->getValue("VPNUUID");
    }

    public function setVpnUuid($vpnUuid)
    {
        $this->setValue("VPNUUID", $vpnUuid, self::TYPE_STRING);
    }

    public function getSafariDomains()
    {
        return $this->get("SafariDomains");
    }

    public function setSafariDomains(array $safariDomains)
    {
        $this->setValue("SafariDomains", $safariDomains, self::TYPE_ARRAY);
    }

    public function isOnDemandMatchAppEnabled()
    {
        return $this->getValue("OnDemandMatchAppEnabled");
    }

    public function setOnDemandMatchAppEnabled($enabled)
    {
        $this->setValue("OnDemandMatchAppEnabled", $enabled, self::TYPE_BOOLEAN);
    }
}
