<?php
namespace Topalis\Profile\Payload;

use CFPropertyList\CFData;

/**
 * Payload for a CA or server certificate.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class RootCertificatePayload extends AbstractPayload
{
    const TYPE = "com.apple.security.root";

    /**
     * @return string The certificate file name.
     */
    public function getCertificateFileName()
    {
        return $this->getValue("PayloadCertificateFileName");
    }

    /**
     * Sets the certificate file name.
     *
     * @param string $certificateFileName The certificate file name to set.
     */
    public function setCertificateFileName($certificateFileName)
    {
        $this->setValue("PayloadCertificateFileName", $certificateFileName, self::TYPE_STRING);
    }

    /**
     * @return string The DER encoded certificate.
     */
    public function getCertificate()
    {
        return $this->getValue("PayloadContent");
    }

    /**
     * Sets the certificate.
     *
     * @param string $certificate The DER encoded certificate to set.
     */
    public function setCertificate($certificate)
    {
        $this->set("PayloadContent", new CFData($certificate));
    }
}
