<?php
namespace Topalis\Profile\Payload;

// @codingStandardsIgnoreStart

    /**
 * This class represents a Web Content Filter payload.
 * A Web Content Filter payload allows to whitelist and blacklist specific web
 * URLs.
 * @see https://developer.apple.com/library/ios/featuredarticles/iPhoneConfigurationProfileRef/iPhoneConfigurationProfileRef.pdf#//apple_ref/doc/uid/TP40010206-CH1-SW45
 * @since iOS 7.0
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
// @codingStandardsIgnoreEnd
class WebContentFilterPayload extends AbstractPayload
{
    const TYPE = "com.apple.webcontent-filter";

    public function isAutoFilterEnabled()
    {
        return $this->getValue("AutoFilterEnabled", false);
    }

    public function setAutoFilterEnabled($autoFilterEnabled)
    {
        $this->setValue("AutoFilterEnabled", $autoFilterEnabled, self::TYPE_BOOLEAN);
    }

    public function getPermittedUrls()
    {
        return $this->getValue("PermittedURLs");
    }

    public function setPermittedUrls(array $permittedUrls = null)
    {
        $this->setValue("PermittedURLs", $permittedUrls, self::TYPE_ARRAY);
    }

    public function getWhitelistedBookmarks()
    {
        return $this->getValue("WhitelistedBookmarks");
    }

    public function setWhitelistedBookmarks(array $whitelistedBookmarks = null)
    {
        $this->setValue("WhitelistedBookmarks", $whitelistedBookmarks, self::TYPE_ARRAY);
    }

    public function getBlacklistedUrls()
    {
        return $this->getValue("BlacklistedURLs");
    }

    public function setBlacklistedUrls(array $blacklistedUrls = null)
    {
        $this->setValue("BlacklistedURLs", $blacklistedUrls, self::TYPE_ARRAY);
    }
}
